// Simulovany FDD ovladac - pres BIOS (INT 13H). Po dobu behu BIOS funkci
// je system samozrejme blokovan (vcetne doby SEEKu atd).

// Predpoklada, ze se do puvodniho ovladace propousti take INT 08H
// (potazmo INT 1CH) a INT 1EH, tj. Diskette Parameter Table (chce ji INT 13H)

// Pro projekt TUOX ze zoufalstvi napsal Petr Grygarek

#define REPORT_ERRORS
/* #define REPORT_ACTIVITY */

#define REPORT_LINE1 20
#define REPORT_ATTR (15)

#if defined(REPORT_ERRORS) || defined(REPORT_ACTIVITY)
#include "test\kprintf.h"
#endif

#if defined(REPORT_ACTIVITY)
long blocksRead=0L,blocksWritten=0L;
#endif

#include "test.cnf"
#include "test\trap.h"

#include "fs\ioctl.h"
#include "kernel\makra.h"

#include "COMMON/com.h"		/* vyuziva se FLOPPY, CLOCK, HARDWARE, SET_ALARM, DISKINT */
#include "COMMON/const.h"	/* vyuziva se PRIVATE, PUBLIC, OK, TRUE, FALSE, HZ, D */
#include "COMMON/type.h"	/* vyuziva se Byte, DWord, vir_bytes, message... */
#include "COMMON/errno.h"	/* vyuziva se EIO, EACCES, EINVAL, ENXIO */

#include "systask\systask.h"

#define NR_DRIVES 2

#define DRV_0_CYLS 80
#define DRV_0_SECS 18

#define DRV_1_CYLS 40
#define DRV_1_SECS 9

static Byte buf[BLOCK_SIZE];

unsigned int composeCylSec(unsigned int cyl, unsigned int sec) {
 // slozi cyl+sec do formatu pro BIOS
 return  ((cyl & 0xff) << 8) | ((cyl & 0x0300) >> 2) | (sec & 0x3f);
}


#define WRITE 1
#define READ  0
int transferBlock(long block, int func, Byte drive, int totalCyls,
int secsPerCyl /* sectors/track */ ) {

 // Vraci: 1, kdyz OK, jinak zaporne cislo chyby z BIOSu

 unsigned int fn,cyl,sec,cylsec,bufptr,s;
 Byte result,hd;
 static int retry;

 fn = ((func==WRITE) ? 0x0301 : 0x0201);
 bufptr=(unsigned int) &buf;
 // razeni bloku: vzdy 2 sektory, postupne pres strany po cylindrech
 // pro 2 strany:
 cyl=block/secsPerCyl;
 if (block%secsPerCyl>=(secsPerCyl/2))
  { hd=1; sec=((block%secsPerCyl)-(secsPerCyl/2))*2+1; }
  else { hd=0; sec=(block%secsPerCyl)*2+1; }

/*
 // blok je pres 2 sektory. Pojmu jej jako cislo sektoru na jedne strane,
 // druhou pulku dam na druhou stranu.

 cyl=block/secsPerCyl;
 if (cyl>=totalCyls) return EINVAL;
 sec=(block%secsPerCyl)+1;
*/

 #define MAXRETRY 8
 result=0;
 retry=0;

 for (s=sec; s<=sec+1; s++) {
	       again:
		cylsec=composeCylSec(cyl,s);
		asm mov dl,drive /* drive # */
		asm mov dh,hd
		asm mov cx,cylsec
		asm mov ax,fn
		asm push ds
		asm pop es
		asm mov bx, bufptr
		asm int 0x13
		asm jnc no_err

		asm inc [retry]
		asm cmp [retry],MAXRETRY
		asm jbe again
		asm mov result,ah

	       no_err:
		bufptr+=512;
 if (result!=0) break;
 }

 if (result==0) {
  #ifdef REPORT_ACTIVITY
   if (func==WRITE)
    kprintf(REPORT_LINE1+2,0,REPORT_ATTR,"W: %l",++blocksWritten);
    else
     kprintf(REPORT_LINE1+1,0,REPORT_ATTR,"R: %l",++blocksRead);
  #endif
  return 1;
 }
  else {
    #ifdef REPORT_ERRORS
     kprintf(REPORT_LINE1,0,REPORT_ATTR,"BIOSFLOP ERROR: block %l, errcode %u, retries: %u",block, result, (int) retry);
    #endif
    return ((int) result)*(-1);
  }
}

void init_fdd(void) {
 asm mov ah,0 /* reset controller */
 asm mov dl,0 /* FDD controller */
 asm int 0x13
}

/*============================================================================*
 *                              floppy_task                                   *
 *============================================================================*/

PUBLIC void floppy_task(void)
{
    static message msg;
    static int  nrMinors;
    static long nrBlocks;
    procnr_t proc_nr;
    Byte d;
    int i;

    init_fdd();

    while(TRUE)
	{
	receive(ANY,msg);
	proc_nr=msg.m_source;
	switch (msg.m_type)
	    {
	    case DEV_IOCTL:
		d=(Byte) msg.m_u.m_dio.device;
		if (d > NR_DRIVES -1 ) msg.REP_STATUS=ENXIO;	/* no such device or address */
		 else msg.REP_STATUS=OK;
		switch (msg.m_u.m_dio.count) {
		    case GETDEVSIZE:
			// pro zacatek natvrdo
			switch(d) {
			 case 0: nrBlocks=(80*18*2)/2; break; //1.44 MB
			 case 1: nrBlocks=(40*9*2)/2; break; // 360 kB
			}

			i=sys_copy(FLOPPY,D,(vir_bytes)&nrBlocks,
				   msg.m_source,D,msg.m_u.m_dio.address,
				   sizeof(nrBlocks));
			if (i!=OK) msg.REP_STATUS=i;
			break;

		    case GETNUMBERMINORDEV:
			nrMinors = 2;
			// pro zacatek taky natvrdo
			i=sys_copy(FLOPPY,D,(vir_bytes)&nrMinors,
				   msg.m_source,D,msg.m_u.m_dio.address,
				   sizeof(nrMinors));
			if (i!=OK) msg.REP_STATUS=i;
			break;
		    default:
			msg.REP_STATUS=EINVAL;	/*Invalid argument */
			break;
		}
		break;

	    case DEV_WRITE:

		if (msg.m_u.m_dio.count!=1) { msg.REP_STATUS=EINVAL; break; }

		// prevezmi buffer z uziv. procesu do buf
		i=sys_copy(msg.m_u.m_dio.proc_nr,D,msg.m_u.m_dio.address,
			   FLOPPY,D,(vir_bytes)&buf,BLOCK_SIZE);

		if (i!=OK) { msg.REP_STATUS=i; break; }

		msg.REP_STATUS=transferBlock(msg.m_u.m_dio.position,WRITE,msg.m_u.m_dio.device,
		 (msg.m_u.m_dio.device==0) ? DRV_0_CYLS : DRV_1_CYLS,
		 (msg.m_u.m_dio.device==0) ? DRV_0_SECS : DRV_1_SECS);

		break;

	    case DEV_READ:
		if (msg.m_u.m_dio.count!=1) { msg.REP_STATUS=EINVAL; break; }

		msg.REP_STATUS=transferBlock(msg.m_u.m_dio.position,READ,msg.m_u.m_dio.device,
		 (msg.m_u.m_dio.device==0) ? DRV_0_CYLS : DRV_1_CYLS,
		 (msg.m_u.m_dio.device==0) ? DRV_0_SECS : DRV_1_SECS);


		if (msg.REP_STATUS==1) {
		 i=sys_copy(FLOPPY,D,(vir_bytes)&buf,
			    msg.m_u.m_dio.proc_nr,D,msg.m_u.m_dio.address,
			    BLOCK_SIZE);
		 if (i!=OK) msg.REP_STATUS=i;
		 }
		break;

	    default:
		msg.REP_STATUS=EINVAL;	/*Invalid argument */
		trap (TRAP_UNKNOWN_MSG);
		break;
	    }
	msg.m_source=FLOPPY;
	msg.m_type=TASK_REPLY;
	send(proc_nr,msg);
	}
}

