/************************************************
*	FLOPPY TASK				*
*		Wrtten by:      		*
*			Jiri Kubica		*
*			Libor Cudek	L93360	*
*			Petr Grygarek		*
*	Sestava:				*
*		floppy.h     header		*
*		floppy.c     main		*
*		floppy_s.h   header struct	*
*		floppy_d.h   header def		*
*						*
*************************************************/

#include "floppy\floppy.h"
#include "kernel\klib88.h"
#include "test.cnf"
#include "test\kprintf.h"
#include "kernel\proc_pub.h"

/* Driver for the Floppy Disk Controller (FDC) using the NEC PD765 chip.
 *
 * The file contains these entry points:
 *
 *   floppy_task:       main entry when system is brought up
 *   floppy_stop:       stop all activity
 */

 /*
  P.G.: AT FDC umi jen 2 mechaniky (zato umi vice kapacit)

 */

/****************************************************************************
 *                           request_motor                                 *
 ***************************************************************************/
PRIVATE void request_motor()
{
  /* Nastavi timeout pro vypnuti motoru. Po vyprseni se vypnou motory
     vsech mechanik s floppy[minorNo].io_timeout=1  */
 /* Pitfall: motory vsech mechanik se toci spolecne, tedy timeout pro
    vsechny mechaniky se zacne odpocitavat znovu po skonceni operace
    s libovolnou mechanikou */

  static message m;

  m.m_source=FLOPPY;
  m.m_type=SET_ALARM;
  m.FUNC=(sighandler_t)do_timeout;
  m.NEW_TIME=MOTOR_OFF;
  m.CLOCK_PROC=FLOPPY;
  timeout_enable=MOTOR_TIMEOUT; // P.G. presunuto pred send_recv
  send_recv(CLOCK,m);	/* pozadavek na timeout -> vypne se motor */
}

/****************************************************************************
 *                              do_timeout                                 *
 ***************************************************************************/
/* Vyvolavano Clock taskem jako watch_dog funkce, kdyz vyprsi timeout */
/* Planovano v request_motor() a f_wait() */

PRIVATE void do_timeout(void)
{
  static message m;
  int i;

  switch (timeout_enable)	/* Jaka cinnost vyvolala timeout */
    {
    case FLOPPY_TIMEOUT:
	m.m_source=HARDWARE;
	m.m_type=ERR_TIMEOUT;
//	send(FLOPPY,m);	 /* Posle se zprava sama sobe o neuspechu operace*/
//P.G.: sice posle, ale jmenem HARDWARE, protoze FDD driver ceka na zpravu
//	v receive(HARDWARE), takze takto by to neprevzal. Proto simulujeme:
	k_interrupt(FLOPPY,&m);
	break;

    case MOTOR_TIMEOUT:
	for ( i = 0; i < NR_DRIVES; i++ )
	    if (floppy[i].io_timeout){
		stop_motor(i);		/* Zastavi se motor u mechanik */
		floppy[i].io_timeout=0; /* s oznacenim io_timeout=1 */
	    }
	break;
    }

  timeout_enable=NONE_TIMEOUT;
}

/*===========================================================================*
 *                              floppy_init                                  *
 *===========================================================================*/
PRIVATE void floppy_init(void)
{
  /* Initialize floppy structure */

  struct FDD *fp;
  Byte dm,aux;
  int i;
  Word m;

  /* Zjisteni hustot jednotlivych instalovanych floppy mechanik z CMOS */

  out_byte(CMOS_ADR_PORT,CMOS_DISK_DATA);
  dm = in_byte(CMOS_DATA_PORT);

/*
 Horni slabika: 1. mechanika, spodni slabika: 2. mechanika
 0=no drive, 1=360K, ...
*/

  for (fp = &floppy[0], i=1; i >= 0; fp++, i--) {
	if ((aux = dm >> (i*DISK_DATA_SHIFT) & 0xF) == 0) aux = NO_DENS;
	  else aux--;
	fp->pcn = NO_CYL; // doposud nekalibrovano
	fp->r = NO_SECTOR;
	fp->density = aux; // P.G.: cislo z CMOS zmensene o 1.
      }

  // enable IRQ6

  m = in_byte(IC_PORT);
  m &= (~0x40);
  out_byte(IC_PORT,m);

  f_reset();

}

/*===========================================================================*
 *                              f_reset                                      *
 *===========================================================================*/
PRIVATE void f_reset(void)
{
// P.G.: pozn.: volano z inicializace a f_finish()

/* Issue a reset to the controller.  This is done after any catastrophe,
 * like the controller refusing to respond.
 */

  int i;
  /* Disable interrupts and strobe reset bit low. */
  I8272.need_reset = FALSE;

  disable();
  /* Digital Output Register vlastne neni soucasti 8272: jde patrne o normalni
     registr, jehoz jednotlive bity ridi zapnuti motoru, hradluji DRQ (zda se,
     ze ve skutecnosti DACK) a INT od radice FDD a umoznuji resetovat 8272
      - reset 8272 vede na tento registr */

  out_byte(DOR, 0);             /* strobe reset bit low */
   // P.G: tohle taky vypne motory
  out_byte(DOR,I8272.dor = ENABLE_INT);  /* strobe it high again */

  enable();
  f_intr_wait();
  /* K tomuto cekani na interrupt a enable()-disable():
     PC snad po zapisu 0 do DOR generuje INT (nevi se proc, bit urcujici
     povoleni IRQ a DMA je znulovan. Proto je treba tento INTR znicit. */

  /* The controller supports 4 drives and returns a result for each of them.
   * Collect all the results now.  The old version only collected the first
   * result.  This happens to work for 2 drives, but it doesn't work for 3
   * or more drives, at least with only drives 0 and 2 actually connected
   * (the controller generates an extra interrupt for the middle drive when
   * drive 2 is accessed and the driver panics).
   *
   * It would be better to keep collecting results until there are no more.
   * For this, fdc_results needs to return the number of results (instead
   * of OK) when it succeeds.
   */

  for (i = 0; i < 4; i++) { // P.G: Asi je nanic delat to 4*
	fdc_out(FDC_SENSE_INT);     /* probe FDC to make it return status */
	(void) fdc_results();   /* flush controller */
  }
  for (i = 0; i < NR_DRIVES; i++)       /* clear each drive */
	floppy[i].calibration = UNCALIBRATED;

  f_specify();

}

/*****************************************************************************
 *                            f_specify                                      *
 *****************************************************************************/
PRIVATE void f_specify(void)
{
 /* Executes FDC SPECIFY command */
  fdc_out(FDC_SPECIFY);		/* command the drive to accept params */
  fdc_out(SPEC1);		/* step rate time and head unload time */
  fdc_out(SPEC2);               /* second parameter to SPECIFY */
}


/****************************************************************************
 *                                f_open                                    *
 ****************************************************************************/
PRIVATE Byte f_open(Byte nr_drive)	/* Open a floppy */
{
  Byte dens,r;
  int cl;
  /* Determine diskette type */

  if (nr_drive > NR_DRIVES - 1) return BAD_DRIVE_NUMBER;

  /* Experimentally determine drive/medium:
   * First check fl_density.  If it is not NO_DENS, the drive has been used
   * before and the value of fl_density tells what was found last time. Try
   * that first.
   */
  if (floppy[nr_drive].density == NO_DENS) return (DRIVE_NOT_INSTALLED);

  /* Either drive type is unknown or a different diskette is now present.
   * Use test_order to try them. */

  /* Skip densities that have been proven to be impossible */
  dens = floppy[nr_drive].density;

  for(cl = NT - 1; cl >= 0; cl--) {
     if (!(test_density[dens] & (1 << cl)))
	continue;

     switch (r = test_read(nr_drive,(Byte) cl)) {
	case ABNORMAL_TERMINATION:
	case OK:
		/* The test succeeded, use this knowledge to limit the
		 * drive class to match the density just read.
		 */
		return(OK);
	case DRIVE_NOT_READY:
	case ERR_FORMAT:
	case BAD_SPECIFICATION:
	case ERR_SECTOR:
		floppy[nr_drive].density = dens;
		return(r);
	case ERR_DRIVE: break;
	default: return(ERR_OPEN);
       }
  }
  floppy[nr_drive].density = dens;
  return(UNKNOWN_FORMAT);
}

/****************************************************************************
 *                              test_read                                   *
 ****************************************************************************/
PRIVATE Byte test_read(Byte nr_drive, Byte density)
{

/* Try to read the highest numbered sector on cylinder 2.  Not all floppy
 * types have as many sectors per track, and trying cylinder 2 finds the
 * ones that need double stepping.
 */

  struct iorequest_s op;

  floppy[nr_drive].density = density;
  op.drive = nr_drive;
  op.io_request = DEV_READ;
  op.io_position = test_sector[density];
  op.io_buf = *tmp_buf;
  op.io_nbytes = DMA_BUF_SIZE;
  return do_request(&op);
}

/*===========================================================================*
 *                             do_request                                    *
 *===========================================================================*/
PRIVATE Byte do_request(struct iorequest_s *iop)
{
  Byte r;

  switch((r = f_schedule(iop))) {
    case BAD_DRIVE_NUMBER:
    case DRIVE_NOT_INSTALLED:
    case UNACCESSIBLE_COMMAND:
    case ERR_WR_PROTECT:
    case DRIVE_NOT_READY:
    case ERR_SECTOR:
    case ERR_FORMAT:
	 return (r);
    case ERR_REQUEST:
	 if (iop->transfered == 0) return (BAD_SPECIFICATION);
	   else return(ABNORMAL_TERMINATION);
    case ERR_OPERATION:
	 if (iop->transfered == 0) return (ERR_DRIVE);
	   else return(ABNORMAL_TERMINATION);
    case OK:
	 return(OK);
    default:
	 return (r);
   }
}

/*===========================================================================*
 *                              f_schedule                                   *
 *===========================================================================*/
PRIVATE Byte f_schedule(struct iorequest_s *iop)
{
  Byte r, sec, s = OK;
  struct FDD *fp;
  DWord bytes;
  int block;       /* Seen any 32M floppies lately? */
  unsigned count, dma_count;
  struct trans tp;

  if (iop->drive > NR_DRIVES -1 ) return (BAD_DRIVE_NUMBER);
    fp = &floppy[iop->drive];
  if (fp->density == NO_DENS) return (DRIVE_NOT_INSTALLED);
  if (iop->io_request!=DEV_WRITE && iop->io_request!=DEV_READ)
     return (UNACCESSIBLE_COMMAND);
  if (iop->io_nbytes == 0 ) return(OK);
  if (iop->io_position > nr_sectors[fp->density]) return(ERR_REQUEST);

  tp.command = iop->io_request;
  bytes = (DWord) iop->io_buf;
  count = (unsigned) bytes;
  tp.address = ((bytes & 0xFFFF0000) >> 12) + count;
  dma_count = dma_bytes_left(tp.address);

  if (iop->io_nbytes > dma_count) {
     count = dma_count & ~(SECTOR_MASK);
     s = ERR_REQUEST;
     }
    else count = (unsigned) iop->io_nbytes;
  if (count == 0) return (ERR_REQUEST);
  bytes = (nr_sectors[fp->density] - iop->io_position + 1) << SECTOR_SHIFT;
  if (count > bytes) {
     count = (unsigned) bytes;
     s = ERR_REQUEST;
   }
  sec = eot[fp->density];
  tp.cylinder = iop->io_position / (NR_HEADS * sec)
		 * steps_per_cyl[fp->density];
  tp.sector = iop->io_position % (NR_HEADS * sec);
  if (tp.sector == 0) {
     tp.sector = sec;
     tp.head = 1;
    }
   else {
     tp.head = tp.sector / sec;
     tp.sector %= sec;
     if (tp.sector == 0) {
       tp.sector = sec;
       tp.head = 0;
      }
    }
  tp.drive = iop->drive;

  iop->transfered = 0;

  do {
	dma_count = (sec * (NR_HEADS - tp.head) - tp.sector + 1)
		     << SECTOR_SHIFT;
	tp.count = count > dma_count ? dma_count : count;
	r = f_finish(&tp);
	switch ( r ) {
	  case OK:
	       count -= tp.count;
	       tp.address += tp.count;
	       iop->transfered += tp.count;
	       break;
	  default:
	       Transfered = iop->transfered;
	       return(r);
	}
  } while (count > 0);
  Transfered = iop->transfered;
  return(s);
}

/*===========================================================================*
 *                              f_finish                                     *
 *===========================================================================*/
PRIVATE Byte f_finish(struct trans *tp)
{
/* Carry out the I/O requests gathered in ftrans[].  */

  struct FDD *fp = &floppy[tp->drive];
  int r, errors;

  if (tp->count == 0) return(OK); /* Spurious finish. */

  /* See if motor is running; if not, turn it on and wait */
  if (start_motor(tp->drive)==ERR_TIMEOUT)
	return(DRIVE_NOT_READY);

  if (PC_AT) out_byte(FDC_RATE, rate[fp->density]);
  /* This loop allows a failed operation to be repeated. */
  errors = 0;
  do {
	/* First check to see if a reset is needed. */
	if (I8272.need_reset) {
	   f_reset();
	   if (PC_AT) out_byte(FDC_RATE, rate[fp->density]);
	 }
	/* If we are going to a new cylinder, perform a seek. */
	r = seek(tp);
	if (r == OK) r = read_id(tp);

	/* Look for the next job in ftrans[] */
	if (r == OK) {
		dma_setup(tp);
		r = f_transfer(tp);
	  }
	switch (r) {
	  case OK:
	  case ERR_WR_PROTECT:
	  case ERR_FORMAT:
	       return r;
	  case ERR_TIMEOUT:
	       return(DRIVE_NOT_READY);
	  default:
	       errors ++;
	       if (errors > MAX_ERRORS / 2) fp->calibration = UNCALIBRATED;
	 }
  } while (errors < MAX_ERRORS);
  return r == ERR_SECTOR ? r : ERR_OPERATION;
}

/*===========================================================================*
 *                                f_wait                                     *
 *===========================================================================*/

/* Naplanuje alarm od CLOCK tasku a zablokuje se cekaje na interrupt od
FDD (HARDWARE). Pokud timeout vyprsi drive drive, vrati error ERR_TIMEOUT,
jinak OK */

PRIVATE int f_wait(DWord tm_out)
{

  static message m;
  int result;

  /* prevod tiku na sekundy pro Clock:SET_ALARM */
  tm_out+=HZ;
  tm_out=tm_out/HZ;

//  I8272.int_flag = FALSE;
 // P.G.: Vyhodil jsem, asi se nepouziva

  m.m_source=FLOPPY;
  m.m_type=SET_ALARM;
  m.FUNC=(sighandler_t)do_timeout;
  m.NEW_TIME=tm_out;
  m.CLOCK_PROC=FLOPPY;
  timeout_enable=FLOPPY_TIMEOUT;
  send_recv(CLOCK,m);

  receive(HARDWARE,m);
  /* P.G.: Odtud to muze vypadnout jednak kdyz kernel informuje o IRQ6
    (DISKINT), jednak z timeout funkce, ktera simuluje zaslani zpravy
     HARDWARE, */

  result=m.m_type;

  request_motor();
  // P.G.: Patri to sem vubec ? Problem je asi v tom, ze FDD muze mit najednou
  // registrovan jen jeden alarm a my potrebujeme soucasne udrzovat alarm
  // k vypnuti motoru a obcas do toho alarm-timeout pri cekani na intr
  // po skonceni exec faze prikazu 8272. Ale rekl bych, ze to tu klidne
  // muze byt - proste po skonceni kazdeho prikazu se zacne znovu odpocitavat
  // timeout pro motor, coz celkem dava smysl.

  if (result==DISKINT) return OK;
   else return ERR_TIMEOUT;
}

/*===========================================================================*
 *                              start_motor                                  *
 *===========================================================================*/
PRIVATE Byte start_motor(Byte nr_drive)
{
  Byte motor_bit;

//  if (I8272.need_reset) return (ERR_OPERATION);
  motor_bit = 1 << nr_drive + MOTOR_SHIFT; /* bit mask for this drive */
  if(!(I8272.dor & motor_bit)) {
     floppy[nr_drive].io_timeout=1;	/* enable timeout for motor stop */
     I8272.dor |= motor_bit | nr_drive;
     out_byte(DOR,I8272.dor);
     return f_wait(mtr_setup[floppy[nr_drive].density]);
				   /* waiting for motor run */
   }
  return OK;
}

/*===========================================================================*
 *                              stop_motor                                   *
 *===========================================================================*/
PRIVATE void stop_motor(Byte nr_drive)
{
/* Zastaveni motoru zadane mechaniky.
   Rutina je volana z do_timeout(), tedy z alarm funkce clock tasku. Alarm
   se planuje z request_motor(), ktera se vola z f_wait() vzdy po skonceni
   operace. Navic se vola z floppy_stop() - deinicializacni funkce.
 */

  Byte motor_bit = 1 << (nr_drive + MOTOR_SHIFT);
  floppy[nr_drive].io_timeout=0;	/* disable timeout for motor stop */

  if (I8272.dor & motor_bit) {
    I8272.dor &= ~motor_bit;
    out_byte(DOR, I8272.dor);
   }
}


/*===========================================================================*
 *                              floppy_stop                                  *
 *===========================================================================*/
PRIVATE void floppy_stop(void)
{
  Byte i;
/* Stop all activity. */
  for ( i = 0; i < NR_DRIVES; i++ )
    stop_motor(i);
}


/*===========================================================================*
 *                              dma_setup                                    *
 *===========================================================================*/
PRIVATE void dma_setup(struct trans *tp)
{
/* The IBM PC can perform DMA operations by using the DMA chip.  To use it,
 * the DMA (Direct Memory Access) chip is loaded with the 20-bit memory address
 * to be read from or written to, the byte count minus 1, and a read or write
 * opcode.  This routine sets up the DMA chip.  Note that the chip is not
 * capable of doing a DMA across a 64K boundary (e.g., you can't read a
 * 512-byte block starting at physical address 65520).
 */

  /* Set up the DMA registers.  (The comment on the reset is a bit strong,
   * it probably only resets the floppy channel.)
   */
  out_byte(DMA_INIT, DMA_RESET_VAL);    /* reset the dma controller */
  out_byte(DMA_FLIPFLOP, 0);            /* write anything to reset it */
  out_byte(DMA_MODE, tp->command == DEV_WRITE ? DMA_WRITE : DMA_READ);
  out_byte(DMA_ADDR, tp->address & 0xFF);
  out_byte(DMA_ADDR, (tp->address >> 8) & 0xFF);
  out_byte(DMA_TOP, (tp->address >> 16) & 0xFF);
  out_byte(DMA_COUNT, (tp->count - 1) & 0xFF);
  out_byte(DMA_COUNT, (tp->count - 1 >> 8) & 0xFF);
  out_byte(DMA_INIT, 2);        /* some sort of enable */
}


/*===========================================================================*
 *                              seek                                         *
 *===========================================================================*/
PRIVATE Byte seek(struct trans * tp)
{
/* Issue a SEEK command on the indicated drive unless the arm is already
 * positioned on the correct cylinder.
 */

  struct FDD *fp;
  int r;

  fp = &floppy[tp->drive];
  /* Are we already on the correct cylinder? */
  if (fp->calibration == UNCALIBRATED)
    if (recalibrate(tp->drive) != OK) return(ERR_SEEK);

  I8272.msr = in_byte(FDC_STATUS);
  if (I8272.msr & (1 << tp->drive)) return(ERR_SEEK_BUSY);

  /* No.  Wrong cylinder.  Issue a SEEK and wait for interrupt. */
  if (fp->pcn == tp->cylinder) return(OK);
  fdc_out(FDC_SEEK);
  fdc_out((tp->head << 2) | tp->drive);
  fdc_out(tp->cylinder);
  if (I8272.need_reset) return(ERR_SEEK);     /* if controller is sick, abort seek */
/*  fp->io_timeout = 0;*/
  if (f_intr_wait() != OK) return(ERR_TIMEOUT);

  /* Interrupt has been received.  Check drive status. */
  fdc_out(FDC_SENSE_INT);       /* probe FDC to make it return interrupt status */
  r = fdc_results();   /* get controller status bytes */
  if (r != OK || (I8272.results[ST0] & ST0_BITS) != SEEK_ST0 ||
	I8272.results[ST_PCN] != tp->cylinder)
	/* seek failed, may need a recalibrate */
	return(ERR_SEEK);
  fp -> pcn = I8272.results[ST_PCN];

  return(OK);
}

/*===========================================================================*
 *                              recalibrate                                  *
 *===========================================================================*/
PRIVATE Byte recalibrate(Byte nr_drive)      /* pointer to the drive struct */
{
/* The floppy disk controller has no way of determining its absolute arm
 * position (cylinder).  Instead, it steps the arm a cylinder at a time and
 * keeps track of where it thinks it is (in software).  However, after a
 * SEEK, the hardware reads information from the diskette telling where the
 * arm actually is.  If the arm is in the wrong place, a recalibration is done,
 * which forces the arm to cylinder 0.  This way the controller can get back
 * into sync with reality.
 */

  Byte r;
  struct FDD *fp = &floppy[nr_drive];

  /* Issue the RECALIBRATE command and wait for the interrupt. */
  if (start_motor(nr_drive)==ERR_TIMEOUT)
	return(DRIVE_NOT_READY);	/* can't recalibrate with motor off */

  I8272.msr = in_byte(FDC_STATUS);
  if (I8272.msr & (1 << nr_drive)) return(ERR_SEEK_BUSY);

  fdc_out(FDC_RECALIBRATE);     /* tell drive to recalibrate itself */
  fdc_out(nr_drive);            /* specify drive */
  if (I8272.need_reset) return(ERR_SEEK);     /* don't wait if controller is sick */
/*  fp->io_timeout = 0;*/
  if (f_intr_wait() != OK) return(ERR_TIMEOUT);

  /* Determine if the recalibration succeeded. */
  fdc_out(FDC_SENSE_INT); /* issue SENSE command to request results */
  r = fdc_results();      /* get results of the FDC_RECALIBRATE command*/
  fp->pcn = NO_CYL;       /* force a SEEK next time */
  if (r != OK ||          /* controller would not respond */
     (I8272.results[ST0] & ST0_BITS) != SEEK_ST0 || I8272.results[ST_PCN] != 0) {
	/* Recalibration failed.  FDC must be reset. */
	I8272.need_reset = TRUE;
	return(ERR_RECALIBRATE);
  } else {
	/* Recalibration succeeded. */
	fp->calibration = CALIBRATED;
	return(OK);
  }
}

/*==========================================================================*
 *                              read_id                                     *
 *==========================================================================*/
PRIVATE Byte read_id(struct trans *tp)      /* pointer to the drive struct */
{
/* Determine current cylinder and sector. */

  struct FDD *fp = &floppy[tp->drive];
  int result;

  /* Never attempt a read id if the drive is uncalibrated or motor is off. */
  if (fp->calibration == UNCALIBRATED) return(ERR_READ_ID);
  if ((I8272.dor & (1 << tp->drive + MOTOR_SHIFT)) == 0) return(ERR_READ_ID);

  /* The command is issued by outputting 2 bytes to the controller chip. */
  fdc_out(FDC_READ_ID);         /* issue the read id command */
  fdc_out((tp->head << 2) | tp->drive);

  /* Block, waiting for disk interrupt. */
  if (I8272.need_reset) return(ERR_READ_ID);  /* if controller is sick, abort op */

/*  fp->io_timeout = 0;*/
  if (f_intr_wait() != OK) return(ERR_TIMEOUT);

  /* Get controller status and check for errors. */
  result = fdc_results();
  if (result != OK) return(result);

  if ((I8272.results[ST0] & ST0_BITS) != TRANS_ST0) return(ERR_FORMAT);
  if (I8272.results[ST1] | I8272.results[ST2]) return(ERR_FORMAT);

  /* The next sector is next for I/O: */
  fp->r = I8272.results[ST_SEC];
  fp->pcn = I8272.results[ST_CYL];
  fp->h = I8272.results[ST_HEAD];
  return(OK);
}

/*===========================================================================*
 *                              f_transfer                                   *
 *===========================================================================*/
PRIVATE int f_transfer(struct trans *tp)
{
/* The drive is now on the proper cylinder.  Read, write or format 1 block. */

  struct FDD *fp;
  int r,s;

  fp = &floppy[tp->drive];

  /* Never attempt a transfer if the drive is uncalibrated or motor is off. */
  if (fp->calibration == UNCALIBRATED) return(ERR_TRANSFER);
  if ((I8272.dor & (1 << tp->drive + MOTOR_SHIFT)) == 0) return(ERR_TRANSFER);

/* The command is issued by outputting several bytes to the controller chip. */
  fdc_out(tp -> command == DEV_WRITE ? FDC_WRITE : FDC_READ);
  fdc_out((tp->head << 2) | tp->drive);
  fdc_out(tp->cylinder);
  fdc_out(tp->head);
  fdc_out(tp->sector);
  fdc_out(SECTOR_SIZE_CODE);
  fdc_out(eot[fp->density]);
  fdc_out(rw_gap[fp->density]);
  fdc_out(DTL);

  /* Block, waiting for disk interrupt. */
  if (I8272.need_reset) return(ERR_TRANSFER); /* if controller is sick, abort op */

/*  fp->io_timeout = 0;*/
  if (f_intr_wait() != OK) return(ERR_TIMEOUT);

  /* Get controller status and check for errors. */
  r = fdc_results();
  if (r != OK) return(r);

  if (I8272.results[ST1] & WRITE_PROTECT) {
	return(ERR_WR_PROTECT);
  }

  if ((I8272.results[ST0] & ST0_BITS) != TRANS_ST0) return(ERR_SECTOR);
  if (I8272.results[ST1] | I8272.results[ST2]) return(ERR_SECTOR);

  tp->cylinder = I8272.results[ST_CYL];
  tp->head = I8272.results[ST_HEAD];
  tp->sector = I8272.results[ST_SEC];
  return(OK);
}


/*==========================================================================*
 *                              fdc_results                                 *
 *==========================================================================*/
PRIVATE Byte fdc_results(void)
{
/* Extract results from the controller after an operation, then allow floppy
 * interrupts again.
 */
  int timeout;

  Byte result_nr = 0,status;

  /* Extract bytes from FDC until it says it has no more.  The loop is
   * really an outer loop on result_nr and an inner loop on status.
   */
  if (I8272.need_reset) return (ERR_STATUS);
  timeout = 0;
  do {
	/* Reading one byte is almost a mirror of fdc_out() - the DIRECTION
	 * bit must be set instead of clear, but the CTL_BUSY bit destroys
	 * the perfection of the mirror.
	 */
	I8272.msr = in_byte(FDC_STATUS);
	status = I8272.msr & (MASTER | DIRECTION | CTL_BUSY);
	if (status == (MASTER | DIRECTION | CTL_BUSY)) {
		if (result_nr >= MAX_RESULTS) break;    /* too many results */
		I8272.results[result_nr++] = in_byte(FDC_DATA);
		continue;
	}
	if (status == MASTER) break;

  } while (timeout++ < SMYCKA_TIMEOUT);

  if (status != MASTER ) {
      I8272.need_reset = TRUE; /* controller chip must be reset */
      return(ERR_STATUS);
    }
  return (OK);
}


/*===========================================================================*
 *                              fdc_out                                      *
 *===========================================================================*/
PRIVATE void fdc_out(Byte val)    /* write this byte to floppy disk controller */
{
/* Output a byte to the controller.  This is not entirely trivial, since you
 * can only write to it when it is listening, and it decides when to listen.
 * If the controller refuses to listen, the FDC chip is given a hard reset.
 */
  int timeout;

  if (I8272.need_reset) return;     /* if controller is not listening, return */

  /* It may take several tries to get the FDC to accept a command. */
  timeout = 0;
  while (((I8272.msr = in_byte(FDC_STATUS)) & (MASTER | DIRECTION))
	!= (MASTER | 0)) {
	if (timeout++ >= SMYCKA_TIMEOUT) {
		/* Controller is not listening.  Hit it over the head. */
		I8272.need_reset = TRUE;
		return;
	}
  }
  out_byte(FDC_DATA, val);
}

/*===========================================================================*
 *                              f_intr_wait                                  *
 *===========================================================================*/
PRIVATE Byte f_intr_wait(void)
{
/* Wait for an interrupt or until timeout */

  int wait_result;

  wait_result=f_wait(WAKEUP);

  if (wait_result==OK) return(OK);
  else
	{
	/* No interrupt from the FDC, this means that there is probably no
	 * floppy in the drive. Return error.
	 */
	I8272.need_reset = TRUE;
	return(ERR_TIMEOUT);
	}

/*  if (!(I8272.int_flag)) {
	/* No interrupt from the FDC, this means that there is probably no
	 * floppy in the drive.  Get the FDC down to earth and return error.
1	I8272.need_reset = TRUE;
	return(ERR_TIMEOUT);
  }
  return(OK);
*/
}

/*****************************************************************************
 *                          f_sense_drive                                    *
 *****************************************************************************/

PRIVATE Byte f_sense_drive(Byte nr_drive)
{
  if ( nr_drive > NR_DRIVES - 1) return(BAD_DRIVE_NUMBER);
 /* Executes FDC SENSE DRIVE STATUS command */
  fdc_out(FDC_SENSE_DRIVE);
  fdc_out(nr_drive);
  if (I8272.need_reset) return(ERR_SENSE);
  if (fdc_results() == OK) floppy[nr_drive].st3 = I8272.results[ST3];
  return(OK);
}

/*============================================================================*
 *                              floppy_task                                   *
 *============================================================================*/

PUBLIC void floppy_task(void)
{
    static message msg;
    struct iorequest_s req;
    int  proc_nr,r,i;
    static int  nrMinors;
    static long nrBlocks;

    static Byte buf[BLOCK_SIZE];

    floppy_init();
    for( i = 0; i < NR_DRIVES; i++ )
     r = f_open(i);

    while(TRUE)
	{
	receive(ANY,msg);
	proc_nr=msg.m_source;
	switch (msg.m_type)
	    {
	    case DEV_IOCTL:
	    // upraveno. P.G.
		i=msg.m_u.m_dio.device;
		if (i > NR_DRIVES -1 ) msg.REP_STATUS=ENXIO;	/* no such device or address */
		else	msg.REP_STATUS=OK;
		switch (msg.m_u.m_dio.count) {
		    case GETDEVSIZE:
			nrBlocks=eot[floppy[i].density] / 2;
			// P.G.: Ten prepocer na bloky je naprosto podivny

			i=sys_copy(FLOPPY,D,(vir_bytes)&nrBlocks,
				   msg.m_source,D,msg.m_u.m_dio.address,
				   sizeof(nrBlocks));
			if (i!=OK) msg.REP_STATUS=i;
			break;

		    case GETNUMBERMINORDEV:
			nrMinors = 2;
			// P.G.: Sem nesmi prijit konstanta! (ale radsi 2 nez 4)
			i=sys_copy(FLOPPY,D,(vir_bytes)&nrMinors,
				   msg.m_source,D,msg.m_u.m_dio.address,
				   sizeof(nrMinors));
			if (i!=OK) msg.REP_STATUS=i;
			break;
		    default:
			msg.REP_STATUS=EINVAL;	/*Invalid argument */
			break;
		}
		break;

	    case DEV_WRITE:
		// P.G. prevezmi buffer z uziv. procesu do buf
		if (msg.m_u.m_dio.count!=1) { msg.REP_STATUS=EINVAL; break; }

		i=sys_copy(msg.m_u.m_dio.proc_nr,D,msg.m_u.m_dio.address,
			   FLOPPY,D,(vir_bytes)&buf,
			   msg.m_u.m_dio.count*BLOCK_SIZE);
		if (i!=OK) { msg.REP_STATUS=i; break; }

	    case DEV_READ:
		// P.G.
		if (msg.m_u.m_dio.count!=1) { msg.REP_STATUS=EINVAL; break; }

		req.drive=msg.m_u.m_dio.device;
		req.io_request=msg.m_type;
		req.io_nbytes=msg.m_u.m_dio.count*BLOCK_SIZE;
		req.io_position=msg.m_u.m_dio.position*2;
		req.io_buf=(DWord)buf;
		r=do_request(&req);
		switch (r)
		{
		    case ERR_WR_PROTECT:        /* diskette is write protected */
			 msg.REP_STATUS=EACCES;	/* permission denied */
			 break;
		    case ERR_SECTOR:            /* Bad sector on diskette */
		    case ERR_FORMAT:            /* Diskette is not formatted */
		    case ERR_DRIVE:             /* Some error with drive  */
		    case ABNORMAL_TERMINATION:  /* Request started, but not successfully*/
			 msg.REP_STATUS=EIO;    /* recalibrate didn't work properly */
			 break;
		    case UNACCESSIBLE_COMMAND:  /* Command was not identified */
		    case BAD_SPECIFICATION:     /* Desired request could not be executed */
			 msg.REP_STATUS=EINVAL;  	/* invalid argument */
			 break;
		    case BAD_DRIVE_NUMBER:      /* Selected bad drive number */
		    case DRIVE_NOT_INSTALLED:   /* Drive is not installed */
		    case DRIVE_NOT_READY:       /* Diskette is not present in drive */
			 msg.REP_STATUS=ENXIO;		/* no such device or address */
			 break;
		    case OK:
			 // upraveno. P.G.
			 if (req.io_request==DEV_READ) {

			  i=sys_copy(FLOPPY,D,(vir_bytes)&buf,
				     msg.m_u.m_dio.proc_nr,D,msg.m_u.m_dio.address,
				     msg.m_u.m_dio.count*BLOCK_SIZE);
			  if (i==OK) msg.REP_STATUS=Transfered;
			   else msg.REP_STATUS=i;
			 } else msg.REP_STATUS=Transfered;
			 break;
		}
		break;
	    default:
		msg.REP_STATUS=EINVAL;	/*Invalid argument */
		break;
	    }
	msg.m_source=FLOPPY;
	msg.m_type=TASK_REPLY;
	send(proc_nr,msg);
	}
}

