/************************************************
*	FLOPPY TASK				*
*		Wrtten by:      		*
*			Jiri Kubica		*
*			Libor Cudek	L93360	*
*	Sestava:				*
*		floppy.h     header		*
*		floppy.c     main		*
*		floppy_s.h   header struct	*
*		floppy_d.h   header def		*
*						*
*************************************************/

#define FLOPPY_TIMEOUT	1	/* specifikovan timeout na cteni/zapis  */
#define MOTOR_TIMEOUT	2	/* specifikovan timeout na vypnuti motoru */
#define NONE_TIMEOUT	0	/* nespecifikovana zadna zadost na timeout */

#define SMYCKA_TIMEOUT	100	/* smycka ceka na FDC, kdysi bylo 10 preruseni od casovace (hodnota TIMEOUT)*/

#define PC_AT        TRUE

#define DEV_FORMAT        3

/* I/O Ports used by floppy disk task. */
#define DOR            0x3F2    /* motor drive control bits */
#define FDC_STATUS     0x3F4    /* floppy disk controller status register */
#define FDC_DATA       0x3F5    /* floppy disk controller data register */
#define FDC_RATE       0x3F7    /* transfer rate register */
#define DMA_ADDR       0x004    /* port for low 16 bits of DMA address */
#define DMA_TOP        0x081    /* port for top 4 bits of 20-bit DMA addr */
#define DMA_COUNT      0x005    /* port for DMA count (count =  bytes - 1) */
#define DMA_FLIPFLOP   0x00C    /* DMA byte pointer flip-flop */
#define DMA_MODE       0x00B    /* DMA mode port */
#define DMA_INIT       0x00A    /* DMA init port */
#define DMA_RESET_VAL   0x06

/* Number of bytes you can DMA before hitting a 64K boundary: */
#define dma_bytes_left(phys)    \
   ((unsigned) (sizeof(int) == 2 ? 0 : 0x10000) - (unsigned) ((phys) & 0xFFFF))

/* Parameters for the disk drive. */
#define SECTOR_SIZE      512    /* physical sector size in bytes */
#define SECTOR_SHIFT       9    /* for division */
#define SECTOR_MASK      511    /* and remainder */

/* Size of the DMA buffer buffer in bytes. */
#define DMA_SECTORS        1
#define DMA_BUF_SIZE    (DMA_SECTORS * SECTOR_SIZE)


/* Status registers returned as result of operation. */
#define ST0             0x00    /* status register 0 */
#define ST1             0x01    /* status register 1 */
#define ST2             0x02    /* status register 2 */
#define ST3             0x00    /* status register 3 (return by DRIVE_SENSE) */
#define ST_CYL          0x03    /* slot where controller reports cylinder */
#define ST_HEAD         0x04    /* slot where controller reports head */
#define ST_SEC          0x05    /* slot where controller reports sector */
#define ST_PCN          0x01    /* slot where controller reports present cyl */

/* Fields within the I/O ports. */
/* Main status register. */
#define CTL_BUSY        0x10    /* bit is set when read or write in progress */
#define DIRECTION       0x40    /* bit is set when reading data reg is valid */
#define MASTER          0x80    /* bit is set when data reg can be accessed */

/* Digital output port (DOR). */
#define MOTOR_SHIFT        4    /* high 4 bits control the motors in DOR */
#define ENABLE_INT      0x0C    /* used for setting DOR port */

/* ST0. */
#define ST0_BITS        0xF8    /* check top 5 bits of seek status */
#define TRANS_ST0       0x00    /* top 5 bits of ST0 for READ/WRITE */
#define SEEK_ST0        0x20    /* top 5 bits of ST0 for SEEK */

/* ST1. */
#define WRITE_PROTECT   0x02    /* bit is set if diskette is write protected */

/* ST2. */
/*#define BAD_CYL         0x1F    NOT USED/* if any of these bits are set, recalibrate */

/* ST3 (not used). */
/*#define ST3_FAULT       0x80    NOT USED/* if this bit is set, drive is sick */
/*#define ST3_WR_PROTECT  0x40    NOT USED/* set when diskette is write protected */
/*#define ST3_READY       0x20    NOT USED/* set when drive is ready */

/* Floppy disk controller command bytes. */
#define FDC_FORMAT      0x4D    /* command the drive to format a track */
#define FDC_READ        0xE6    /* command the drive to read */
#define FDC_READ_ID     0x4A    /* command the drive to read sector identity */
/*#define FDC_READ_TRACK  0x62    NOT USED/* command the drive to read a track */
#define FDC_RECALIBRATE 0x07    /* command the drive to go to cyl 0 */
#define FDC_SEEK        0x0F    /* command the drive to seek */
#define FDC_SENSE_DRIVE 0x04    /* command the controller to tell its status */
#define FDC_SENSE_INT   0x08    /* command the controller to tell interrupt status */
#define FDC_SPECIFY     0x03    /* command the drive to accept params */
#define FDC_WRITE       0xC5    /* command the drive to write */

/* DMA channel commands. */
#define DMA_READ        0x46    /* DMA read opcode */
#define DMA_WRITE       0x4A    /* DMA write opcode */

/* Parameters for reading information of installed disk drives from CMOS. */
#define DISK_DATA_SHIFT 0x04    /* Bit shift of byte of diskette parameters */
#define CMOS_ADR_PORT   0x70    /* CMOS address port */
#define CMOS_DATA_PORT  0x71    /* CMOS input output data port */
#define CMOS_DISK_DATA  0x10    /* Address of installed drives in CMOS */

/* Parameters for the disk drive. */
/*#define HC_SIZE         2880    NOT USED/* # sectors on largest legal disk (1.44MB) */
#define NR_HEADS        0x02    /* two heads (i.e., two tracks/cylinder) */
#define DTL             0xFF    /* determines data length (sector size) */
#define FILLING_CHAR    0xF6    /* filling char for format */
#define HEAD_SETTLE     0x0F    /* head settling time */
#define SPEC1           0xDF    /* step rate time and head unload time */
#define SPEC2           0x02    /* second parameter to SPECIFY */
#define MOTOR_OFF       3*HZ    /* how long to wait before stopping motor */
#define WAKEUP          2*HZ    /* timeout on I/O, FDC won't quit.  */

/* Error codes */

#define ERR_SEEK           1    /* bad seek */
#define ERR_TRANSFER       2    /* bad transfer */
#define ERR_TIMEOUT        3    /* interrupt timeout */
#define ERR_READ_ID        4    /* bad read id */
#define ERR_RECALIBRATE    5    /* recalibrate didn't work properly */
#define ERR_SEEK_BUSY      6    /* Drive is in seek mode during new seek */
#define ERR_WR_PROTECT     7    /* diskette is write protected */
#define ERR_STATUS         8    /* something wrong when getting status */
#define ERR_SECTOR         9    /* Bad sector on diskette */
#define ERR_FORMAT        10    /* Diskette is not formatted */
#define ERR_OPERATION     11    /* Operation can not be succcessfuly finished */

#define ERR_OPEN          12    /* Error during openning drive */
#define ERR_SENSE         13    /* Error while drive sense */
#define ERR_DRIVE         14    /* Some error with drive  */
#define ERR_REQUEST       15    /* Condition on request set dirty */

#define BAD_DRIVE_NUMBER      20  /* Selected bad drive number */
#define DRIVE_NOT_INSTALLED   21  /* Drive is not installed */
#define DRIVE_NOT_READY       22  /* Diskette is not present in drive */
#define UNKNOWN_FORMAT        23  /* Diskette is not detected */
#define UNACCESSIBLE_COMMAND  25  /* Command was not identified */
#define BAD_SPECIFICATION     26  /* Desired request could not be executed */
#define ABNORMAL_TERMINATION  27  /* Request started, but not successfully
				     terminated */

/* Miscellaneous. */
#define MAX_ERRORS         3    /* how often to try rd/wt before quitting */
#define MAX_RESULTS        7    /* max number of bytes controller returns */
#define NR_DRIVES          2    /* maximum number of drives */
/*#define DIVISOR          128    NOT USED/* used for sector size encoding */
#define SECTOR_SIZE_CODE   2    /* code to say "512" to the controller */
/*#define TIMEOUT           10    NOT USED/* tics waiting for FDC */
#define NT                 4    /* number of diskette/drive combinations PC AT*/
#define UNCALIBRATED       0    /* drive needs to be calibrated at next use */
#define CALIBRATED         1    /* no calibration needed */
/*#define BASE_SECTOR        1    NOT USED/* sectors are numbered starting at 1 */
#define NO_SECTOR          0    /* current sector unknown */
#define NO_CYL           (-1)   /* current cylinder unknown, must seek */
#define NO_DENS          100    /* current media unknown */

#define IC_PORT		0x20