/************************************************
*	FLOPPY TASK				*
*		Wrtten by:      		*
*			Jiri Kubica		*
*			Libor Cudek	L93360	*
*	Sestava:				*
*		floppy.h     header		*
*		floppy.c     main		*
*		floppy_s.h   header struct	*
*		floppy_d.h   header def		*
*						*
*************************************************/

/* Variables. */
struct FDD {                    /* main drive struct, one entry per drive */
  Byte pcn;                     /* present cylinder number*/
  Byte r;                       /* sector addressed */
  Byte h;                       /* head number addressed */
  Byte calibration;             /* CALIBRATED or UNCALIBRATED */
  Byte density;                 /* NO_DENS = ?, 0 = 360K; 1 = 360K/1.2M; etc.*/
  Byte st3;                     /* drive status */
  Byte io_timeout;         	/* Counter of system tics since last I/O */
} floppy[NR_DRIVES];

struct FDC {
  Byte dor;                /* current digital output register */
  Byte need_reset;         /* set to 1 when controller must be reset */
  Byte msr;                /* FDC main status register */
  Byte results[MAX_RESULTS];/* status registers */
  Byte int_flag;          /* Flag is set - floppy interrupt was received */
 } I8272;

struct iorequest_s  {
   DWord io_nbytes;           /* Number of bytes to transfer */
   DWord transfered;          /* Bytes successfully transfered */
   DWord io_position;         /* Physical sector I/O starts  from */
   DWord io_buf;             /* Pointer to buffer in RAM */
   Byte io_request;           /* Type of request - DEV_READ, DEV_WRITE */
   Byte drive;
  };

/* Gather transfer data for each sector. */
struct trans {         /* precomputed transfer params */
  Byte command;        /* command to exucute */
  Byte cylinder;
  Byte sector;
  Byte head;
  Byte drive;
  unsigned count;       /* byte count */
  DWord address;        /* DMA physical address */
};

/* Combinations of diskette/drive are supported.
 *
 * PC AT
 * # Drive  diskette  Sectors  Tracks  Rotation Data-rate  Comment
 * 1  1.2M    360K      9       40     360 RPM  300 kbps   PC disk in AT drive
 * 2  1.2M    1.2M     15       80     360 RPM  500 kbps   AT disk in AT drive
 * 3  1.44M   720K      9       80     300 RPM  250 kbps   PS/2, et al.
 * 4  1.44M   1.44M    18       80     300 RPM  500 kbps   PS/2, et al. *
 * In addition, 720K diskettes can be read in 1.44MB drives, but that does
 * not need a different set of parameters.
 */

#define b(d)    (1 << (d))      /* bit for density d. */

Byte rw_gap[NT] =
	{ 0x1B, 0x23, 0x2A, 0x1B}; /* gap size for read write operation */
Byte format_gap[NT] =
	{ 0x2A, 0x54, 0x54, 0x54}; /* gap size for format*/
Byte rate[NT] =
	{ 0x01, 0x00, 0x02, 0x00}; /* 2=250,1=300,0=500 kbps*/
Byte eot[NT] =
	{    9,   15,    9,   18}; /* sectors/track */
unsigned nr_sectors[NT] =
	{  720, 2400, 1440, 2880}; /* sectors/diskette*/
/*Byte nr_tracks[NT] =
	{   40,   80,   80,   80}; NOT USED/* sectors/diskette*/
Byte steps_per_cyl[NT] =
	{    2,    1,    1,    1}; /* 2 = dbl step */
Byte mtr_setup[NT] =
	{3*HZ/4,3*HZ/4,4*HZ/4,4*HZ/4}; /* in ticks, seconds */
Byte test_sector[NT] =
	{  2*9,  14,  4*9,   17 };  /* to recognize it */
Byte test_density[NT] =
	{ b(1), b(1)|b(2), b(3), b(3)|b(4)};


#define _DISKPARM_H
Byte tmp_buf[DMA_BUF_SIZE];   /* the DMA buffer */

Byte timeout_enable=FLOPPY_TIMEOUT;	/* jaky typ timeoutu je zvolen (floppy,motor,none)*/
unsigned Transfered;			/* kolik byte bylo preneseno (nacteno,zapsano)*/

#define  in_byte(adr)          	k_port_in(adr)        	/* returns data from port adr */
#define  out_byte(adr,val)	k_port_out(adr,val)   	/* writes val to port adr */
#define  disable()              k_lock()		/* - zakaz preruseni	*/
#define  enable()              	k_unlock()            	/* - povoleni preruseni */
