/************************************
 *  File system group               *
 *  cache.c                         *
 *  Miroslav Hastik     l93123      *
 *  Zdenek Rykala       l94034      *
 * mail: Miroslav.Hastik.fei@vsb.cz *
 * mail: Zdenek.Rykala.fei@vsb.cz   *
 ************************************/


/* The file system maintains a buffer cache to reduce the number of disk
 * accesses needed.  Whenever a read or write to the disk is done, a check is
 * first made to see if the block is in the cache.  This file manages the
 * cache. All accesses to disk are done by requesting the cache.
 *
 * The entry points into this file are:
 *   buf_pool:    initialize the buffer pool
 *   get_block:   request to fetch a block for reading or writing from cache
 *   put_block:   return a block previously requested with get_block
 *   alloc_zone:  allocate a new zone (to increase the length of a file)
 *   free_zone:   release a zone (when a file is removed)
 *   invalidate:  remove all the cache blocks on some device (umount)
 *   flushall:    flush all dirty blocks for one device
 *   sync:        flush all dirty blocks for all device  
 *
 * Internal:
 *   rw_block:    read or write a block from the disk itself
 *   cachepanic   error code - file system panic
 */

#define EXTERN
//#include "fs/cache/buf.h"
#undef EXTERN
#define EXTERN extern
#include "fs/inode/super.h"
#include "fs/inode/inode.h"
#include "fs/device/device.h"
#include "fs/cache/cache.h"
#include "kernel/makra.h"
#include "test/trap.h"
#undef EXTERN


/*===========================================================================*
 *                              rw_block                                     *
 *===========================================================================*/
PUBLIC void rw_block(struct buf *bp, int rw_flag)
/*register struct buf *bp;         buffer pointer
 *int rw_flag;                     READING or WRITING
 * Read or write a disk block. This is the only routine in which actual disk
 * I/O is invoked. If an error occurs, a message is printed here, but the error
 * is not reported to the caller.  If the error occurred while purging a block
 * from the cache, it is not clear what the caller could do about it anyway.
 */

{
/*        long pos;*/
	int op_flag;
/*        pos=bp->b_blocknr * BLOCK_SIZE;*/
/*        op_flag=dev_io(rw_flag,bp->b_dev,pos,BLOCK_SIZE,&(bp->b));*/ /* include device.h */
	op_flag=dev_io(rw_flag,	bp->b_dev, bp->b_blocknr, 1,FS, bp->b.b__data); /* include device.h */

	if (op_flag<0)  // kvuli chybe prekladace musi byt { }
	 { trap(TRAP_DISK_ERROR); /* Unrecoverable disk error */ }
	bp->b_dirt = CLEAN;
}

void cachepanic(int err)
{
 trap(err);
}

/*===========================================================================*
 *                              buf_pool                                     *
 *===========================================================================*/
PUBLIC void buf_pool()
{
	register int i;
	struct buf buffer;

/* bufs_in_use=0; */
buffer.b_blocknr=NO_BLOCK;
buffer.b_dev=NO_DEV;
buffer.b_dirt=CLEAN;
buffer.b_count=0;

for (i=0; i<NR_BUFS; i++) buf[i]=buffer;

rear=&(buf[0]);                 /* 'rear' pointing */
front=&(buf[NR_BUFS-1]);        /* 'front' pointing */

for (i=0; i<NR_BUFS; i++)       /* link all buffers in a chain */
  {
    if (i != NR_BUFS-1)
      {
	buf[i].b_next=&(buf[i+1]);
/*	buf[i].b_hash=&(buf[i+1]); P.G. */
      }
    else
     {
	buf[i].b_next=NULL;
/*	buf[i].b_hash=NULL; P.G. */
     }
    if (i != 0) buf[i].b_prev=&(buf[i-1]);
      else buf[i].b_prev=NULL;
    buf[i].b_hash=NULL;
  }

/* initialize hash table */
for (i=0; i<NR_BUF_HASH; i++) buf_hash[i]=NULL;

for (i=0; i<NR_BUFS; i++)       /* removing the blocks over 64kB */
    {
	long a;
	struct buf far *p; /*nejake divne melo by se predelat*/
       /* struct buf *p;*/
      p = &(buf[i]);
      a=(long) p;
      a=(a>>16);
      a=a+sizeof(buf);

/*    a = (a>>16)<<4 + a&0xffff;
      a&=0xffff;
      if ((0x10000-a)<BLOCK_SIZE) */

      if (a>0xffff)
	if ((i != 0) && (i != (NR_BUFS - 1)))
	{
	  buf[i-1].b_next=&(buf[i+1]);
/*	  buf[i-1].b_hash=&(buf[i+1]); P.G. */
	  buf[i+1].b_prev=&(buf[i-1]);
	}
	else
	{
	  if (i == 0)
	    {
	      rear=&(buf[1]);
	      buf[1].b_prev=NULL;
/*	      buf_hash[0]=&(buf[1]); P.G. */
	    }
	  if (i == (NR_BUFS - 1))
	    {
	      front=&(buf[i-1]);
	      buf[i-1].b_next=NULL;
	    }
	}
  }
}

/*===========================================================================*
 *                              get_block                                    *
 *===========================================================================*/
PUBLIC struct buf *get_block(
	register dev_t dev,
	register block_t block,
	int only_search)

/* register dev_t dev;              on which device is the block?
 * register block_t block;          which block is wanted?
 * int only_search;                 if NO_READ, don't read, else act NORMAL
 */

{
	struct buf *elem,*ep,*e;

/* Check to see if the requested block is in the block cache */
elem=buf_hash[block % NR_BUF_HASH];

while (elem != NULL)
  {
    if ((elem->b_blocknr == block) && (elem->b_dev == dev))
      {
	elem->b_count++;
	return elem;    /* return a pointer to it */
      }
    elem=elem->b_hash;
  }

/*
if (bufs_in_use == NR_BUFS)
  {
    cachepanic(TRAP_CACHE_NO_BUFF);
    return NIL_BUF;
  }
*/

/* find the block that is not in use */
elem=front;
while (elem != NULL)
  {
    if ((elem->b_count == 0) || (elem->b_dev == NO_DEV))
      {
	struct buf *h;
/*	bufs_in_use++; */
	if (elem->b_dirt == DIRTY) rw_block(elem,WRITING);

	/* remove block from hash chain */

	if (buf_hash[elem->b_blocknr % NR_BUF_HASH] == elem)
	 buf_hash[elem->b_blocknr % NR_BUF_HASH]=elem->b_hash;
	else {
	 ep=buf_hash[elem->b_blocknr % NR_BUF_HASH];
	 e=buf_hash[elem->b_blocknr % NR_BUF_HASH]->b_hash;
	 while (e!=NULL && e!=elem) {
	  ep=e;
	  e=e->b_hash;
	 }
	 if (e) ep->b_hash=e->b_hash;
	}

	elem->b_blocknr=block;
	elem->b_dev=dev;
	elem->b_count=1;

	if (only_search != NO_READ) rw_block(elem,READING);

	/* add to a hash chain */
	elem->b_hash=buf_hash[block % NR_BUF_HASH];
	buf_hash[block % NR_BUF_HASH]=elem;
	return elem;
      }
    elem=elem->b_prev;
  }

cachepanic(TRAP_CACHE_NO_BUFF);
return NIL_BUF;
}

/*===========================================================================*
 *                              put_block                                    *
 *===========================================================================*/
PUBLIC void put_block(register struct buf *bp, int block_type)

/*register struct buf *bp;         pointer to the buffer to be released
 *int block_type;                  INODE_BLOCK, DIRECTORY_BLOCK, or whatever
 */

{
       //	struct buf *b;
bp->b_count--;
if (bp->b_count > 0) return ;   /* block is still in use */

/* bufs_in_use--; */

/* Remove the block from it's original position in the LRU chain */

if (bp->b_prev==NULL) rear=bp->b_next;
 else if (bp->b_next==NULL) front=bp->b_prev;
  else {
   bp->b_prev->b_next=bp->b_next;
   bp->b_next->b_prev=bp->b_prev;
  }

/* return a block to the LRU chain depending on block_type */

if (!(block_type & ONE_SHOT))
  {
    bp->b_next=rear;
    bp->b_prev=NULL;
    rear=bp;
  }
 else
  {
    bp->b_prev=front;
    bp->b_next=NULL;
    front=bp;
  }

if (block_type & WRITE_IMMED)  /* write immedialety blocks */
 if (bp->b_dirt == DIRTY) rw_block(bp,WRITING);
}

/*===========================================================================*
 *                              alloc_zone                                   *
 *===========================================================================*/
PUBLIC zone_t alloc_zone(dev_t dev)

/* dev_t dev;                       device where zone wanted
 */
{
        struct super_block *sup = get_super(dev);  /* load the superblock - include super.h */
	zone_t zone;
  zone=alloc_bit(sup,ZONE_BITMAP);      /* alocate a new zone - include super.h */
  return ((zone == 0) ? NO_ZONE : zone+sup->s_firstdatazone-1);
}

/*===========================================================================*
 *                              free_zone                                    *
 *===========================================================================*/
PUBLIC void free_zone(dev_t dev, zone_t number)

/* dev_t dev;                               device where zone located
 * zone_t numb;                             zone to be returned
 */

{
        struct super_block *sup = get_super(dev); /* load the superblock - include super.h */
  free_bit(sup,ZONE_BITMAP,number);               /* free zone - include super.h */
}

/*===========================================================================*
 *                              invalidate                                   *
 *===========================================================================*/
PUBLIC void invalidate(dev_t dev)
/* Remove all the blocks belonging to some device from the cache. */
/* (writting them to disk if needed                               */
/* dev_t dev;                    device whose blocks are to be purged */

{
        int i;
for (i=0; i<NR_BUFS; i++)
  if (buf[i].b_dev == dev)
    {
      if (buf[i].b_dirt == DIRTY) rw_block(&(buf[i]),WRITING);
      buf[i].b_dev=NO_DEV;      /* remove the block */
      buf[i].b_blocknr=NO_BLOCK;
    }
}

/*===========================================================================*
 *                              flushall                                     *
 *===========================================================================*/

PUBLIC void flushall(dev_t dev)
/* Flush all dirty blocks for one device. */
/*dev_t dev;                      device to flush */
{
        int i;
for (i=0; i<NR_BUFS; i++)
  if ((buf[i].b_dev == dev) || (dev == NO_DEV))
    if (buf[i].b_dirt == DIRTY) rw_block(&(buf[i]),WRITING);
}

/*===========================================================================*
 *                              sync                                         *
 *===========================================================================*/
PUBLIC int sync(void)
/* Flush all dirty blocks for all device. */
{
  register int i;
  struct super_block *pSuperblock;

  for (i = 0; i < NR_SUPERS; i++) {
    pSuperblock = super_blocks+i;
    if (pSuperblock->s_dev != NO_DEV) flushall(pSuperblock->s_dev);
  }
  return 0;
}
