/* Vrstva device pro FILESYSTEM
 * Programuje Karel Krecmer l93589
 * 
 * Updated 4. 1.  98
 *
 * When a needed block is not in the cache, it must be fetched from the disk.
 * Special character files also require I/O.  The routines for these are here.
 *
 * The entry points in this file are:
 *   dev_open:   called when special file is opened
 *   dev_close:  called when special file is closed 
 *   dev_io:     perform a read/write on a block or character device
 *   do_ioctl:   perform IOCTL system call
 *   rw_dev:     procedure that actually calls the kernel tasks
 *   no_call:	 nothing to do, return OK
 */

#include "fs/device/device.h"
/*#include "common/glo.h"*/
#define EXTERN extern
#include "fs/common/fproc.h"
#include "fs/common/file.h"
#include "fs/common/fsglo.h"
#include "fs/device/pipe.h"
#include "fs/inode/inode.h"
#include "kernel/makra.h"
#undef EXTERN

/*
 *  Definitions
 */
/*#define USE_DEVICE_TABLE */ /*zda-li pouzije tabulku funkci obsluhujicich zarizeni*/
/* Nyni v tuoxu mame standardni format zprav pro tasky obsluhujici zarizeni
 * V pripade,  ze by bylo treba posilat ruzne formaty zprav lze udelat vice
 * funkci a ty dat do tabulky (dmap), v funkcich dev_io,  dev_open,  dev_close
 * by pak se volaly funkce pomoci dmap device table.
 */

#define no_dev no_call
/*
 * END of Definitions
 */

#ifdef USE_DEVICE_TABLE

int rw_dev(int task_nr, message* msg);
int no_call(int task_nr, message* msg);

struct dmap_t {
  int (*dmap_open)(int task_nr, message* msg);
  int (*dmap_rw)(int task_nr, message* msg);
  int (*dmap_close)(int task_nr, message* msg);
  int dmap_task; /* tash which handles these device class */
};

/*
 * Device table.  This table is indexed by major device number.  It provides
 * the link between major device numbers and the routines that process them.
 */

PRIVATE struct dmap_t dmap[] = {
/*      Open       Read/Write   Close       Task #      Device  File
 *	----       ----------   -----       -------     ------  ----           */
       {no_dev,    no_dev,      no_dev,     0},        /* 0 = not used         */
       {no_dev,    no_dev,      no_dev,     0},        /* 1 = not used         */
       {no_dev,    no_dev,      no_dev,     0},        /* 2 = not used         */
       {no_call,   rw_dev,      no_call,  FLOPPY},     /* 3 = /dev/fd[0-9]     */
       {no_dev,    no_dev,      no_dev,     0},        /* 4 = not used         */
       {no_call,   rw_dev,      no_call,  MEM},        /* 5 = /dev/mem         */
       {no_call,   rw_dev,      no_call,  TTY},        /* 6 = /dev/tty[0-9]    */
       {no_call,   rw_dev,      no_call,  HDD}         /* 7 = /dev/hd[a-z][0-9]*/
};
#endif

/*===========================================================================*
 *                              dev_open                                     *
 *===========================================================================*/
PUBLIC int dev_open(dev_t dev, mode_t mode)
{
  /* Momentalne reseno takto, nebot vsechna zarizeni pri funkci open
   * nebo close volaji no_call().
   * d_major - makro na zjisteni device major cisla
   * cislo tasku je rovno major cislu
   * Bude-li treba provest pro nektere zarizeni neco,  musi se udelat
   *  funkce pro to neco, dat do dmap a zformulovat zde message pro ni.
   */
#ifdef USE_DEVICE_TABLE
  return (dmap[d_major(dev)].dmap_open)(d_major(dev),&mess);
#else
  return OK;
#endif
}


/*===========================================================================*
 *                              dev_close                                    *
 *===========================================================================*/
PUBLIC int dev_close(dev_t dev)
{
  /* Momentalne reseno takto, nebot vsechna zarizeni pri funkci open
   * nebo close volaji no_call().
   * d_major - makro na zjisteni device major cisla
   * cislo tasku je rovno major cislu
   * Bude-li treba provest pro nektere zarizeni neco,  musi se udelat
   *  funkce pro to neco, dat do dmap a zformulovat zde message pro ni.
   */
#ifdef USE_DEVICE_TABLE
  return (dmap[d_major(dev)].dmap_close)(d_major(dev),&mess);
#else
  return OK;
#endif
}

/*===========================================================================*
 *                               dev_io                                      *
 *===========================================================================*/
PUBLIC int dev_io(int rw_flag, dev_t dev, block_t pos, int count, int proc, char *buf)
{
  /* Cte/zapisuje na zarizeni dev na pozici pos pocet count dat z bufferu buf
   * pricemz pozadavek vznesl proces proc a rw_flag rika zda-li jde o zapis ci cteni
   *
   * d_major - makro na zjisteni device major cisla
   * cislo tasku je rovno major cislu
   */
  /*mess.m_source - dosadi sendrec */
  static message mess;
  int m = d_major(dev);
  mess.m_type = (rw_flag == READING) ? DEV_READ : DEV_WRITE; /* kontrola neni ! */
  mess.m_u.m_dio.device = d_minor(dev);
  mess.m_u.m_dio.proc_nr = proc;
  mess.m_u.m_dio.count = count;
  mess.m_u.m_dio.position = pos;
  mess.m_u.m_dio.address = (vir_bytes) buf;
#ifdef USE_DEVICE_TABLE
  return (*dmap[m].dmap_rw)(m,&mess);
#else

  send_recv(m,mess);

  /*if (mess.REP_STATUS == SUSPEND) suspend(proc);*/ /* U Dalibora read.c */
  return mess.REP_STATUS; /*status*/
#endif
}


/*===========================================================================*
 *                               do_ioctl                                    *
 *===========================================================================*/
PUBLIC int do_ioctl()
{
/* Perform the ioctl(fd, request, arg) system call */
/*
Over platnost descriptoru, najdi pozici ve filp[]
Over, ze jde skutecne o specialni (znakovy) soubor
Vypln pole odpovedi ve zprave: erase & kill chars, flags */
  /*message mess; je uz globalni */
  /*fs_msg.m_source - dosadi sendrec */
  dev_t dev;
  int m;
  dev = fp->fp_filp[fs_msg.m_u.m_dio.device]->filp_ino->i_dev;
  m = d_major(dev);
  fs_msg.m_type = DEV_IOCTL;
  fs_msg.m_u.m_dio.device = d_minor(dev);
  fs_msg.m_u.m_dio.proc_nr = who;
  /*fs_msg.m_u.m_dio.count prednastaveno*/
  fs_msg.m_u.m_dio.position = 0;
  /*fs_msg.m_u.m_dio.address prednastaveno*/
  send_recv(m,fs_msg);
  return fs_msg.REP_STATUS; /*status*/
}

#ifdef USE_DEVICE_TABLE
/*===========================================================================*
 *			      Pomocne funkce
 *		    pouzivaji se jen pri device table
 *===========================================================================*/
/*===========================================================================*
 *                               rw_dev
 * na tuto funkci je nasmerovano dmap[].dmap_rw
 *===========================================================================*/
int rw_dev(int task_nr, message* msg)
{
  /* Na tuto funkci je nasmerovano dmap[].dmap_rw, momentalne pro vsechna dev
   *
   */
  //send_recv(task_nr,*msg);
  if (msg.REP_STATUS == SUSPEND) suspend(msg.m_u.m_dio.proc_nr);
  return msg.REP_STATUS;
}


/*===========================================================================*
 *                               no_call
 * prazdna funkce, vzdy v message result vraci OK                                  
 *===========================================================================*/
int no_call(int task_nr, message* msg)
{
   return OK;
}
#endif
