/***********************************************************
PIPEs jsme po dohode se cleny FS Development Team pro prvni
verzi systemu zrusili.
Jsou zde vsak nektere dulezite funkce, jako suspend,
revive a unpause
***********************************************************/
#define EXTERN extern
#include "common/type.h"
#include "common/const.h"
#include "common/errno.h"
#include "fs/common/fsconst.h"
#include "fs/common/file.h"
#include "fs/common/fproc.h"
#include "fs/common/fsglo.h"
#include "fs/inode/inode.h"
#include "kernel/makra.h"
#undef EXTERN
#include "fs/device/pipe.h"
#include "mm/mm2fsmsg.h"

  static message mess;

/* This file deals with the suspension and revival of processes.  A process can
 * be suspended because it wants to read or write from a pipe and can't, or
 * because it wants to read or write from a special file and can't.  When a
 * process can't continue it is suspended, and revived later when it is able
 * to continue.
 *
 * The entry points into this file are
 *   do_pipe:     perform the PIPE system call
 *   pipe_check:  check to see that a read or write on a pipe is feasible now
 *   suspend:     suspend a process that cannot do a requested read or write
 *   release:     check to see if a suspended process can be released and do it
 *   revive:      mark a suspended process as able to run again
 *   do_unpause:  a signal has been sent to a process; see if it suspended
 */

#ifdef PIPES
/*===========================================================================*
 *                              do_pipe                                      *
 *===========================================================================*/
PUBLIC int do_pipe()
{
/* Perform the pipe(fil_des) system call. 
Vytvor dva descriptory (+pozice ve filp[]) - jeden pro cteni, druhy pro zapis (get_fd())
Alokuj inode v current working directory, nastav v nem i_pipe=I_PIPE,
zvys usage count tohoto inode (dup_inode()), nastav odkaz na inode 
v obou polozkach filp[]
Vrat oba descriptory*/
}


/*===========================================================================*
 *                              pipe_check                                   *
 *===========================================================================*/
PUBLIC int pipe_check(rip, rw_flag, oflags, bytes, position, canwrite)
register struct inode *rip;     /* the inode of the pipe */
int rw_flag;                    /* READING or WRITING */
register int bytes;             /* bytes to be read or written (all chunks) */
register off_t position;        /* current file position */
{
/* Pipes are a little different.  If a process reads from an empty pipe for
 * which a writer still exists, suspend the reader.  If the pipe is empty
 * and there is no writer, return 0 bytes.  If a process is writing to a
 * pipe and no one is reading from it, give a broken pipe error.
 */
/*
Jde-li o cteni:
 Over cteni z prazdne roury (position>i_size)
 Zjisti z filp[], zda do roury nekdo pise
  pokud ne: vrat 0, 
  jinak suspend(XPIPE), aktivace pripadne suspendovaneho zapisovatele (release)

Jde-li o zapis:
 Zjisti z filp[], zda z roury nekdo cte. Pokud ne, generuj SIGPIPE
 jinak: je-li position+pozadovany_pocet_byte>PIPE_SIZE suspend(XPIPE)  pipe full 
	Je-li position=0 (zapis do prazdne roury): 
	  release() - uvolni suspendovane citatele*/
}


/*===========================================================================*
 *                              release                                      *
 *===========================================================================*/
PUBLIC void release(ip, call_nr, count)
register struct inode *ip;      /* inode of pipe */
int call_nr;                    /* READ or WRITE */
int count;                      /* max number of processes to release */
{
/* Check to see if any process is hanging on the pipe whose inode is in 'ip'.
 * If one is, and it was trying to perform the call indicated by 'call_nr',
 * release it.
 */
/*Projdi fproc[]. vyhledej SUSPENDED procesy vyhovujici zadanym kriteriim:
 revive(), susp_count--; <- maximalne (count) takovych
*/
}

#endif


/*===========================================================================*
 *                              suspend                                      *
 *===========================================================================*/
/*PUBLIC void suspend(int proc)
{*/
/* Take measures to suspend the processing of the present system call.
 * Store the parameters to be used upon resuming in the process table.
 * (Actually they are not used when a process is waiting for an I/O device,
 * but they are needed for pipes, and it is not worth making the distinction.)
 */
/*
je-li task=XPIPE: susp_count++;
fproc[].fp_suspended=SUSPENDED;
       .fp_fd= fd; nejspis tam natlacime i cislo system call
       .fp_buffer,fp_nbytes,fp_task
nastav dont_reply: neodpovidej z hlavni smycky

  fp->fp_suspended=SUSPENDED;*/
/* Dela Dalibor primo v fs/inode/read.c ... 
}*/


/*===========================================================================*
 *                              revive                                       *
 *===========================================================================*/
PUBLIC int do_revive()
/*int proc_nr;                     process to revive */
/*int bytes;                       if hanging on task, how many bytes read */
{
/* Revive a previously blocked process. When a process hangs on tty, this
 * is the way it is eventually released.
 */
/*
Proces revive je odlisny pro pipes a TTY: pri ozivivani TTY je akce jiz
provedena, u pipes se musi znovu spustit
Jde-li o XPIPE:
 fp_revived=REVIVING; reviving++; */
  /* zbytek se udela v hlavni smycce FS, resp. v rutine get_work()
jinak: fp_suspended=NOT_SUSPENDED; fp_nbytes=bytes;
       odpovez predtim zablokovanemu procesu: reply(proc_nr,bytes)
      */
      /* Zformulov */
  /*fproc[fs_msg->m1_u3]*/
  int m = fs_msg.m_u.m_dio.proc_nr;
  fproc[m].fp_suspended = NOT_SUSPENDED;
  fproc[m].fp_nbytes = fs_msg.m_u.m_dio.count;
  //mess.m_type = TASK_REPLY;
  mess.REP_STATUS = fs_msg.m_u.m_dio.count;
  send(m, mess);
  dont_reply = 1;
  return 0;
}


/*===========================================================================*
 *                              do_unpause                                   *
 *===========================================================================*/
PUBLIC int do_unpause()
{
/* A signal has been sent to a user who is paused on the file system.
 * Abort the system call with the EINTR error message.
 */
/*
 nejde-li o pipe (takze jde o TTY...):
  posli tasku CANCEL (minor device zjisti z fproc[].fp_fd --> inode.i_zone[0]
 revive(proc_nr,EINTR)
 (???)
 */
  int pn = fs_msg.fs_proc;  // process number
  if (fproc[pn].fp_suspended == SUSPENDED)
  {
    int fn = fproc[pn].fp_fd;
    dev_t dev = fproc[pn].fp_filp[fn]->filp_ino->i_dev;
    int madev = d_major(dev);  // major device number
    mess.m_type = TTY_CANCEL; // cancel request of proces
    mess.m_u.m_dio.device = d_minor(dev);
    mess.m_u.m_dio.proc_nr = pn;
    send(madev,mess);
    mess.m_type = TASK_REPLY;
    mess.REP_STATUS = EINTR;
    send(pn,mess);
  }
  return 0;
}
