/*************************************************************************
 * 		       filedes.c - implementation file			 *
 * 		   A part of TUOX file system project '97                *
 *			  by Dalibor Lukas, L 94578                      *
 *************************************************************************

/* This file contains the procedures that manipulate file descriptors.
 *
 * The entry points into this file are
 *   get_fd:    look for free file descriptor and free filp slots
 *   get_filp:  look up the filp entry for a given file descriptor
 *   find_filp: find a filp slot that points to a given inode
 */
#define EXTERN extern
#include "common/const.h"
#include "common/type.h"
#include "common/glo.h"
#include "common/errno.h"

#include "fs/common/fstype.h"
#include "fs/common/fproc.h"
#include "fs/inode/inode.h"
#include "fs/common/file.h"
#undef EXTERN

struct fproc_struct *fp;

/*===========================================================================*
 *                              get_fd                                       *
 *===========================================================================*/
PUBLIC int get_fd (mode_t bits, int *k, struct struct_filp **fpt)
/* mode_t bits;                  mode of the file to be created (RWX bits) */
/* int *k;                       place to return file descriptor */
/* struct filp **fpt;            place to return filp slot */
{
/*   Look for a free file descriptor and a free filp slot.  Fill in the mode word
   in the latter, but don't claim either one yet, since the open() or creat()
   may yet fail.*/

  int i;

  for (i=0; i < NR_FDS; ++i)
    if (fp->fp_filp[i] == NIL_FILP) break;

  if (i < NR_FDS) *k = i;
  else return EMFILE;

  for (i=0; i < NR_FILPS; ++i)
    if (filp[i].filp_count == 0) {
      filp[i].filp_mode = bits;
      filp[i].filp_pos = 0;
      *fpt = filp+i;
      break;
    }

  if (i < NR_FILPS) return OK;
  return EMFILE;

/*
   hledej ve fproc[].fp_filp[]=NIL_FILP, nastav *k
   Nenajdes-li, return(EMFILE);
   prohledej filp[], najdi filp_count=0, nastav *fpt
   najdes_li, return OK
   */
}


/*===========================================================================*
 *                              get_filp                                     *
 *===========================================================================*/
PUBLIC struct struct_filp *get_filp (int fild)
/* int fild;                       file descriptor */
{
  struct struct_filp *result = fp->fp_filp[fild];

  if ((result-filp)%sizeof(*filp) == 0)
  return NIL_FILP;

/* See if 'fild' refers to a valid file descr.  If so, return its filp ptr (else NIL_FILP). */
}


/*===========================================================================*
 *                              find_filp                                    *
 *===========================================================================*/
PUBLIC struct struct_filp *find_filp (register struct inode *rip, mode_t bits)
/* register struct inode *rip;   inode referred to by the filp to be found */
/* mode_t bits;                  mode of the filp to be found (RWX bits) */
{
/* Find a filp slot that refers to the inode 'rip' in a way as described
 * by the mode bit 'bits'. Used for determining whether somebody is still
 * interested in either end of a pipe.
 * Like 'get_fd' it performs its job by linear search through the filp table.
 */

  int i;
  struct struct_filp *fp_slot;

  for (i=0; i < NR_FILPS; ++i) {
    fp_slot = filp+i;
    if (fp_slot->filp_mode == bits && fp_slot->filp_ino == rip)
      return fp_slot;
  }
  return NIL_FILP;
}
