/* header file for read and write file system calls */

/* The entry points into this file are
 *   do_read:    perform the READ system call by calling read_write
 *   read_write: actually do the work of READ and WRITE
 *   read_map:   given an inode and file position, look up its zone number
 *   rw_user:   call the kernel to read and write user space
 *   do_write:     call read_write to perform the WRITE system call
 *   write_map:    add a new zone to an inode   
 *   clear_block:   erase a zone in the middle of a file
 *   new_block:    acquire a new block

 */

#ifndef __read_h
#define __read_h

#include "common/const.h"
#include "fs/common/fsconst.h"
#include "fs/cache/buf.h"
#include "fs/inode/inode.h"

#define TO_USER		0
#define FROM_USER	1

PUBLIC int do_read();
/*
m_type	=	READ
m1_i1		=	fd
m1_i2		=	buffer
m1_i3		=	nbytes
m1_p1		=	segment
*/

PUBLIC int do_write();
/*
m_type	=	WRITE
m1_i1		=	fd
m1_i2		=	buffer
m1_i3		=	nbytes
m1_p1		=	segment
*/

PUBLIC int read_write(int fd, char *buffer, int nbytes, int rw_flag, int seg);
/*int fd				     file descriptor */
/*char *buffer;	                 virtual address of the user buffer */
/*int nbytes;                     number of bytes to read or write */
/*int rw_flag;                     READING or WRITING */
/*int seg;                        T or D segment in user space */


/*PRIVATE int rw_chunk(register struct inode *rip, off_t f_position, int nbytes,
		     int rw_flag, char *buff, int seg, int usr);*/
/*register struct inode *rip;     pointer to inode for file to be rd/wr */
/*off_t f_position;               position within file to read or write */
/*int nbytes;                     number of bytes to read or write */
/*int rw_flag;                    READING or WRITING */
/*char *buff;                     virtual address of the user buffer */
/*int seg;                        T or D segment in user space */
/*int usr;                        which user process */


PUBLIC block_t read_map(register struct inode *rip, off_t position);
/*register struct inode *rip;		ptr to inode to map from */
/*off_t position;                	position in file whose blk wanted */

PUBLIC block_t write_map(register struct inode *rip, off_t position);
/*register struct inode *rip;      pointer to inode to be changed */
/*off_t position;                  file address to be mapped */

PUBLIC int rw_user(int s, int u, char *vir, int bytes, char *buff, int direction);
/*Prenos dat mezi adr. prostorem (I/D) uzivatele a FS */
/*int s;			D or T space in user proc */
/*int u;			process # */
/*vir_bytes vir;		address within user space */
/*vir_bytes bytes;	# of bytes to transfer */
/*char *buff;		buffer in FS's space */
/*int direction;		TO_USER /FROM_USER */

PUBLIC struct buf *new_block(register struct inode *rip, off_t position);
/* register struct inode *rip;     pointer to inode */
/* off_t position;                 file pointer */

PUBLIC struct buf *clear_block(struct buf *blok);

#endif	/* __read_h */
