/***************************************************************************
 * Projekt do predmetu OPERACNI SYSTEMY - TUOX                             *
 * Vypracoval  Miroslav GRELA  1997                                        *
 * Prepracoval Karel Pisala 1997  L94587                                   *
/***************************************************************************

/* This file performs the MOUNT and UMOUNT system calls.
 *
 * The entry points into this file are
 *   do_mount:  perform the MOUNT system call
 *   do_umount: perform the UMOUNT system call
 */

#define EXTERN extern

#include "common/const.h"
#include "common/type.h"
#include "common/com.h"
#include "fs/inode/inode.h"
/* #include "common/proc.h" */
#include "fs/common/fproc.h"
#include "common/errno.h"
#include "fs/inode/super.h"
#include "fs/cache/cache.h"
//#include "common/glo.h"
#include "fs/common/fsglo.h"
#include "fs/common/file.h"
#include "fs/common/fsmes.h"
#include "lib/include/string.h"
#include "fs/main.h"

#undef EXTERN

/*===========================================================================*
 * 				get_dir					     *
 *===========================================================================*/
 char *get_dir(char *path,int depth)
/* path:  cesta k nejakemu souboru ci adresari */
/* depth: hloubka adresare, ktery chceme vypreparovat (cislovani od 0) */
/* funkce vypreparuje urcity adresar, ktery je dan hloubkou, z cesty path */
{
  static char out[255];
  int count=0,outOffs=0,pathOffs=0;
  if(path[0]=='/') pathOffs++;
  while((path[pathOffs]!='\0')&&(count<=depth))
    {
      if(path[pathOffs]=='/') count++;
      else if(count==depth)
	     {
	       out[outOffs]=path[pathOffs];
	       outOffs++;
	     }
	     pathOffs++;
    }

  out[outOffs]='\0';
  return out;
}

/*===========================================================================*
 *                               search_dir                                  *
 *===========================================================================*/
struct inode *search_dir(struct inode *sinode, char *name)
{
  struct buf *dirent; /* ukazatel na buffer s adresarem */
  int i,j,
      nrzones,nrent;
  if ((sinode->i_mode & I_DIRECTORY)!=I_DIRECTORY)
    /* neni-li i-node adresar chyba */
    return NIL_INODE;
  if (sinode->i_mount==I_MOUNT)
    {
      for (i=0; i<NR_SUPERS; i++)
	{
	  struct super_block * sb=&(super_blocks[i]);
	  if (sb->s_imount==sinode)
	    {
	      sinode=sb->s_isup;
	      break;
	    }
	}
    }
  nrzones=(int)(sinode->i_size / BLOCK_SIZE);
  for (i=0; i<=nrzones; i++)
    {
      dirent=get_block(sinode->i_dev, sinode->i_zone[i], NORMAL);
      if (i<nrzones)
	nrent=NR_DIR_ENTRIES;
      else
	nrent=(int)((sinode->i_size - (i*BLOCK_SIZE)) /
		sizeof(struct dir_struct)) + 1;
      for (j=0; j<nrent; j++)
	if (strcmp(dirent->b.b__dir[j].d_name,name)==0) {
	  struct inode *dInode=get_inode(sinode->i_dev,
				       dirent->b.b__dir[j].d_inum);
	  put_block(dirent, DIRECTORY_BLOCK);
	  return dInode;
	}
      put_block(dirent, DIRECTORY_BLOCK);
    }
  return NIL_INODE;
}

/*===========================================================================*
 *                                name_i                                     *
 *===========================================================================*/
PUBLIC struct inode *name_i(char *path)
/* path:  cesta k nejakemu souboru ci adresari */
/* funkce vraci ukazate na inod souboru ci adresare, jehoz cesta je path */
/* pokud neni takovy adresar ci soubor nalezen ja vracena hodnota NIL_INODE */
{
  struct inode *dirInode,*ndirInode;
  int depth=0;
  char *dir;

  if (path[0] == '/') dirInode = get_inode(fp->fp_rootdir->i_dev, fp->fp_rootdir->i_num);
   /* = get_inode(ROOT_DEV, ROOT_INODE); */
  else dirInode=get_inode(fp->fp_workdir->i_dev, fp->fp_workdir->i_num);

  while(*(dir=get_dir(path,depth))!='\0')
    {
      ndirInode=search_dir(dirInode,dir);
      put_inode(dirInode);
      dirInode=ndirInode;
      if (dirInode==NIL_INODE) return NIL_INODE;
      depth++;
    }
  return dirInode;
}

/*===========================================================================*
 *                              do_mount                                     *
 *===========================================================================*/
PUBLIC int do_mount()
/* Perform the mount(name, mfile, rd_only) system call. */
/* Funkce provede namontovani noveho file systemu do adresaroveho stromu /*
/* Vraci hodnotu TRUE pokud namontovani probehlo uspesne */
/* V opacnem pripade vraci hodnotu FALSE */

/* Pred samotnym volanim funkce do_mount() je treba naplnit */
/* globalni promennou fs_msg (struct message) nasledovne: */
/* fs_msg.m_u.m_m1.m1p1 = name;  nazev adresare, na ktery se ma montovat */
/* fs_msg.m_u.m_m1.m1p2 = mfile; specialni soubor FS, ktery se ma montovat */
/* fs_msg.m_u.m_m1.m1i1 = rd_only;  urcuje, zda bude montovany FS read only */

/* Po provedeni funkce je naplnena globalni promenna fs_repmsg nasledovne : */
/* fs_repmsg.m_u.m_m1.m1i1 = errcode; chybovy kod */
{
  struct inode *pinodemf /* i-node mount file */
	 ,*pinoded        /* i-node mount directory */
	 ,*pinoder;       /* root i-node mount file */
  struct super_block* psuper;  /* ukazatel na superblock */

/* Over, zda volani provadi superuser (jen on smi) */
  if(!superuser)
    {
      fs_repmsg.fs_errcode=EPERM;
      return FALSE;
    }

/* overeni specialniho souboru FS */
/* Zjisti ze super_block[], zda jiz zarizeni neni namontovano (podle s_dev) */
  fetch_name((vir_bytes) fs_msg.m1_p2);
  if((pinodemf=name_i(user_path))==NIL_INODE)   /* cesta neexistuje */ 
    {
      fs_repmsg.fs_errcode=ENOENT;
      return FALSE;
    }
  if((pinodemf->i_mode & I_BLOCK_SPECIAL)!=I_BLOCK_SPECIAL)
		  /* soubor neni spec. zarizeni */
    {
      fs_repmsg.fs_errcode=ENODEV;
      return FALSE;
    }
  if(get_super(pinodemf->i_zone[0])!=NIL_SUPER) /* zarizeni je jiz mountovano */
    {
      fs_repmsg.fs_errcode=EBUSY;
      return FALSE;
    }

/* Najdi volny slot super_block[], nacti do nej superblock montovaneho FS */
  if ((psuper=get_super(NO_DEV))==NIL_SUPER)  /* neni volna polozka */
    {
      fs_repmsg.fs_errcode=EBUSY;
      return FALSE;
    }

/* Najdi inode zadaneho adresare (na ktery se ma mountovat)
 jeho i_count nesmi byt > 1 ) (ne busy), nesmi to byt specialni soubor
 (ale adresar) */
  fetch_name((vir_bytes) fs_msg.m1_p1);
  if((pinoded=name_i(user_path))==NIL_INODE) /* adresar neexistuje */
    {
      fs_repmsg.fs_errcode=ENOENT;
      return FALSE;
    }
  if((pinoded->i_mode & I_DIRECTORY)!=I_DIRECTORY)  /* neni adresar */
    {
      fs_repmsg.fs_errcode=ENOTDIR;
      return FALSE;
    }
  if(pinoded->i_count>1)     /* na adresar existuji odkazy */
    {
      fs_repmsg.fs_errcode=EBUSY;
      return FALSE;
    }

/* Najdi root inode nove montovaneho file systemu, nacti do inode[]
	 Nacti bitmapy inode a zon noveho file systemu */

  psuper->s_dev=pinodemf->i_zone[0];  /* zadej cislo zarizeni */
  rw_super(psuper,READING);           /* a zapis do tabulky superblock[] */

  pinoder=get_inode(pinodemf->i_zone[0],ROOT_INODE);
  load_bit_maps(pinodemf->i_zone[0]);

/* Zavazbi:
	 U adresare, na ktery se montuje nastav i_mount = I_MOUNT
	 v super_block[]nastav s_imount,.s_isup, s_rd_only */
  pinoded->i_mount=I_MOUNT;
  psuper->s_imount=pinoded;
  psuper->s_isup=pinoder;
  psuper->s_rd_only=fs_msg.m1_i1;

  fs_repmsg.fs_errcode=OK;
  return TRUE;
}


/*===========================================================================*
 *                              do_umount                                    *
 *===========================================================================*/
PUBLIC int do_umount()
/* Perform the umount(name) system call. */
/* Funkce provede odmontovani file systemu z adresaroveho stromu /*
/* Vraci hodnotu TRUE pokud odmontovani probehlo uspesne */
/* V opacnem pripade vraci hodnotu FALSE */

/* Pred samotnym volanim funkce do_umount() je treba naplnit */
/* globalni promennou fs_msg (struct message) nasledovne: */
/* fs_msg.m_u.m_m1.m1p1 = ufile; specialni soubor FS, ktery se ma odmontovat */

/* Po provedeni funkce je naplnena globalni promenna fs_repmsg nasledovne : */
/* fs_repmsg.m_u.m_m1.m1i1 = errcode; chybovy kod */
{
  struct inode* pinodemf;     /* i-node mounted file */
  int i, count=0;
  struct super_block *psuper;

/* Over, zda je volano superuserem */
  if(!superuser)
    {
      fs_repmsg.fs_errcode=EPERM;
      return FALSE;
    }

/* Zadane jmeno musi byt pojmenovanim block special file */
  fetch_name((vir_bytes) fs_msg.m1_p1);
  if((pinodemf=name_i(user_path))==NIL_INODE) /* cesta neexistuje */
    {
      fs_repmsg.fs_errcode=ENOENT;
      return FALSE;
    }
  if((pinodemf->i_mode & I_BLOCK_SPECIAL)!=I_BLOCK_SPECIAL)
     /* soubor neni spec. zarizeni */
    {
      fs_repmsg.fs_errcode=ENODEV;
      return FALSE;
    }

/* Projed inode[], zjisti, kolik otevrenych inode miri na odmontovavane device
	 Pokud vice nez 1 (root inode): return (EBUSY) */
  for (i=0;i<NR_INODES;i++)
    {
      if (inodes[i].i_dev==pinodemf->i_zone[0] && inodes[i].i_count!=0)
	count++;
    }
  if(count>1)
    {
      fs_repmsg.fs_errcode=EBUSY;
      return FALSE;
    }

/* Najdi superblok odmontovaveneho zarizeni, uvolni bitmapy, invalidate cache
	 Zrus flag I_MOUNT u adresare, kde bylo montovano a odkazy v superbloku */
  psuper=get_super(pinodemf->i_zone[0]);
  unload_bit_maps(pinodemf->i_zone[0]);
  invalidate(pinodemf->i_zone[0]);
  psuper->s_imount->i_mount=NO_MOUNT;
  put_inode(psuper->s_isup);
  psuper->s_isup=NIL_INODE;
  psuper->s_imount=NIL_INODE;

  fs_repmsg.fs_errcode=OK;
  return TRUE;
}
