	    /**********************************************\
	    **                                            **
	    **  TUOX                                      **
	    **                                            **
	    **  File System - LINK.C                      **
	    **                                            **
	    **  Prepracovali David Spalt, L94607          **
	    **               Daniel Belicek               **
	    **                                            **
	    \**********************************************/

/**************************************************************************\
**  int do_link()       vytvori link                                      **
**  int do_unlink()	zrusi link                                        **
**  void truncate()     odstrani vsechny bloky asociovane s inodem        **

Pomocne funkce - pridavani/ruseni polozek adresaru:
 PUBLIC int add_entry(struct inode *dInode, int Inum, char *name)
 PUBLIC int remove_entry(struct inode *dInode, int Inum)
\**************************************************************************/

#include "lib/include/string.h"

#define EXTERN extern

#include "common/const.h"
#include "common/type.h"
#include "fs/common/fsconst.h"
#include "fs/common/fsglo.h"
#include "common/errno.h"
#include "fs/common/fproc.h"
#include "fs/common/fstype.h"
#include "common/stat.h"
#include "fs/inode/inode.h"
#include "fs/mount/mount.h"
#include "fs/common/fsmes.h"
#include "fs/syscall/protect.h"
#include "fs/syscall/open.h"
#include "fs/common/file.h"
#include "fs/main.h"

#undef EXTERN

#define MAX_LINKS 	10	/* doplnit do .h */

int add_entry(struct inode *dInode, int Inum, char *name);
int remove_entry(struct inode *dInode, char* name);

/**************************************************************************\
**                                                                        **
**  int do_link()		vytvori link                              **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_LINK					  **
**	    fs_name		ukazatel na jmeno souboru                 **
**          fs_name2 	        ukazatel na jmeno linku                   **
**                                                                        **
**  vystup: fs_errcode		chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_link()
{
  char name2[NAME_SIZE];
  struct inode *pinode1,*dinode2,*tpinode;
  int i = 0, s_position = -1;

  /* prevod na inode, pokud existuje */

  fetch_name((vir_bytes)fs_msg.fs_name);
  pinode1 = name_i(user_path);

  if (pinode1 == NIL_INODE) { /* nejde o regulerni soubor nebo adresar */
	  fs_repmsg.fs_errcode=ENOENT;
	  return 0;
  }

  if (access(pinode1,S_IWRITE)!= OK) {  /* zakazany pristup */
	  fs_repmsg.fs_errcode=EACCES; put_inode(pinode1);
	  return 0;
  }

  if (pinode1 -> i_nlinks ==  MAX_LINKS) { /* prilis mnoho linku */
	  fs_repmsg.fs_errcode=EMLINK; put_inode(pinode1);
	  return 0;
  }

  /* pokud se jedna o adresar, muze linkovat jen super_user */
  if ((pinode1 -> i_mode & I_TYPE) == I_DIRECTORY)
  {
    if (superuser == FALSE) {
      fs_repmsg.fs_errcode=EACCES; put_inode(pinode1);
      return 0; }
  }

  /* zjisti, zda fs_name2 uz nahodou neexistuje */

  fetch_name((vir_bytes)fs_msg.fs_name2);
  if ((tpinode = name_i(user_path))!=NIL_INODE) {
	put_inode(tpinode);
	put_inode(pinode1);
	return EEXIST;
  }

  /* zjisti, zda je fs_name2 jmeno souboru nebo cela cesta */

  while (user_path[i] != END_S)
  {
    if (user_path[i] == SLASH) s_position=i;
    i++;
   }

  if (s_position == -1)		/* pouze jmeno */
  {
    if ((tpinode=name_i(user_path)) != NIL_INODE) {
      put_inode(tpinode); put_inode(pinode1);
      fs_repmsg.fs_errcode=EEXIST;
      return 0; /* jmeno uz existuje */
    }

    if (access(fp -> fp_workdir,S_IWRITE) != OK) {
      fs_repmsg.fs_errcode=EACCES; put_inode(pinode1);
      return 0; /* neni pravo zapisu do work dir */
    }

    if (pinode1 -> i_dev != (fp -> fp_workdir) -> i_dev) {
      fs_repmsg.fs_errcode=EXDEV; /* link mezi ruznymi zarizenimi nepovolit*/
      put_inode(pinode1);
      return 0;
    }

    pinode1 -> i_nlinks++;      /* zvyseni poctu linku */
    pinode1 -> i_dirt = DIRTY;
    /* pridani polozky do aktualniho adresare */
    add_entry(fp -> fp_workdir, pinode1 -> i_num, user_path);
    put_inode(pinode1);
  }
  else                   	/* cela cesta */
  {
    i=s_position+1;
    while((name2[i-(s_position+1)]=user_path[i]) != END_S) i++;
    user_path[s_position]=END_S;

    /* prevod adresarove casti cesty na inode adresare */
    dinode2 = name_i(user_path);

    if (dinode2 == NIL_INODE) {
     fs_repmsg.fs_errcode=ENOENT; /* nespravna cesta */
     put_inode(pinode1);
     return 0;
    }

    if (access(dinode2,S_IWRITE)!= OK) {
     fs_repmsg.fs_errcode=EACCES; /* zakazany pristup do ciloveho adresare */
     put_inode(pinode1); put_inode(dinode2);
     return 0;
    }

    if (((dinode2 -> i_mode)&I_TYPE) != I_DIRECTORY) {
      fs_repmsg.fs_errcode=ENOTDIR; /* cesta k cilovemu adresari neni adresar */
      put_inode(pinode1); put_inode(dinode2);
      return 0;
    }

    if (pinode1 -> i_dev != dinode2 -> i_dev) {
      fs_repmsg.fs_errcode=EXDEV; /* link mezi ruznymi zarizenimi nepovolit */
      put_inode(pinode1); put_inode(dinode2);
      return 0;
    }

    pinode1 -> i_nlinks++;	/* zvysi pocet linku */
    pinode1 -> i_dirt = DIRTY;

    /* pridani polozky do zadaneho adresare */
    add_entry(dinode2, pinode1 -> i_num, name2);
    put_inode(pinode1);
    put_inode(dinode2);
  }
  fs_repmsg.fs_errcode=OK;
  return 0;
}

/**************************************************************************\
**                                                                        **
**  int do_unlink()     	zrusi link                                **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_UNLINK				  **
**	    fs_name     	ukazatel na jmeno souboru                 **
**                                                                        **
**  vystup: fs_errcode     	chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_unlink()
{
  char name[NAME_SIZE];
  struct inode *pinode, *dinode;
  int i,s_position = -1;

  fetch_name((vir_bytes)fs_msg.fs_name);
  pinode = name_i(user_path); /* prevod na inode */
  if (pinode == NIL_INODE) { fs_repmsg.fs_errcode=ENOENT;
			     return 0; /* neni soubor nebo adresar */
  }

  if (access(pinode,S_IWRITE)!= OK) { fs_repmsg.fs_errcode=EACCES;
				      put_inode(pinode);
				      return 0; /* zakazany pristup */ }

  /* pokud se jedna o adresar, muze unlinkovat pouze super_user */
  if (((pinode ->i_mode)&I_TYPE) == I_DIRECTORY)
  {
    if (superuser == FALSE) { fs_repmsg.fs_errcode=EACCES;
			      put_inode(pinode);
			      return 0;}
  }

  /* zjisti, zda je fs_name jmeno souboru nebo  cela cesta */
  i=0;
  while (user_path[i] != END_S)
  {
    if (user_path[i] == SLASH) s_position=i;
    i++;
  }

  if (s_position == -1)          /* jen jmeno */
  {
    if (access(fp -> fp_workdir,S_IWRITE) != OK) {
      fs_repmsg.fs_errcode=EACCES; /* neni pravo zapisu (mazani) ve work dir */
      put_inode(pinode);
      return 0;
    }

    remove_entry(fp -> fp_workdir, user_path);
				/* odstrani polozku z aktualniho adresare */
    pinode -> i_nlinks--;	/* snizi pocet linku */
    pinode -> i_dirt = DIRTY;
    put_inode(pinode);
    fs_repmsg.fs_errcode=OK;
    return 0;
  }
  else			/* cela cesta */
  {
    i=s_position+1;
    while((name[i-(s_position+1)]=user_path[i]) != END_S) i++;
    user_path[s_position]=END_S;

    /* prevod adresarove casti cesty na inode adresare */
    dinode = name_i(user_path);
    /* P.G.: existence souboru drive overena, musi projit */

    if (access(dinode, S_IWRITE) != OK) { fs_repmsg.fs_errcode=EACCES;
					  put_inode(pinode); put_inode(dinode);
					  return 0; /* neni pravo mazani */}

    pinode -> i_nlinks--;	    	/* snizi pocet linku */
    pinode -> i_dirt = DIRTY;
    put_inode(pinode);
    remove_entry(dinode, name); /* odstrani polozku ze zadaneho adresare */
    put_inode(dinode);
  }
  fs_repmsg.fs_errcode=OK;
  return 0;
}

/**************************************************************************\
**                                                                        **
**  void truncate()     		odstrani vsechny bloky asociovane **
**					s inodem        		  **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  register struct inode *rip	ukazatel na inod, ktery bude      **
**         				odstranen                         **
**                                                                        **
**  vystup: 								  **
**                                                                        **
\**************************************************************************/

PUBLIC void truncate(register struct inode *rip)

{
  struct buf *pIndirectZone1,*pIndirectZone2;
  int	i,j;

  for (i = 0; i < NR_DZONE_NUM; i++)
  {
    if (rip->i_zone[i] != NO_ZONE) free_zone(rip->i_dev,rip->i_zone[i]);
  }

			/* indirect access to zone numbers */
  if (rip->i_zone[NR_DZONE_NUM] != NO_ZONE)
  {
    pIndirectZone1 = get_block(rip->i_dev,rip->i_zone[NR_DZONE_NUM],NORMAL);
    for (i = 0; i < NR_INDIRECTS; i++)
    {
      if (pIndirectZone1->b.b__ind[i] != NO_ZONE)
	free_zone(rip->i_dev,pIndirectZone1->b.b__ind[i]);
    }
    free_zone(rip->i_dev,rip->i_zone[NR_DZONE_NUM]);
  }

			/* double indirect access to zone numbers */
  if (rip->i_zone[NR_DZONE_NUM+1] != NO_ZONE)
  {
    pIndirectZone1 = get_block(rip->i_dev,rip->i_zone[NR_DZONE_NUM+1],NORMAL);
    for (i = 0; i < NR_INDIRECTS; i++)
    {
      if (pIndirectZone1->b.b__ind[i] != NO_ZONE)
      {
	pIndirectZone2 = get_block(rip->i_dev,pIndirectZone1->b.b__ind[i],NORMAL);
	for (j = 0; j < NR_INDIRECTS; j++)
	{
	  if (pIndirectZone2->b.b__ind[j] != NO_ZONE)
	    free_zone(rip->i_dev,pIndirectZone2->b.b__ind[j]);
	}
	free_zone(rip->i_dev,pIndirectZone1->b.b__ind[i]);
      }
    }
    free_zone(rip->i_dev,rip->i_zone[NR_DZONE_NUM+1]);
  }
  rip -> i_dirt = DIRTY;
  put_inode(rip);
}



PUBLIC int add_entry(struct inode *dInode, int Inum, char *name)
{
  /* P.G: Predpoklad: delka bloku je celociselnym nasobkem delky
	  adresarove polozky. */

  /* zjednoduseni: adresar jen v blocich 1. urovne */
  #define MAXNUMENTRIES (NR_DZONE_NUM*BLOCK_SIZE/sizeof(struct dir_struct))

  int last,i,j,nb;
  int entryno,numentries;

  struct buf *dirblk;


  if (dInode->i_size==0) { /* adresar zatim prazdny, alokuj novy blok */
    last=alloc_zone(dInode->i_dev);
    if (last==NO_ZONE) return FALSE;
    dirblk=get_block(dInode->i_dev, last, NO_READ);
    strcpy(dirblk->b.b__dir[0].d_name,name);
    dirblk->b.b__dir[0].d_inum=Inum;
    dirblk->b_dirt=DIRTY;
    put_block(dirblk, DIRECTORY_BLOCK);
    dInode->i_size+=sizeof(struct dir_struct);
    dInode->i_zone[0]=last;
    dInode->i_dirt=DIRTY;
    return TRUE;
  }

  last=(int ) (dInode->i_size / BLOCK_SIZE); /* posledni blok */

  entryno=0;
  numentries=dInode->i_size/sizeof(struct dir_struct);
  /* cyklus pres alokovane bloky adresare, hledame zrusenou polozku */

  for (i=0; i<=last; i++) {
   dirblk=get_block(dInode->i_dev, dInode->i_zone[i], NORMAL);
   for (j=0; j<BLOCK_SIZE/sizeof(struct dir_struct) && entryno<numentries;
	j++,entryno++) {
    if (dirblk->b.b__dir[j].d_inum==0) {
     /* nalezena prazdna polozka, pouzij */
     strcpy(dirblk->b.b__dir[j].d_name,name);
     dirblk->b.b__dir[j].d_inum=Inum;
     dirblk->b_dirt=DIRTY;
     put_block(dirblk, DIRECTORY_BLOCK);
     /* size netreba modifikovat */
     return TRUE;
    }
   }
   if (i<last) /* posledni blok zatim nevracej */
    put_block(dirblk, DIRECTORY_BLOCK);
  }

  /* prazdny slot nenalezen, musime pridat na konec */
  if (numentries>MAXNUMENTRIES) {
   put_block(dirblk, DIRECTORY_BLOCK);
   return FALSE;
  }

  if (j==BLOCK_SIZE/sizeof(struct dir_struct)) {
    /* je treba alokovat novy blok */
    put_block(dirblk, DIRECTORY_BLOCK);
    nb=alloc_zone(dInode->i_dev);
    if (nb==NO_ZONE) return FALSE;
    dirblk=get_block(dInode->i_dev, nb, NO_READ);
    strcpy(dirblk->b.b__dir[0].d_name,name);
    dirblk->b.b__dir[0].d_inum=Inum;
    dirblk->b_dirt=DIRTY;
    put_block(dirblk, DIRECTORY_BLOCK);
    dInode->i_size+=sizeof(struct dir_struct);
    dInode->i_zone[last+1]=nb;
    dInode->i_dirt=DIRTY;
    return TRUE;

  } else { /* staci pridat do stavajiciho bloku */
     strcpy(dirblk->b.b__dir[j].d_name,name);
     dirblk->b.b__dir[j].d_inum=Inum;
     dirblk->b_dirt=DIRTY;
     put_block(dirblk, DIRECTORY_BLOCK);
     dInode->i_size+=sizeof(struct dir_struct);
     dInode->i_dirt=DIRTY;
     return TRUE;
   }
}

PUBLIC int remove_entry(struct inode *dInode, char* name)
{
  /* P.G: Predpoklad: delka bloku je celociselnym nasobkem delky
	  adresarove polozky. */

  /* zjednoduseni: adresar jen v blocich 1. urovne. Adresar se nezmensuje,
		   polozky se pouze oznaciji jako zrusene */

  #define MAXNUMENTRIES (NR_DZONE_NUM*BLOCK_SIZE/sizeof(struct dir_struct))

  int last,i,j;
  int entryno,numentries;

  struct buf *dirblk;
  last=(int ) (dInode->i_size / BLOCK_SIZE); /* posledni blok */

  entryno=0;
  numentries=dInode->i_size/sizeof(struct dir_struct);
  /* cyklus pres alokovane bloky adresare, hledame polozku */
  for (i=0; i<=last; i++) {
   dirblk=get_block(dInode->i_dev, dInode->i_zone[i], NORMAL);
   for (j=0; j<BLOCK_SIZE/sizeof(struct dir_struct) && entryno<numentries;
	j++,entryno++) {
    if (strcmp(dirblk->b.b__dir[j].d_name,name)==0) {
     dirblk->b.b__dir[j].d_inum=0;
     dirblk->b.b__dir[j].d_name[0]=0;
     return TRUE;
    }
   }
   put_block(dirblk, DIRECTORY_BLOCK);
  }

  return FALSE;
}