/* MISC.C, Last update 17.1.98, created by D.Belicek & D.Spalt */

#define EXTERN extern

#include "common/const.h"
#include "fs/common/fsconst.h"
#include "common/type.h"
#include "fs/common/fsglo.h"
#include "common/errno.h"
#include "fs/common/fproc.h"
#include "fs/common/file.h"
#include "fs/common/fsmes.h"
#include "fs/inode/inode.h"
#include "fs/syscall/open.h"
#include "fs/common/fsmes.h"
#include "mm/mm2fsmsg.h"
#include "kernel/makra.h"
#include "test/trap.h"

#undef EXTERN

struct fproc_struct *cp;

PUBLIC int do_dup()

/* udela ja nevim co treba dopsat  */

{
  return 0;
}

/*===========================================================================*
 *                             do_fork()                                     *
 *===========================================================================*/
/* 
int do_fork()
  a, overit je-li opravdu volano od MM
  b, fproc[child]=fproc[parent]
  c, zvysit pocitadla v 'filp' tabulce - zdedili jsme soubory
  d, vyplnit nove id procesu
  e, root i working adresar maji dalsiho uzivatele
*/

PUBLIC int do_fork()
{
  int child,parent,i;

  child=fs_msg.fs_child;
  parent=fs_msg.fs_parent;

  if(who!=MM)
  {
    trap(TRAP_MMTOFS_NOTMM);
    fs_repmsg.fs_errcode=EACCES;
    return;
  }

  cp=&fproc[child];

  cp->fp_umask=fproc[parent].fp_umask;
  cp->fp_workdir=fproc[parent].fp_workdir;
  cp->fp_rootdir=fproc[parent].fp_rootdir;
  cp->fp_realuid=fproc[parent].fp_realuid;
  cp->fp_effuid=fproc[parent].fp_effuid;
  cp->fp_realgid=fproc[parent].fp_realgid;
  cp->fp_effgid=fproc[parent].fp_effgid;
  cp->fp_tty=fproc[parent].fp_tty;
  cp->fp_fd=fproc[parent].fp_fd;
  cp->fp_buffer=fproc[parent].fp_buffer;
  cp->fp_nbytes=fproc[parent].fp_nbytes;
  cp->fp_suspended=fproc[parent].fp_suspended;
  cp->fp_revived=fproc[parent].fp_revived;
  cp->fp_task=fproc[parent].fp_task;

  for(i=0;i<NR_FDS;i++) {
    cp->fp_filp[i]=fproc[parent].fp_filp[i];
    if(cp->fp_filp[i] != NIL_FILP) cp->fp_filp[i]->filp_count++;
  }

  dup_inode(cp->fp_rootdir);
  dup_inode(cp->fp_workdir);

  fs_repmsg.fs_errcode=OK;
  return;
}

/*===========================================================================*
 *                             do_exit                                       *
 *===========================================================================*/
 /*
  a, otestuje who - jestli to poslal MM
  b, opet zjistim id. cislo procesu
  c, pozaviram vsechny deskriptory
  d, uvolni rootdir a workdir
*/
PUBLIC int do_exit()
{
  int i;
  if(who!=MM)
  {
    trap(TRAP_MMTOFS_NOTMM);
    fs_repmsg.fs_errcode=EACCES;
    return;
  }
  cp=&fproc[fs_msg.fs_proc];

  for(i=0;i<NR_FDS;i++)
    if(cp->fp_filp[i] != NIL_FILP)
    {
      fs_msg.fs_fd=i;
      do_close();
    }

  put_inode(cp->fp_rootdir);
  put_inode(cp->fp_workdir);

  fs_repmsg.fs_errcode=OK;
  return;
}


/*===========================================================================*
 *                              do_sync                                      *
 *===========================================================================*/
PUBLIC int do_sync()
{
  /* Perform the sync() system call.  Flush all the tables. */

  struct inode* ip;
  struct super_block* sp;

  /* The order in which the various tables are flushed is critical.  The
   * blocks must be flushed last, since rw_inode() leaves its results in
   * the block cache.
   */

  /* Write all the dirty inodes to the disk. */
  for (ip=&inodes[0]; ip<&inodes[NR_INODES]; ip++)
   if (ip->i_count>0 && ip->i_dirt==DIRTY) rw_inode(ip,WRITING);

  /* Write all the dirty superblocks to the disk. */
  for (sp=&super_blocks[0]; sp<&super_blocks[NR_SUPERS]; sp++)
   if (sp->s_dev != NO_DEV && sp->s_dirt==DIRTY) rw_super(sp,WRITING);

  /* Write all the dirty blocks to the disk */
  return sync();
}

