	    /**********************************************\
            **                                            **
	    **  TUOX                                      **
            **                                            **
	    **  File System - OPEN.C                      **
	    **                                            **
	    **  Prepracovali David Spalt, L94607          **
	    **		     Daniel Belicek               **
	    **                                            **
	    \**********************************************/

/**************************************************************************\
**  int do_creat()      vytvori soubor                                    **
**  int do_open()       otevre soubor                                     **
**  int do_mknod()      vytvori specialni soubor                          **
**  int do_close()      zavre soubor                                      **
**  int do_lseek()      zmeni aktualni pozici v souboru                   **
**  struct inode *new_node()	alokuje novy inod                         **
\**************************************************************************/
/*Dodal Karel. je treba zjistit co s tim , kde ap.*/
#define READING_WRITING 1

#define EXTERN extern

#include "common/const.h"
#include "fs/common/fsconst.h"
#include "common/type.h"
#include "fs/common/fsglo.h"
#include "common/errno.h"
#include "fs/common/file.h"
#include "common/stat.h"
#include "fs/common/fstype.h"
#include "fs/common/fproc.h"
#include "fs/inode/inode.h"
#include "fs/mount/mount.h"
#include "fs/common/fsmes.h"
#include "fs/device/device.h"
#include "fs/syscall/protect.h"
#include "fs/syscall/link.h"
#include "fs/syscall/open.h"
#include "fs/main.h"

#undef EXTERN

int chyba;	//promena, ktera urcuje jaky typ errcode nastal
		//protoze name_i vraci pouze NIL_INODE a z toho
		//se nic nezjisti

struct inode *new_node(char *path, mode_t bits, zone_t z0);

/**************************************************************************\
**                                                                        **
**  int do_creat()      	vytvori soubor                            **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CREAT					  **
**	    fs_name     	ukazatel na jmeno souboru                 **
**	    fs_mode		flags souboru				  **
**			        I_REGULAR/I_DIRECTORY, rwxrwxrwx          **
**                                                                        **
**  vystup: fs_fd		file desctiptor nebo chybovy kod          **
**                                                                        **
\**************************************************************************/

PUBLIC int do_creat()
{
  struct inode *pinode;
  int i, filp_p, fd;

  filp_p=fd=-1;

  for (i=0; i < NR_FDS && fd == -1; i++)
  {
    if (fp->fp_filp[i] == NULL) fd=i;
  }
/* hledani volne pozice file descriptoru */

  if (fd == -1) { fs_repmsg.fs_fd=EMFILE; return EMFILE;}

/* volny slot nenalezen */

  for (i=0; i < NR_FILPS && filp_p == -1; i++)
  {
    if (filp[i].filp_count == 0) filp_p=i;
  }
/* hledani volne pozice pro filp */

  if (filp_p == -1) { fs_repmsg.fs_fd=EMFILE; return EMFILE;}
/* volny slot nenalezen */

  fetch_name((vir_bytes)fs_msg.fs_name);

  if ((Word)(fs_msg.fs_mode & I_TYPE) == I_REGULAR || (fs_msg.fs_mode & I_TYPE) == I_DIRECTORY)
  {
    pinode = new_node(user_path, fs_msg.fs_mode & fp -> fp_umask, NO_ZONE);
/* vytvoreni noveho inodu */
  }
  else
  {
    fs_repmsg.fs_fd=ENOENT;
    return ENOENT;
/* nejde o regularni soubor nebo adresar */
  }

  if (pinode==NIL_INODE) {
   if (chyba == EACCES) { fs_repmsg.fs_fd=EACCES; return EACCES; }
    /* pristup nepovolen */

   if (chyba == ENOTDIR) { fs_repmsg.fs_fd=ENOTDIR; return ENOTDIR; }
    /* pravdepodobne nespravna cesta */

   if (chyba == EEXIST) { /* soubor jiz existoval */
     pinode = name_i(user_path);
     switch (pinode -> i_mode & I_TYPE) {
      case I_REGULAR:
	 /* over pravo WRITE (zda umoznit truncate) */

	 if ((i=forbidden(pinode,S_IWRITE,0))==0) truncate(pinode);
	  else {
	   put_inode(pinode);
	   fs_repmsg.fs_fd=i;
	   return i;
	  }
	 break;
      case I_DIRECTORY:
	 put_inode(pinode);
	 fs_repmsg.fs_fd=EISDIR;
	 return EISDIR;
      case I_CHAR_SPECIAL:
      case I_BLOCK_SPECIAL:
	 put_inode(pinode);
	 fs_repmsg.fs_fd=EEXIST;
	 return EEXIST;
     }
   }
  }

  filp[filp_p].filp_count=1;
  filp[filp_p].filp_mode = S_IWRITE;
  filp[filp_p].filp_pos = 0; /* nastaveni pozice v souboru na zacatek */
  filp[filp_p].filp_ino = pinode; /* prirazeni inodu polozce ve filp */

  fp -> fp_filp[fd] = &filp[filp_p]; /* prirazeni filp file descriptoru */

  fs_repmsg.fs_fd=fd;
  return fd; /* navrat file descriptoru */
}

/**************************************************************************\
**                                                                        **
**  int do_mknod()      	vytvori specialni soubor                  **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_MKNOD					  **
**	    fs_name		ukazatel na jmeno specialniho souboru     **
**	    fs_mode		prava souboru (rwxrwxrwx)		  **
**	    fs_major_minor	major a minor device number		  **
**                                                                        **
**  vystup: fs_errcode		chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_mknod()
{
  if (superuser == TRUE) /* Volat smi jen superuser*/
  {
    fetch_name((vir_bytes)fs_msg.fs_name);
    put_inode(new_node(user_path,fs_msg.fs_mode & fp->fp_umask,fs_msg.fs_major_minor));
    fs_repmsg.fs_errcode=OK;
    return OK;
  }
  else { fs_repmsg.fs_errcode=EPERM; return EPERM;}
}

/**************************************************************************\
**                                                                        **
**  struct inode *new_node()	alokuje novy inod                         **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  char *name		ukazatel na jmeno souboru                 **
**          mode_t bits		mod noveho inodu                          **
**	    zone_t z0		zona cislo 0 pro novy inod                **
**				(pouzivano pro zarizeni)		  **
**                      						  **
**  vystup: struct inode pinode	novy inod nebo chybovy kod                **
**                                                                        **
\**************************************************************************/

PRIVATE struct inode *new_node(char *path, mode_t bits, zone_t z0)
{
  char name[NAME_SIZE];
  struct inode *pinode,*dinode;
  int i=0,s_position = -1;
  char* n;

  while (path[i] != END_S)
  {
    if (path[i] == SLASH) s_position=i;
    i++;
  }
/* zjisteni, zda je path jmeno souboru nebo cela cesta */

  if (s_position == -1)
   /* jen jmeno */
  {
    pinode = name_i(path); /* prevod jmena na inode */

    if (pinode != NIL_INODE)
    { /* inode jiz existoval */
      put_inode(pinode);
      chyba= EEXIST;
      return NIL_INODE;
    }

    if (access(fp -> fp_workdir,S_IWRITE)!= OK)
    { /* pristup nepovolen */
      chyba= EACCES;
      return NIL_INODE;
    }

    pinode = alloc_inode((fp -> fp_workdir) -> i_dev, bits);
    /* alokace noveho inodu */

    pinode -> i_count = 1;
    pinode -> i_zone[0] = z0;
    pinode -> i_dirt = DIRTY;
    /* put_inode(pinode); - az pri close() */
    add_entry(fp -> fp_workdir, pinode -> i_num, path);
     /* pridani polozky do aktualniho adresare */

    return pinode;
  }
  else
  {
    /* cela cesta */

    /* P.G.: otestuj duplicitu jmena - JE TO OTRESNE, cesta se traverzuje
	     dvakrat, ale aspon je to citelne -> nekdy prepsat */
    pinode = name_i(path); /* prevod jmena na inode */

    if (pinode != NIL_INODE)
    { /* inode jiz existoval */
      put_inode(pinode);
      chyba= EEXIST;
      return NIL_INODE;
    }

    i=s_position+1;
    n=name;
    while((*n++=path[i]) != END_S) i++;
    path[s_position]=END_S;
    dinode = name_i(path); /* prevod cesty na inode adresare */

    if (dinode == NIL_INODE)
    {
      chyba=ENOTDIR;
      return NIL_INODE;
    }

    if (access(dinode,S_IWRITE)!= OK)
    { /* pristup do adresare nepovolen */
      put_inode(dinode);
      chyba=EACCES;
      return NIL_INODE;
    }

    if (((dinode -> i_mode)&I_TYPE) != I_DIRECTORY)
    { /* nejedna se o adresar */
      put_inode(dinode);
      chyba = ENOTDIR;
      return NIL_INODE;
    }

    pinode = alloc_inode(dinode -> i_dev, bits); /* alokace noveho inodu */

    pinode -> i_count = 1;
    pinode -> i_zone[0] = z0;
    pinode -> i_dirt = DIRTY;
   /* put_inode(pinode); - az pri close() */

    add_entry(dinode, pinode -> i_num, name);
      /* pridani polozky do zadaneho adresare */

    return pinode;
  }
}

/**************************************************************************\
**                                                                        **
**  int do_open()      		otevre soubor                             **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_OPEN					  **
**	    fs_name		ukazatel na jmeno souboru                 **
**          fs_mode     	S_IREAD | S_IWRITE                        **
**                                                                        **
**  vystup: fs_fd		file descriptor nebo chybovy kod          **
**                                                                        **
\**************************************************************************/

PUBLIC int do_open()
{
  struct inode *pinode;
  int i, filp_p, fd;

  filp_p=fd=-1;

  for (i=0; i < NR_FDS && fd == -1; i++)
  {
    if (fp -> fp_filp[i] == NULL) fd=i;
  }
/* hledani volne pozice file descriptoru */

  if (fd == -1) {fs_repmsg.fs_fd=EMFILE;
		 return EMFILE;}
/* volny slot nenalezen */

  for (i=0; i < NR_FILPS && filp_p == -1; i++)
  {
    if (filp[i].filp_count == 0) filp_p=i;
  }
/* hledani volne pozice pro filp */

  if (filp_p == -1) {fs_repmsg.fs_fd=EMFILE;
		     return EMFILE;}
/* volny slot nenalezen */

  fetch_name((vir_bytes)fs_msg.fs_name);

  pinode = name_i(user_path);
  if (pinode == NIL_INODE) {fs_repmsg.fs_fd=ENOENT;
			    return ENOENT;}

/* NEVIM, NA CO BY TO MELO BYT DOBRE - POZUSTATEK Z CREAT() ? P.G.
  if (access(pinode,S_IWRITE)!= OK) {fs_repmsg.fs_fd=EACCES;
				     return EACCESS;}
  if (((pinode -> i_mode)&I_TYPE) != I_DIRECTORY) {fs_repmsg.fs_fd=ENOTDIR;
							return ENOTDIR;}
*/
  if (access(pinode, fs_msg.fs_mode) != OK) {fs_repmsg.fs_fd=EACCES;
					     return EACCES;}
  switch ((pinode -> i_mode)&I_TYPE)
  {
    case I_DIRECTORY:		if (fs_msg.fs_mode == WRITING)
				{
				fs_repmsg.fs_fd=EISDIR;
				return EISDIR;
				/* jedna se o adresar */
				}
				else break;

    case I_CHAR_SPECIAL:        if (fp -> fp_tty == NO_DEV) fp -> fp_tty = pinode -> i_dev;
				dev_open(pinode -> i_dev, fs_msg.fs_mode);
				break;

    case I_BLOCK_SPECIAL:       dev_open(pinode -> i_dev, fs_msg.fs_mode);
				break;

  }
  pinode -> i_count++;		/* byl zvysen pocet odkazu na inod */
  pinode -> i_dirt = DIRTY;
/*  put_inode(pinode); - musi zustat v tabulce */

  filp[filp_p].filp_count++;
  filp[filp_p].filp_mode = fs_msg.fs_mode;
  filp[filp_p].filp_pos = 0;
  filp[filp_p].filp_ino = pinode; 	/* prirazeni inodu polozce ve filp */
  fp ->fp_filp[fd] = &filp[filp_p]; 	/* prirazeni filp file descriptoru */
  fs_repmsg.fs_fd=fd;
  return fd;

}

/**************************************************************************\
**                                                                        **
**  int do_close()      	zavreni souboru                           **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CLOSE					  **
**	    fs_fd		file descriptor                           **
**                                                                        **
**  vystup: fs_errcode		chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_close()
{
  struct inode *pinode;

  if(fp -> fp_filp[fs_msg.fs_fd] == NULL) {fs_repmsg.fs_errcode=EBADF;
					   return 0;}
/* spatny file descriptor */

  pinode=((fp -> fp_filp[fs_msg.fs_fd]) -> filp_ino);
  switch ((pinode -> i_mode)&I_TYPE)
  {
    case I_CHAR_SPECIAL:	fs_repmsg.fs_errcode=dev_close(pinode -> i_dev);
				return 0;

    case I_BLOCK_SPECIAL:       flushall(pinode -> i_dev);
				fs_repmsg.fs_errcode=dev_close(pinode -> i_dev);
				return 0;

    default:                    (fp -> fp_filp[fs_msg.fs_fd]) -> filp_count--;
					/* snizeni poctu fd ukazujicich */
					/*na slot ve filp */
				fp -> fp_filp[fs_msg.fs_fd]=NULL;
					/* nastaveni fd na NULL */

				 pinode -> i_dirt = DIRTY;
				 put_inode(pinode);
				 fs_repmsg.fs_errcode=OK;
				 return 0;
  }

}

/**************************************************************************\
**                                                                        **
**  int do_lseek()      	zmeni aktualni pozici v souboru           **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_LSEEK					  **
**	    fs_fd1     		file descriptor                           **
**	    fs_offset1		velikost posunu				  **
**	    fs_whence		vztazny bod posunu			  **
**                                                                        **
**  vystup: fs_pos1		nova pozice v souboru                     **
**                                                                        **
\**************************************************************************/

PUBLIC int do_lseek()
{
  if(fp -> fp_filp[fs_msg.fs_fd1] == NULL) return EBADF;
  switch (fs_msg.fs_whence)
  {
    case 0:	fs_repmsg.fs_pos1=(fp -> fp_filp[fs_msg.fs_fd1]) -> filp_pos = fs_msg.fs_offset1;
		break;

    case 1:	fs_repmsg.fs_pos1=(fp -> fp_filp[fs_msg.fs_fd1]) -> filp_pos += fs_msg.fs_offset1;
		break;

    case 2:	fs_repmsg.fs_pos1=(fp -> fp_filp[fs_msg.fs_fd1]) -> filp_pos =
			 ((fp -> fp_filp[fs_msg.fs_fd1]) -> filp_ino) -> i_size - fs_msg.fs_offset1;
		break;

    default:    fs_repmsg.fs_pos1=EINVAL;
		return 0;
  }
  return 1;
}

