	    /**********************************************\
            **                                            **
            **  TUOX                                      **
            **                                            **
            **  File System - PROTECT.C                   **
            **                                            **
	    **  Stanislav Sikora, L92053                  **
	    **  Prepracovali David Spalt, L94607	  **
	    **               Daniel Belicek               **
	    **                                            **
	    \**********************************************/

/**************************************************************************\
**  int do_chmod()      zmena prav souboru                                **
**  int do_chown()      zmena vlastnika souboru                           **
**  int do_umask()      zmena defaultni masky prav                        **
**  int do_access()     test prav souboru                                 **
**  int forbidden()     test prav souboru                                 **
**  int readonly()      test prav souboru                                 **
\**************************************************************************/

#define EXTERN extern

#include "common/const.h"
#include "fs/common/fsconst.h"
#include "common/com.h"
#include "common/type.h"
#include "common/errno.h"
#include "common/stat.h"
#include "fs/common/fsglo.h"
#include "fs/common/fproc.h"
#include "fs/common/fsmes.h"
#include "fs/inode/super.h"
#include "fs/common/file.h"
#include "fs/inode/inode.h"
#include "fs/main.h"
#include "fs/syscall/protect.h"
#include "fs/mount/mount.h"

#undef EXTERN

/**************************************************************************\
**                                                                        **
**  int do_chmod()      	zmena prav souboru                        **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CHMOD					  **
**	    fs_name    		ukazatel na jmeno souboru                 **
**          fs_mode     	nova maska souboru                        **
**                                                                        **
**  vystup: fs_errcode      	chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_chmod()
{
  struct inode *pinode;

  fetch_name((vir_bytes) fs_msg.fs_name);
  if( (pinode=name_i(user_path))==NIL_INODE) {fs_repmsg.fs_errcode=ENOENT;
					      return 0;}

  if (read_only(pinode)==OK &&
     (pinode->i_uid==((struct fproc_struct*) fp)->fp_effuid || superuser ))
  {
    pinode->i_mode = fs_msg.fs_mode;
    pinode->i_dirt=DIRTY;
    fs_repmsg.fs_errcode=OK;
    return 0;
  }
  else {fs_repmsg.fs_errcode=EACCES;
	return 0;}
}

/**************************************************************************\
**                                                                        **
**  int do_chown()      	zmena vlastnika souboru                   **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CHOWN					  **
**	    fs_name     	ukazatel na jmeno souboru                 **
**          fs_owner     	identifikacni cislo uzivatele uid         **
**          fs_group     	identifikacni cislo skupiny gid           **
**                                                                        **
**  vystup: fs_errcode		chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_chown()
{
  struct inode *pinode;

  fetch_name((vir_bytes)fs_msg.fs_name);
  if( (pinode=name_i(user_path))==NIL_INODE) {fs_repmsg.fs_errcode=ENOENT;
					      return 0;}

  if ( read_only(pinode)==OK && superuser )
  {
    pinode->i_uid = fs_msg.fs_owner;
    pinode->i_gid = fs_msg.fs_group;
    pinode->i_dirt= DIRTY;
    fs_repmsg.fs_errcode=OK;
    return 0;
  }
  else {fs_repmsg.fs_errcode=EACCES;
	return 0;}
}

/**************************************************************************\
**                                                                        **
**  int do_umask()      	zmena defaultni masky prav                **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_UMASK					  **
**	    fs_mode     	nova defaultni maska                      **
**                                                                        **
**  vystup: fs_oldmode		stara maska                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_umask()
{
  int um;

  um=((struct fproc_struct*)fp)->fp_umask;
  ((struct fproc_struct*)fp)->fp_umask=fs_msg.fs_mode;

  fs_repmsg.fs_oldmode=um;
  return 0;
}

/**************************************************************************\
**                                                                        **
**  int do_access()     	test prav souboru                         **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_ACCESS				  **
**	    fs_name     	ukazatel na jmeno souboru                 **
**          fs_mode     	testovana maska prav souboru              **
**                                                                        **
**  vystup: fs_errcode      	chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_access()
{
  fetch_name((vir_bytes)fs_msg.fs_name);
  fs_repmsg.fs_errcode=access(name_i(user_path),fs_msg.fs_mode);
  return 0;
}

PUBLIC int access(struct inode *pinode,mode_t mask /* S_IREAD etc. */)
{
  if (pinode==NIL_INODE) return ENOENT;
  return forbidden(pinode,mask,0);
}

/**************************************************************************\
**                                                                        **
**  int forbidden()     	test prav souboru                         **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  register struct inode *rip         ukazatel na inode          **
**          mode_t access_desired  	testovana maska prav souboru      **
**          int real_uid        	testovat na tento uid,jinak z *fp **
**                                                                        **
**  vystup: int errcode      		chybovy kod                       **
**                                                                        **
\**************************************************************************/
PUBLIC int forbidden(rip, access_desired, real_uid)

register struct inode *rip;
mode_t access_desired; /*  S_IREAD | S_IWRITE | S_IEXEC */
int real_uid;

{
 uid_t uid=real_uid ? real_uid : fp->fp_effuid;

 /* root ? */
 if (uid==SU_UID) {
  /* only check for executability, if needed */
  if ((access_desired & S_IEXEC) && ((rip->i_mode & (UX_BIT | GX_BIT | OX_BIT))==0))
   return EACCES;
  return 0;
 }
 /* owner ? */
 if (uid==rip->i_uid && ((rip->i_mode>>6)&access_desired)!=access_desired)
  return EACCES;
 /* group ? */
 if (fp->fp_effgid==rip->i_gid && ((rip->i_mode>>3)&access_desired)!=access_desired)
    return EACCES;
 /* other */
 if (rip->i_mode&access_desired!=access_desired)
    return EACCES;
 return 0;

 /*
  if (((rip->i_uid==((struct fproc_struct*)fp)->fp_effuid ||
	real_uid==((struct fproc_struct*)fp)->fp_realuid) &&
	(rip->i_mode&access_desired)==access_desired ||
	(superuser &&
	(!(access_desired&(OX_BIT|GX_BIT|UX_BIT)) ||
	rip->i_mode&(OX_BIT|GX_BIT|UX_BIT)))) &&
	(access_desired&(OW_BIT|GW_BIT|UW_BIT) && read_only(rip)==OK)) return OK;
   else
 */
}


/**************************************************************************\
**                                                                        **
**  int readonly()      	test prav souboru                         **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  struct inode *ip 	ukayatel na inode                         **
**                                                                        **
**  vystup: int errcode       	chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int read_only(ip)

struct inode *ip;

{
  struct super_block *psb;

  psb=get_super(ip->i_dev);
  if( psb->s_rd_only==1) return EROFS;
  else return OK;

}

