            /**********************************************\
            **                                            **
            **  TUOX                                      **
            **                                            **
            **  File System - PROTECT.H                   **
            **                                            **
	    **  Stanislav Sikora, L92053                  **
	    **	Prepracoval David Spalt, L94607		  **
            **                                            **
            \**********************************************/

/**************************************************************************\
**  int do_chmod()      zmena prav souboru                                **
**  int do_chown()      zmena vlastnika souboru                           **
**  int do_umask()      zmena defaultni masky prav                        **
**  int do_access()     test prav souboru                                 **
**  int forbidden()     test prav souboru                                 **
**  int readonly()      test prav souboru                                 **
\**************************************************************************/

#include "fs/inode/super.h"
#include "fs/inode/inode.h"


/**************************************************************************\
**                                                                        **
**  int do_chmod()      zmena prav souboru                                **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_CHMOD                                           **
**          fs_name     pointer na nazev souboru                          **
**          fs_mode     maska                                             **
**                                                                        **
**  vystup: fs_errcode  chybovy kod                                       **
**                                                                        **
\**************************************************************************/
PUBLIC int do_chmod();


/**************************************************************************\
**                                                                        **
**  int do_chown()      zmena vlastnika souboru                           **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_CHOWN                                           **
**          fs_name     pointer na nazev souboru                          **
**          fs_owner    identifikacni cislo uzivatele uid                 **
**          fs_group    identifikacni cislo skupiny gid                   **
**                                                                        **
**  vystup: fs_errcode  chybovy kod                                       **
**                                                                        **
\**************************************************************************/
PUBLIC int do_chown();


/**************************************************************************\
**                                                                        **
**  int do_umask()      zmena defaultni masky prav                        **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_UMASK                                           **
**          fs_mode     nova maska                                        **
**                                                                        **
**  vystup: fs_oldmode  stara maska                                       **
**                                                                        **
\**************************************************************************/
PUBLIC int do_umask();


/**************************************************************************\
**                                                                        **
**  int do_access()     test prav souboru                                 **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_ACCESS                                          **
**          fs_name     pointer na nazev souboru                          **
**          fs_mode     testovana maska prav souboru                      **
**                                                                        **
**  vystup: fs_errcode  chybovy kod                                       **
**                                                                        **
\**************************************************************************/
PUBLIC int do_access();

PUBLIC int access(struct inode *,mode_t);

/**************************************************************************\
**                                                                        **
**  int forbidden()     test prav souboru                                 **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  rip             pointer na inode                              **
**          access_desired  testovana maska prav souboru                  **
**          real_uid        realne ident c. uzivatele                     **
**                                                                        **
**  vystup: EACCES      prava nejsou splnena                              **
**          OK          prava jsou splnena                                **
**                                                                        **
\**************************************************************************/
PUBLIC int forbidden(
    register struct inode *rip,
    mode_t access_desired,
    int real_uid
    );


/**************************************************************************\
**                                                                        **
**  int readonly()      test prav souboru                                 **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  ip          pointer na inode                                  **
**                                                                        **
**  vystup: EROFS       FS je read only                                   **
**          OK          FS neni read only                                 **
**                                                                        **
\**************************************************************************/
PUBLIC int read_only(
    struct inode *ip
    );
