            /**********************************************\
            **                                            **
            **  TUOX                                      **
            **                                            **
            **  File System - STADIR.C                    **
            **                                            **
	    **  Stanislav Sikora, L92053                  **
	    **  Prepracovali David Spalt, L94607          **
	    **		     Daniel Belicek               **
            **                                            **
            \**********************************************/

/**************************************************************************\
**  int do_chdir()      zmena adresare                                    **
**  int do_chroot()     zmena korenoveho adresare                         **
**  int do_stat()       vyplnuje struct stat podle jmena souboru          **
**  int do_fstat()      vyplnuje struct stat podle cisla inodu            **
**  int stat_inode()    vyplnuje struct stat podle file descriptor-u      **
**                      volana do_stat() a do_fstat()                     **
**  int change()        meni adresar                                      **
**                      volana do_chdir() a do_chroot()                   **
\**************************************************************************/

#define EXTERN extern

#include "fs/common/fsconst.h"
#include "common/const.h"
#include "common/type.h"
#include "common/stat.h"
#include "common/errno.h"
#include "fs/common/fsglo.h"
#include "fs/main.h"
#include "fs/inode/inode.h"
#include "fs/mount/mount.h"
#include "fs/common/fsmes.h"
#include "fs/syscall/stadir.h"
#include "fs/inode/read.h"
#include "mm/mm2fsmsg.h"
#include "test/trap.h"
#include "kernel/makra.h"

#undef EXTERN

PRIVATE int stat_inode(
    register struct inode *rip,
 //   struct struct_filp *fil_ptr,
    char *user_addr
    );

PRIVATE int change(
    struct inode **iip,
    char *name_ptr
    );


PUBLIC int do_setuid()
{
  /*
     Pride message od MM, v ni ze mame zmenit uid
     v nasi tabulce procesu u nejakeho procesu.
     Funkce predpoklada, ze vsechny nalezitosti jsou OK
     a ze kdyby byla chyba tak uz ji vypise MM.
     Karel
  */

  if (who == MM)
  {
    fproc[fs_msg.fs_proc].fp_effuid = fs_msg.fs_value_eff;
    fproc[fs_msg.fs_proc].fp_realuid = fs_msg.fs_value_real;

/*
    Tyhle testy snad nejsou potrebne

    int uid = fs_msg.fs_value; // na co zmenit
    int nc = fs_msg.fs_proc; // ktery proces zada o SETUID

    if (fproc[nc].fp_effuid == SUPER_USER)
    {
      fproc[nc].fp_effuid = uid;
      fproc[nc].fp_realuid = uid;
    }
    else
      if (uid == fproc[nc].fp_realuid)
	fproc[nc].fp_effuid = uid;
*/
  } else { trap(TRAP_MMTOFS_NOTMM); fs_repmsg.fs_errcode=-1; return -1; }

  return 0;
}

PUBLIC int do_setgid()
{
  /*
     Pride message od MM, v ni ze mame zmenit gid
     v nasi tabulce procesu u nejakeho procesu.
     Funkce predpoklada, ze vsechny nalezitosti jsou OK
     a ze kdyby byla chyba tak uz ji vypise MM.
     Karel
  */
  if (who == MM)  // only memory manager can call this
  {
    fproc[fs_msg.fs_proc].fp_effgid = fs_msg.fs_value_eff;
    fproc[fs_msg.fs_proc].fp_realgid = fs_msg.fs_value_real;

/*
    int gid = fs_msg.m1_i1; // na co zmenit
    int nc = fs_msg.m1_i2; // ktery proces zada o SETGID
    if (fproc[nc].fp_effuid == SUPER_USER)
    {
      fproc[nc].fp_effgid = gid;
      fproc[nc].fp_realgid = gid;
    }
    else
      if (gid == fproc[nc].fp_realgid)
	fproc[nc].fp_effgid = gid;
*/
  } else { trap(TRAP_MMTOFS_NOTMM); fs_repmsg.fs_errcode=-1; return -1; }
  return 0;
}

/**************************************************************************\
**                                                                        **
**  int do_chdir()      	zmena adresare                            **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CHDIR					  **
**	    fs_name 		ukazatel na jmeno noveho prac. adresare   **
**                                                                        **
**  vystup: fs_errcode		chybovy kod                               **
**                                                                        **
\**************************************************************************/
PUBLIC int do_chdir()

{
  fetch_name((vir_bytes)fs_msg.fs_name);
  fs_repmsg.fs_errcode=change(&( ( (struct fproc_struct*) fp)->fp_workdir), user_path);
  return 0;

}


/**************************************************************************\
**                                                                        **
**  int do_chroot()     	zmena korenoveho adresare                 **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_CHROOT				  **
**	    fs_name     	ukazatel na jmeno noveho kor. adresare    **
**                                                                        **
**  vystup: fs_errcode      	chybovy kod                               **
**                                                                        **
\**************************************************************************/

PUBLIC int do_chroot()

{
  if (superuser)
  {
    fetch_name((vir_bytes)fs_msg.fs_name);
    fs_repmsg.fs_errcode=change(&( (struct fproc_struct*) fp)->fp_rootdir, user_path);
    return 0;
  }
  else {fs_repmsg.fs_errcode=EACCES;
	return 0;}

}


/**************************************************************************\
**                                                                        **
**  int change()        	meni adresar                              **
**                      	volana do_chdir() a do_chroot()           **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  struct inode **iip  ukazatel na ukazatel inodu                **
**          char *name_ptr    	ukazatel na nazev adresare                **
**                                                                        **
**  vystup: int errcode      	chybovy kod                               **
**                                                                        **
\**************************************************************************/
PRIVATE int change(iip, name_ptr)

struct inode **iip;
char *name_ptr;

{
  struct inode *pinode;

  if( (pinode=name_i(name_ptr))==NIL_INODE) return ENOENT;

  if( pinode->i_mode&I_DIRECTORY)
    if((pinode->i_uid==((struct fproc_struct*) fp)->fp_effuid &&
      pinode->i_mode&UX_BIT) ||
      (pinode->i_gid==( (struct fproc_struct*) fp)->fp_effgid &&
      pinode->i_mode&GX_BIT) ||
      (pinode->i_mode&OX_BIT) )
    {
      *iip=pinode;
      return OK;
    }
    else return EACCES;
  else return ENOTDIR;

}


/**************************************************************************\
**                                                                        **
**  int do_stat()       vyplnuje struct stat podle jmena souboru          **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type		C_STAT					  **
**	    fs_name     	ukazatel na nazev souboru                 **
**          fs_buff     	ukazatel na vyst. bufer (struct stat)     **
**                                                                        **
**  vystup: fs_errcode      	chybovy kod                               **
**          return rw_user()                                              **
**                                                                        **
\**************************************************************************/

PUBLIC int do_stat()

{
  struct inode *pinode;

  fetch_name((vir_bytes)fs_msg.fs_name);

  if ( (pinode=name_i(user_path)) == NIL_INODE ) {fs_repmsg.fs_errcode=ENOENT;
						  return 0;}

  fs_repmsg.fs_errcode=stat_inode(pinode, fs_msg.fs_buff);
  return 0;
}


/**************************************************************************\
**                                                                        **
**  int do_fstat()      vyplnuje struct stat podle cisla inodu            **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_FSTAT						  **
**	    fs_fd     	file descriptor                                   **
**          fs_buff     ukazatel na vyst. bufer (struct stat)             **
**                                                                        **
**  vystup: return rw_user()                                              **
**                                                                        **
\**************************************************************************/

PUBLIC int do_fstat()

{
  struct inode *pinode;
  struct struct_filp *pfilp;

  pfilp=(( (struct fproc_struct*) fp)->fp_filp)[fs_msg.fs_fd];
  pinode=pfilp->filp_ino;

  fs_repmsg.fs_errcode=stat_inode(pinode, fs_msg.fs_buff);
  return 0;
}


/**************************************************************************\
**                                                                        **
**  int stat_inode()    vyplnuje struct stat podle file descriptor-u      **
**                      volana do_stat() a do_fstat()                     **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  register struct inode *rip 		ukazatel na inode         **
**          struct struct_filp *fil_ptr     	ukazatel na filp          **
**          char *user_addr   			ukazatel na vyst. bufer   **
**                                                                        **
**  vystup: return rw_user()                                              **
**                                                                        **
\**************************************************************************/

PRIVATE int stat_inode(register struct inode *rip,char *user_addr)

//register struct inode *rip; /* pointer to inode to stat */
//struct struct_filp *fil_ptr; /* filp pointer, supplied by 'fstat' */
//char *user_addr; /* user space address where stat buf goes */

{
  struct stat s;

  s.st_dev=rip->i_dev;
  s.st_ino=rip->i_num;
  s.st_mode=rip->i_mode;
  s.st_nlink=rip->i_nlinks;
  s.st_uid=rip->i_uid;
  s.st_gid=rip->i_gid;
  s.st_size=rip->i_size;
  s.st_atime=s.st_mtime=s.st_ctime=rip->i_mtime;

  return rw_user(D,who,user_addr,
    (vir_bytes) sizeof(struct stat),(char *) &s,TO_USER);
}
