	    /**********************************************\
            **                                            **
            **  TUOX                                      **
            **                                            **
            **  File System - TIME.C                      **
            **                                            **
	    **  Stanislav Sikora, L92053                  **
	    **  Prepracovali David Spalt, L94607          **
            **		     Daniel Belicek               **
	    **                                            **
	    \**********************************************/

/**************************************************************************\
**  int do_utime()      meni cas v souboru                                **
**  int do_time()       zjistuje aktualni cas                             **
**  int do_stime()      nastavuje cas                                     **
**  int do_tims()       zjistuje casy procesu                             **
\**************************************************************************/

#define EXTERN extern
#include "kernel/makra.h"
#include "common/const.h"
#include "fs/common/fsconst.h"
#include "common/com.h"
#include "common/type.h"
#include "common/errno.h"
#include "fs/common/fsglo.h"
#include "fs/common/fproc.h"
#include "fs/common/file.h"
#include "fs/main.h"
#include "fs/inode/inode.h"
#include "fs/mount/mount.h"
#include "fs/common/fsmes.h"
#include "fs/syscall/time.h"
#include "fs/inode/read.h"
#include "systask/syst_pub.h"
#include "clock/h_clock.h"

#undef EXTERN

/**************************************************************************\
**                                                                        **
**  int do_utime()      meni cas v souboru                                **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_UTIME                                           **
**          fs_namet    pointer na nazev souboru                          **
**          fs_times    novy cas (sekundy od 1.1.1970)                    **
**                                                                        **
**  vystup: fs_errcode	chybovy kod 		                          **
**                                                                        **
\**************************************************************************/

PUBLIC int do_utime()

{
  struct inode *pinode;

  fetch_name((vir_bytes)fs_msg.fs_namet);

  if( (pinode=name_i(user_path))==NIL_INODE ) {fs_repmsg.fs_errcode=ENOENT;
					       return 0;}

  if (superuser ||
     (pinode->i_uid==( (struct fproc_struct*)fp)->fp_effuid &&
     pinode->i_mode&UW_BIT) ||
     (pinode->i_gid==( (struct fproc_struct*)fp)->fp_effgid &&
     pinode->i_mode&GW_BIT) ||
     pinode->i_mode&OW_BIT )
  {
    pinode->i_mtime=(real_time)fs_msg.fs_time_s;
    fs_repmsg.fs_errcode=OK;
    return 0;
  }
  else {fs_repmsg.fs_errcode=EACCES;
	return 0;}

}


/**************************************************************************\
**                                                                        **
**  int do_time()       zjistuje aktualni cas                             **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_TIME                                            **
**                                                                        **
**                                                                        **
**  vystup: fs_time_s,fs_time_ms                                          **
**                                                                        **
\**************************************************************************/

PUBLIC int do_time()

{
  static message mm;

  mm.m_type=GET_TIME;
  send_recv(CLOCK,mm);
  fs_repmsg.fs_time_s=mm.RET_REAL_TIME_S;
  fs_repmsg.fs_time_ms=mm.RET_REAL_TIME_M;

  fs_repmsg.fs_errcode=OK;
  return 0;
}


/**************************************************************************\
**                                                                        **
**  int do_stime()      nastavuje cas                                     **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_STIME                                           **
**          fs_time_s   novy cas - sekundy od 1.1.1970                    **
**                                                                        **
**  vystup:  return send_recv(CLOCK,mm) - RET_ERROR_CODE	          **
**                                                                        **
\**************************************************************************/

PUBLIC int do_stime()

{
  static message mm;

  mm.m_type=SET_TIME;
  mm.NEW_TIME=fs_msg.fs_time_s;
  send_recv(CLOCK,mm);

  fs_repmsg.fs_errcode=mm.RET_ERROR_CODE;
  return mm.RET_ERROR_CODE;
}


/**************************************************************************\
**                                                                        **
**  int do_tims()       zjistuje casy procesu                             **
**                                                                        **
****************************************************************************
**                                                                        **
**  vstup:  m_type	C_TIMES                                           **
**          fs_buffer   pointer na vyst bufer (pole) typu real_time       **
**                      (tiky spotrebovane procesem a jeho potomky        **
**			 v rezimu uzivatelskem a jadra)			  **
**                                                                        **
**  vystup: return rw_user()                                              **
**                                                                        **
\**************************************************************************/

PUBLIC int do_tims()

{
  static time_struct tt;
  static message fs;
  int ret;

  fs.m_type=SYS_TIMES;
  fs.PROC=who;
  fs.MEM_PTR= (char *) &tt;
  send_recv(SYSTASK, fs);

  ret=rw_user(D,who,fs_msg.fs_buf,
    (vir_bytes) sizeof(time_struct),(char *) &tt,TO_USER);
  fs_repmsg.fs_errcode=ret;
  return ret;
}
