/*
   This file contains the table used to map system call numbers onto the
   routines that perform them.
   Calls assigned to no_sys routine are serviced by MM
 */

#include "fs/syscall/protect.h"
#include "fs/inode/read.h"
#include "fs/syscall/link.h"
#include "fs/syscall/open.h"
#include "fs/syscall/stadir.h"
#include "fs/syscall/time.h"
#include "fs/device/pipe.h"
#include "fs/syscall/misc.h"
//#include "mm/getmproc.h" neni mozne prilinkovat si od nekoho neco
#include "fs/device/device.h"
#include "fs/mount/mount.h"
#include "fs/cache/cache.h"

#define NCALLS 49
int no_sys() {return 0;}


PUBLIC int (*call_vector[NCALLS]) (void)  = {
	no_sys,         /*  0 = unused  */

/* The following three system calls are actually implemented by MM,
   but FS also implements parts of them and carries some housekeeping work
   when instructed by MM servicing these call to do so */ 

	do_exit,        /*  1 = exit    */
	do_fork,        /*  2 = fork    */
	no_sys,		/*  3 = exec    */

	no_sys,         /*  4 = break   */

	do_read,        /*  5 = read    */
	do_write,       /*  6 = write   */
	do_open,        /*  7 = open    */
	do_close,       /*  8 = close   */
	do_lseek,       /*  9 = lseek   */
	do_creat,       /* 10 = creat   */
	do_mknod,       /* 11 = mknod   */
	do_link,        /* 12 = link    */
	do_unlink,      /* 13 = unlink  */
	do_stat,        /* 14 = stat    */
	do_fstat,       /* 15 = fstat   */
	do_ioctl,       /* 16 = ioctl   */
 	do_utime,       /* 17 = utime   */
	do_mount,       /* 18 = mount   */
	do_umount,      /* 19 = umount  */
	do_access,      /* 20 = access  */
	do_dup,         /* 21 = dup     */
	no_sys,         /* 22 = pipe - not implemented   */

	no_sys,         /* 23 = waitpid */
	do_sync, 	/* 24 = flush all device */

	do_chmod,       /* 25 = chmod   */
	do_chown,       /* 26 = chown   */
	do_chdir,       /* 27 = chdir   */
	do_chroot,      /* 28 = chroot  */
	do_umask,       /* 29 = umask   */

	no_sys,         /* 30 = getpid  */
	no_sys,         /* 31 = getpgrp */
	no_sys,         /* 32 = setpgrp */
	no_sys,         /* 33 = getuid  *//* Divne dodelat treba. */
	do_setuid,         /* 34 = setuid  */
	no_sys,         /* 35 = getgid  */
	do_setgid,         /* 36 = setgid  */


	do_time,        /* 37 = time    */
	do_stime,       /* 38 = stime   */

	no_sys,         /* 39 = signal  */
	no_sys,         /* 40 = kill    */
	no_sys,         /* 41 = alarm   */
	no_sys,         /* 42 = pause   */

	do_tims,        /* 43 = times   */
	no_sys,         /* 44           */

	no_sys,         /* 45 = KSIG: signals originating in the kernel */
	do_unpause,     /* 46 = UNPAUSE */
	do_revive,      /* 47 = REVIVE  */
	no_sys //do_getmproc	/* 48 = getproctbl */
};


