/****************************************************************************/
/*                         TUOX kernel: modul MPXB88.C                      */
/****************************************************************************/

/*****************************************************************************
 Soubor s assemblerovymi rutinami nejnizsi urovne pro kernel
*****************************************************************************/

// Obsazene rutiny:
//      s_call ()
//              -SEND,RECEIVE,TRAP,DIE_TRAP
//              - trap od send/receive message (hlavni a jediny vstupni bod SW do jadra)
//              vola k_save, nachysta parametry a zavola sys_call a k_restart
//      tty_int () ... interrupt
//              - vstupni bod preruseni od klavesnice
//              vola k_save, keyboard(), k_restart
//      hdd_int () ... interrupt
//              - vstupni bod preruseni od disku
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//      floppy_int () ... interrupt
//              - vstupni bod preruseni od FDD
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//      clock_int () ... interrupt
//              - vstupni bod preruseni od hodin
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//      k_save ()
//              - uschova registry do proc[cur_proc]
//      k_restart ()
//              - obnovi registry z proc[cur_proc] a obnovi beh procesu/tasku cur_proc.
//              Je-li cur_proc==IDLE : skoc na idle (WAIT, skok zpet na WAIT)

#define AXR_O(o) [o+2*AX_REG]             //jsou to prvni polozky v proc [], takze od nuly
#define BXR_O(o) [o+2*BX_REG]           //jeden registr za druhym
#define CXR_O(o) [o+2*CX_REG]
#define DXR_O(o) [o+2*DX_REG]
#define SIR_O(o) [o+2*SI_REG]
#define DIR_O(o) [o+2*DI_REG]
#define BPR_O(o) [o+2*BP_REG]
#define DSR_O(o) [o+2*DS_REG]
#define ESR_O(o) [o+2*ES_REG]
#define SSR_O(o) [o+2*SS_REG]

#include "kernel\makra.h"
#include "kernel\proc.h"
#include "COMMON/com.h"
#include "COMMON/glo.h"

#include "test\kprintf.h"
#include "test.cnf"

extern int sys_call(int function, int caller, int src_dest, message *m_ptr);

extern void saveDS(void);
extern void restoreDS(void);

message int_mess;

char dtrap_txt[]     = "System halted: DIE_TRAP #: %u.";
char trap_txt[]        = "TRAP #: %u.";
char unexpected_int_txt[] = "Unexpected int";

#define TRAP_ATTR (16*1+7)
#define DIE_TRAP_ATTR (16*4+7)

void s_call ();
void tty_int ();
void floppy_int ();
void hdd_int ();
void clock_int ();
void k_save (void);
void k_restart (void);
void keyboard (void);
void unexpected_int (void);


//=============================================================================
//      s_call ()
//              - trap od send/receive message (hlavni a jediny vstupni bod SW do jadra)
//              vola k_save, nachysta parametry a zavola sys_call a k_restart
//=============================================================================

static unsigned int src_dest;
static unsigned int fn;
static message* msg;

void s_call ()
{
	k_save();

	asm cmp ax, TRAP
	asm jb sendrec

	asm cmp ax,TRAP
	asm jne dietrap

/* obsluha TRAP */

	asm push bx /* err code (non-fixed param) */
	kprintf(0,0,TRAP_ATTR,trap_txt);

	asm add sp,2 /*///// pop non-fixed param */
	asm jmp konec

dietrap:

/* obsluha DIE_TRAP */

	asm push bx /* err code (non-fixed param) */
	kprintf(0,0,DIE_TRAP_ATTR,dtrap_txt);
	asm add sp,2 /*///// pop non-fixed param */
	asm cli
fajront:
	asm jmp fajront

/* obsluha send/receive/send_recv */
sendrec:
	asm mov fn, ax
	asm mov src_dest,bx
	asm mov msg,dx
	sys_call(fn,cur_proc,src_dest,msg);
konec:
	k_restart ();
};

//=============================================================================
//      tty_int () ... interrupt
//              - vstupni bod preruseni od klavesnice
//              vola k_save, keyboard(), k_restart
//=============================================================================

void  tty_int ()

{
	k_save ();
	keyboard (); /* EOI uvnitr */
	k_restart ();
}

/*
void rs232_1_int(void)
{
	k_save ();
	receive_rs232_com1 (); // EOI uvnitr
	k_restart ();
}

void rs232_2_int(void)
{
	k_save ();
	receive_rs232_com2 (); // EOI uvnitr
	k_restart ();
}
*/

//=============================================================================
//      floppy_int () ... interrupt
//              - disk_int - vstupni bod preruseni od disku
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//=============================================================================

void  floppy_int ()

{

	k_save ();
	k_port_out(0x20,0x20); // EOI

	int_mess.m_source = HARDWARE;
	int_mess.m_type = DISKINT;
	k_interrupt (FLOPPY,&int_mess);

	k_restart ();
}


//=============================================================================
//      hdd_int () ... interrupt
//              - disk_int - vstupni bod preruseni od disku
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//=============================================================================

void  hdd_int ()
/* HDD driver interrupt momentalne stejne vubec nepouziva */
{
	k_save ();
	int_mess.m_source = HARDWARE;
	int_mess.m_type = DISKINT;
	k_interrupt (HDD,&int_mess);
// ???	k_port_out(0x20,0x20); // EOI
	k_restart ();

}

//=============================================================================
//      clock_int () ... interrupt
//              - vstupni bod preruseni od hodin
//              vola k_save, nachysta message, vola k_interrupt, k_restart
//=============================================================================

#ifdef TEST_FDD_USING_BIOS
static void interrupt (*oldInt08)(void);
#endif

void  clock_int ()
{
	k_save ();
	#ifdef TEST_FDD_USING_BIOS
	 // zavolej puvodni obsluhu, kvuli floppy driveru simulovanemu
	 // pres BIOS
	 oldInt08=orig_vectors[8];
	 oldInt08();
	#else
	 k_port_out(0x20,0x20); // EOI (hned, co nejdrive)
	#endif

	int_mess.m_source = HARDWARE;
	int_mess.m_type = CLOCK_TICK;
	k_interrupt (CLOCK,&int_mess);
	k_restart ();
}

//=============================================================================
//      k_save ()
//              - uschova registry do proc[cur_proc]
//              - nastavi DS=DS jadra
// Zachovava AX, BX a DX (kvuli makrum SEND/RECEIVE/TRAP)
//=============================================================================

void  k_save ()
{
// prekladac prida: push bp, mov bp,sp, push si, push di

	asm push ds
	restoreDS();
	asm push ax
	asm push si
	asm mov si, word ptr proc_ptr    //adresa struktury
	asm add sp,10+4

	/* 4 za push si,di */
	asm pop ax                                                                                       //prectu neprepsane bp
	asm mov BPR_O(si),ax

	asm mov BXR_O(si),bx
	asm mov CXR_O(si),cx
	asm mov DXR_O(si),dx
	asm mov DIR_O(si),di
	asm mov ax, es    //prepisu ax kde byl BP
	asm mov ESR_O(si),ax
	asm mov ax, ss
	asm mov SSR_O(si),ax

	asm mov ax,sp       //v teto chvili ukazuje sp na IP ?!,viz 3
	asm mov [si+NR_REGS*2],ax
	asm sub sp, 12+4
	asm pop ax                              //si-puvodni
	asm mov SIR_O(si),ax
	asm pop ax                              //ax-puvodni
	asm mov AXR_O(si),ax

	asm pop ax                              // ds-puvodni
	asm mov DSR_O(si),ax

			   // flags,cs a ip p_pcpsw
	asm mov bp,sp
	asm mov ax,[bp+6+4] // ip ze stacku
	asm mov [si+NR_REGS*2+2],ax
	asm mov ax,[bp+8+4] // cs ze stacku
	asm mov [si+NR_REGS*2+4],ax
	asm mov ax,[bp+10+4] // flags ze stacku
	asm mov [si+NR_REGS*2+6],ax

	asm mov ax,AXR_O(si)
	asm mov bx,BXR_O(si)
	asm mov dx,DXR_O(si)
}

//=============================================================================
//      k_restart ()
//              - obnovi registry z proc[cur_proc] a obnovi beh procesu/tasku cur_proc.
//
// Pri vstupu se predpoklada DS=DS jadra
//=============================================================================

reg_t new_DS,new_AX,new_BX;

void  k_restart ()
// prekladac prida: push bp, mov bp,sp, push si, push di

{
	asm add sp,4 ; /* zahodime si a di */
	asm mov si, word ptr proc_ptr
	asm mov bx, BXR_O(si)
	asm mov cx, CXR_O(si)
	asm mov dx, DXR_O(si)
	asm mov di, DIR_O(si)
	asm mov bp, BPR_O(si)
	asm mov ax, ESR_O(si)
	asm mov es,ax
	asm mov ax, AXR_O(si)
	asm push ax
	asm mov ax, SIR_O(si)
	asm push ax
	asm mov ax, DSR_O(si)
	asm mov new_DS,ax
	asm mov ax, SSR_O(si)
	asm push ax
	asm mov ax,[si+NR_REGS*2]
	asm push ax

	asm add sp,4 /* zpatky na si */

	asm pop si
	asm pop ax
	asm sub sp,8 /* a zase na SP */

	asm mov new_AX,ax /* schovat (jeste do kernel_DS) */
	asm mov new_BX,bx
	asm pop ax /* do ax dame sp */
	asm pop bx /* do bx dame ss */
	asm mov ss,bx
	asm mov sp,ax /* nastaven novy stack */

/* dokud jeste mame stary DS (kerneluv), vybereme schovane AX, BX */
/* a vlozime na NOVY stack. Nakonec prepiseme DS */

	asm mov ax,new_DS
	asm push ax
	asm mov ax,new_AX
	asm mov bx,new_BX
	asm pop ds

	/* UF! */

	asm iret
}

//=============================================================================
//      unexpected_int()
//=============================================================================

void unexpected_int (void)
{ 
	k_save();
	kprintf(0,0,TRAP_ATTR,unexpected_int_txt);
	k_restart();
}


