/****************************************************************************/
/*                         TUOX kernel: modul PROC_PUB.H                    */
/****************************************************************************/
/*     Verejna cast vyclenena specialne pro ostatni ze souboru proc.h.      */
/*   Tento hlavickovy soubor je includovan z hlavickoveho souboru proc.h.   */
/****************************************************************************/

#ifndef __PROC_PUB_H_
#define __PROC_PUB_H_

#include "COMMON\type.h"
#include "COMMON\const.h"


#ifndef EXTERN
  #define EXTERN extern 
  #define EXTERN_OVERRIDE
#endif


#define NR_REGS  10
#define NQ 3                      /* # of scheduling queues */

struct pc_psw {
  reg_t pc; /* pocita se s tim, ze prekladac da 1. polozku na nejnizsi adresu */
  phys_clicks cs;
  unsigned psw;
};

/* Bits for p_flags in proc[].  A process is runnable if p_flags == 0. */
/* POZOR cisla jsou v osmickove soustave */

#define P_SLOT_FREE      001      /* set when slot is not in use */
#define NO_MAP           002      /* keeps unmapped forked child from running */
#define SENDING          004      /* set when process blocked trying to send */
#define RECEIVING        010      /* set when process blocked trying to recv */

EXTERN struct proc {
/*** Polozky se nesmi prehazet (k_save(), k_restore() vyuziva poradi **/
  reg_t p_reg[NR_REGS];           /* process' registers saved here */
  int p_sp;                       /* stack pointer */

  struct pc_psw p_pcpsw;          /* pc and psw as pushed on interrupt */
  int p_flags;                    /* P_SLOT_FREE, SENDING, RECEIVING, etc. */
  struct mem_map p_map[NR_SEGS];  /* memory map */
  int* p_splimit;                 /* lowest legal stack value */
  pid_t p_pid;                    /* process id passed in from MM */

  real_time user_time;            /* user time in ticks */
  real_time sys_time;             /* sys time in ticks */
  real_time child_utime;          /* cumulative user time of children */
  real_time child_stime;          /* cumulative sys time of children */
  real_time p_alarm;              /* time of next alarm in ticks, or 0 */

  struct proc *p_callerq;         /* head of list of procs wishing to send */
  struct proc *p_sendlink;        /* link to next proc wishing to send */
  message *p_messbuf;             /* pointer to message buffer */
  int p_getfrom;                  /* from whom does process want to receive? */

  struct proc *p_nextready;       /* pointer to next ready process */
  sigset_t p_pending;             /* bit map for pending signals */
};


EXTERN unsigned busy_map;            /* bit map of busy tasks */
EXTERN message task_mess[NR_TASKS];  /* messages for busy tasks */
 // P.G. : opraveno z pole pointru na zpravy na pole zprav

EXTERN struct proc proc[NR_TASKS + NR_PROCS + NR_SERVS];   
  /* process table */
EXTERN void far *orig_vectors[256];     
  /* original interrupt vectors */

/* Ted prijde neco duleziteho nejen pro nas, ale i napr. pro systask  */
/* Sekce konstant pro odkazovani se na registry v poli proc[].p_reg[] */

#define AX_REG 0
#define BX_REG 1
#define CX_REG 2
#define DX_REG 3
#define SI_REG 4
#define DI_REG 5
#define BP_REG 6
#define DS_REG 7
#define ES_REG 8
#define SS_REG 9

/* ---------------------------------------------------------------------- */
/* - Pomocne nizkourovnove rutiny uzitecne pro moduly slinkovane s jadrem */

void k_phys_copy (DWord src, DWord dest, Word count);
// Kopie dat odkudkoli kamkoli v ramci cele pameti
// (adresy jsou 32-bitove linearni)

void k_cp_mess (Word src,DWord src_addr,DWord dest_addr);
// Kopie message od odesilatele (src_addr) k prijemci (dest_addr)
// Ve zkopirovanem message navic nastavi cislo procesu odesilatele=src.
// (adresy jsou 32-bitove linearni)

void k_port_out (Word port, Byte value);
// Zapis hodnoty na port

Byte k_port_in (Word port);
// Cteni hodnoty z portu

void k_lock (void);
// Zakaz preruseni, uschova puvodni hodnotu (povoleno/zakazano)
// do globalni promenne, k_restore() obnovi puvodni stav

void k_restore ();
// Obnoveni predchoziho stavu interrupts (z glob. promenne)

void k_unlock ();
// Povoleni preruseni

void k_interrupt(int task_nr, message* m_ptr);
// Obsluha HW preruseni (message zformuluje assembler MPX88.C)
// Vola se z nejnizsich vrstev obsluznych rutin preruseni a take
// se pouziva v tascich pro nasimulovani preruseni od HARDWARE (floppy)

/* ---------------------------------------------------------------------- */
/*                    Rutiny planovace		  			  */


PUBLIC void pick_proc(void);
PUBLIC void sched(void);
PUBLIC void ready(register struct proc *rp);
PUBLIC void unready(register struct proc *rp);




#ifdef EXTERN_OVERRIDE
  #undef EXTERN_OVERRIDE
  #undef EXTERN
#endif


#endif /* __PROC_PUB_H_ */
