#include "kernel\proc.h"
#include "kernel\intrs.h"
#include "test.cnf"

#include "COMMON\com.h"
#include "COMMON\const.h"
#include "COMMON\glo.h"
#include "test\kprintf.h"
#include "test\execldr.h"
#include "test\fsldr.h"

#if (defined(TEST_EXIT_ON_KEYEND) && (TEST_PHASE != 0))
#include "test\exitor.h"
#endif


#define N_MODULES 4

Word k_code_segment,k_data_segment;

extern void saveDS(void);
typedef void (*handler_t)(void);

#define MK_FP( seg,ofs )( (void _seg * )( seg ) +( void near * )( ofs ))
void schovejInterrupty(void) {
int i;
char far* s=MK_FP(0,0);
char far* d=MK_FP(k_data_segment,orig_vectors);
asm cli;
for (i=0; i<256*4; i++) *d++=*s++;
asm sti;
}


void obnovInterrupty(void) {
int i;
char far* d=MK_FP(0,0);
char far* s=MK_FP(k_data_segment,orig_vectors);
asm cli;
for (i=0; i<256*4; i++) *d++=*s++;
asm sti;
}

void mysetvect(int intNo, handler_t handler) {

 asm {
  cli
  push es
  xor ax,ax
  mov es,ax
  mov bx,[intNo]
  shl bx,1
  shl bx,1
  mov ax,[handler]
  mov es:[bx],ax
  push cs
  pop ax
  add bx,2
  mov es:[bx],ax
  pop es
  sti
 }
}

void ohniInterrupty(void) {

 int i;
/* !!! Kdyz to presmeruju, tak to chcipne s UNEXPECTED_INT trap.
  Je treba prijit na to, ktery int to dela */
/*
Prinejmensim se pro BIOSFLOP.C nesmi presmerovat int6.
 for (i=0; i<256; i++) {
  #ifdef TEST_FDD_USING_BIOS
  if (i!=0x08 && i!=0x1C && i!=0x13 && i!=0x1E)
  #endif
   mysetvect(i,unexpected_int);
 }
*/
 mysetvect(0x31,s_call);
 mysetvect(0x08,clock_int);

 #if (defined(TEST_EXIT_ON_KEYEND)  && (TEST_PHASE != 0))
  kprintf(24,0,7*16,"TUOX Kernel is running. Press [End] to return to DOS");
  #ifndef TEST_USE_TTY
   mysetvect(0x09,tty_exitor);
  #endif
 #endif

 #if defined(TEST_USE_FDD)
  mysetvect(0x0e,floppy_int);
 #endif

 #if defined(TEST_USE_TTY)
  mysetvect(0x09,tty_int);
 #endif

}

#include "kernel\intrs.h"
/*

		 slot   symb. index     task/process        queue
		 ========================================================================
		 0      IDLE            idle                -
		 1      CLOCK           clock_task          TASK_Q    head
		 2      FLOPPY          floppy_task         TASK_Q
		 3      SYSTASK         sys_task            TASK_Q
		 4      MEM             mem_task            TASK_Q
		 5      TTY             tty_task            TASK_Q    tail
		 6	HDD		hdd_task	    TASK_Q
		 7      MM              Memory Manager      SERVER_Q  head
		 8      FS              File System         SERVER_Q  tail
		 9      INIT            Init                USER_Q    head & tail
	      */




/* Tasks */
void idle(void);
void clock_task(void);
void floppy_task(void);
void sys_task(void);
void mem_task(void);
void tty_task(void);
void hdd_task(void);



/* POZOR: poradi podle COMMON/com.h */

#if (TEST_PHASE == 0)

static void (*task[]) () =
  { idle, clock_task, floppy_task, sys_task, mem_task, tty_task, hdd_task };

#else

 void task1(void); void task2(void); void task3(void);
 void proc1(void); void proc2(void); void proc3(void);
 void serv1(void);

 #if (TEST_STACK_FOR_TESTPROCS != 0)
  struct p_stack { int stk[TASK_STACK_BYTES/sizeof(int)]; }
   p_stack[TEST_STACK_FOR_TESTPROCS];
 #endif

 #if (TEST_PHASE <= 4)
 static void (*task[]) () =
  { idle, clock_task, task1, task2, task3, idle,idle };
 #else

 static void (*task[]) () =
  { idle, clock_task,
    #if (defined(TEST_USE_FDD) || defined(TEST_FDD_USING_BIOS))
     floppy_task,
    #else
     idle,
    #endif
    sys_task,
    idle /*mem_task*/,
    #if (defined(TEST_USE_TTY))
     tty_task,
    #else
     idle,
    #endif
    idle /*hdd_task*/ };


 #endif

#endif

#if defined(TEST_JOIN_FS_WITH_KERNEL)
void fs_main(void);
#endif

#if defined(TEST_JOIN_MM_WITH_KERNEL)
void mm_main(phys_clicks h_base);
#endif

#if !defined(TEST_USE_TTY)
void keyboard(void) { }
#endif

void idle() {
 idleLoop:
 asm {
  wait
  jmp short idleLoop
 }
}

void addToProc(int procPos, Word csegment, void (*mainOffset)(void), int* paramsForMain, int numParams,
		int queueToAdd) {
/*
   Pridani tasku nebo serveru do proc[] a do zadane fronty planovace (na konec).
   Je-li queueToAdd==-1, neprida se do zadne fronty
*/
 struct proc* procAddr=proc_addr(procPos);
 int i;

 if (paramsForMain) {
 /* POZOR: do procesu se vstupuje IRETem, takze na zasobniku musi byt
	   PSW,CS,IP a parametry nesmi nasledovat, jak by se cekalo,
	   ale musi byt jeste NAD PSW: po IRETu tam zbydou a skoci se do
	   rutiny, ktera tyto parametry ocekava; ze tam uz neni zadna dalsi
	   navratova adresa nevadi, protoze hlavni rutina procesu nekonci
	   RETem.
	   Dalsi finta je v tom, ze parametr neni bezprostredne nad PSW,
	   protoze prekladac ocekava, ze pred parametrem bude jeste navratova
	   adresa (NEAR). */

  if (procPos<NR_TASKS+NR_SERVS) {
   for (i=0; i<numParams; i++)
    t_stack[procPos].stk[(TASK_STACK_BYTES/2-1)-numParams+i+1]=*paramsForMain++;
  }
  else {
   /**** PRO PROCESY DODELAT */
  }
 }

 /* nachystat stack */
 if (numParams) numParams++;
 /* jedna pozice navic jako ze za navratovou adresu */
 if (procPos<NR_TASKS+NR_SERVS) {
  t_stack[procPos].stk[TASK_STACK_BYTES/2-1-numParams]=INIT_PSW;
  t_stack[procPos].stk[TASK_STACK_BYTES/2-2-numParams]=csegment;
  t_stack[procPos].stk[TASK_STACK_BYTES/2-3-numParams]=(int) mainOffset;

  proc[procPos].p_sp=(int) &(t_stack[procPos].stk[TASK_STACK_BYTES/2-1-2-numParams]);
  proc[procPos].p_splimit= &(t_stack[procPos].stk[0]);

 } else {
     #if (TEST_STACK_FOR_TESTPROCS != 0)
      if ((procPos-NR_TASKS-NR_SERVS)>=TEST_STACK_FOR_TESTPROCS) {
       // cannot call trap() because interrupts are not redirected yet
       kprintf(0,0,16*7,"Attempt to add a new proces when no stack allocated for it");
       kprintf(1,0,16*7,"(TEST_STACK_FOR_TESTPROCS too low). KMAIN.C, line %u",__LINE__);
       while(1);
      }

      #if (TEST_PHASE>MAX_PHASE_KERNEL_FS)
       /* testujeme MM, stack uplne nahoru v segmentu (pod FFFEh) */
       asm { /* SS=DS */
	mov bx,0xffee /* [osledni pargraf segmentu nevyuzivam */
	mov WORD PTR [bx],INIT_PSW
	dec bx
	dec bx
	mov ax,csegment
	mov [bx],ax
	dec bx
	dec bx
	mov ax,mainOffset
	mov [bx],ax
       }

       proc[procPos].p_sp=0xffea;
       proc[procPos].p_splimit=0; /* nevim, co rozumneho tam dat - konec DS */

      #else
       p_stack[procPos-NR_TASKS-NR_SERVS].stk[TASK_STACK_BYTES/2-1]=INIT_PSW;
       p_stack[procPos-NR_TASKS-NR_SERVS].stk[TASK_STACK_BYTES/2-2]=csegment;
       p_stack[procPos-NR_TASKS-NR_SERVS].stk[TASK_STACK_BYTES/2-3]=(int) mainOffset;

       proc[procPos].p_sp=(int) &(p_stack[procPos-NR_TASKS-NR_SERVS].stk[TASK_STACK_BYTES/2-1-2]);
       proc[procPos].p_splimit= &(p_stack[procPos-NR_TASKS-NR_SERVS].stk[0]);

      #endif
     #else
      // cannot call trap() because interrupts are not redirected yet
      kprintf(0,0,16*7,"Attempt to add a new proces when no stack allocated for it");
      kprintf(1,0,16*7,"(TEST_STACK_FOR_TESTPROCS=0). KMAIN.C, line %u",__LINE__);
      while(1);
     #endif
   }

 proc[procPos].p_reg[AX_REG]=proc[procPos].p_reg[BX_REG]=
 proc[procPos].p_reg[CX_REG]=proc[procPos].p_reg[DX_REG]=
 proc[procPos].p_reg[SI_REG]=proc[procPos].p_reg[DI_REG]=
 proc[procPos].p_reg[BP_REG]=0;

 proc[procPos].p_reg[DS_REG]=proc[procPos].p_reg[ES_REG]=
 proc[procPos].p_reg[SS_REG]=k_data_segment;

 proc[procPos].p_pcpsw.pc=(int) mainOffset;
 proc[procPos].p_pcpsw.cs=csegment;
 proc[procPos].p_pcpsw.psw=INIT_PSW;

 proc[procPos].p_flags=0;
 proc[procPos].p_pid=NO_PID;

 proc[procPos].p_map[T].mem_phys = csegment;
 proc[procPos].p_map[D].mem_phys = k_data_segment;
 proc[procPos].p_map[S].mem_phys = k_data_segment;

 if (procPos==INIT) {
 /* !!!??? */
    proc[procPos].p_map[T].mem_len = k_data_segment-k_code_segment;
    proc[procPos].p_map[D].mem_len = (0xfff0-JOINT_DEBUG_STACK_SIZE-JOINT_DEBUG_GAP_SIZE)/CLICK_SIZE;
    proc[procPos].p_map[S].mem_len = JOINT_DEBUG_STACK_SIZE/CLICK_SIZE;
    proc[procPos].p_splimit=(int*) (0xfff0-JOINT_DEBUG_STACK_SIZE);
 } else {
     proc[procPos].p_map[T].mem_len =
     proc[procPos].p_map[D].mem_len =
     proc[procPos].p_map[S].mem_len = VERY_BIG;
   }

 proc[procPos].p_map[T].mem_vir =
 proc[procPos].p_map[D].mem_vir = 0;
 proc[procPos].p_map[S].mem_vir =(vir_bytes) proc[procPos].p_splimit;

 proc[procPos].user_time=proc[procPos].sys_time=
  proc[procPos].child_utime=proc[procPos].child_stime=0;
 proc[procPos].p_alarm=0;
 proc[procPos].p_callerq=proc[procPos].p_sendlink=
  proc[procPos].p_nextready=NULL;
  proc[procPos].p_messbuf=NULL;
 proc[procPos].p_pending=0;
 proc[procPos].p_getfrom=-1;

 if (queueToAdd>=0) {
  /* pridej na konec fronty */
  if (rdy_head[queueToAdd]==NULL)
   rdy_head[queueToAdd]=rdy_tail[queueToAdd]=procAddr;
   else {
    (rdy_tail[queueToAdd])->p_nextready=procAddr;
    rdy_tail[queueToAdd]=procAddr;
   }
 }
}

/*===========================================================================*
 *                                   main                                    *
 *===========================================================================*/
PUBLIC void main() //(struct tuox_header module[N_MODULES])
{
    int i;
    #if defined(TEST_JOIN_MM_WITH_KERNEL)
     int memoryStart=0;
    #endif

//    struct tuox_header *Th = module + 1;

    busy_map = 0;
    for (i=0; i<NQ; i++) rdy_head[i]=rdy_tail[i]=NULL;

    /* zjisti hodnotu CS a DS kernelu */
    asm {
     push cs
     pop ax
     mov k_code_segment,ax
     mov ax,ds
     mov k_data_segment,ax
    }

    saveDS();

    /*
     * Nastaveni p_sp tasku/procesu na t_stack[]/INIT_SP
     a Nastaveni p_splimit, p_flags=0, pro tasky:p_pcpsw.pc=task[i]
     */

    for(i = 0; i < NR_TASKS + NR_SERVS + NR_PROCS; i++)
    {
	if (i < NR_TASKS)
	    addToProc(i,k_code_segment,task[i],NULL,0, (task[i]==idle) ? -1 : TASK_Q);

	else if (i<NR_TASKS+NR_SERVS) /* MM, FS */
	{
//	    addToProc(i,task[i],NULL,0,SERVER_Q);
/* Tady to bude zavisle na adresach, kde bude FS a MM lezet */
/*

	    proc[i].p_sp = (reg_t) (Th->Total - Th->TextSize);
	    proc[i].p_splimit = (int) (Th -> DataLen + Th -> BssLen);
	    proc[i].p_map[T].mem_phys = Th->dontcare1 / CLICK_SIZE;
	    proc[i].p_map[D].mem_phys =
	    proc[i].p_map[S].mem_phys = Th->dontcare2 / CLICK_SIZE;

	    proc[i].p_map[T].mem_len = Th->TextLen / CLICK_SIZE;
	    proc[i].p_map[D].mem_len =
	    proc[i].p_map[S].mem_len = (Th->Total - Th->TextLen) / CLICK_SIZE;
	    proc[i].p_map[T].mem_vir =
	    proc[i].p_map[D].mem_vir = 0;
	    proc[i].p_map[S].mem_vir = (Th->Total - Th->TextLen) / CLICK_SIZE;
	    proc[i].p_reg[DS_REG] =
	    proc[i].p_reg[ES_REG] = (reg_t) proc[i].p_map[D].mem_phys;
	    // next process
	    Th++;
*/
	}
	 else if (i==NR_TASKS+NR_SERVS) {
		/* INIT */
		}

	else proc[i].p_flags = P_SLOT_FREE; /* other slot */
    }

    #if (TEST_PHASE==4)
     addToProc(NR_TASKS+NR_SERVS,k_code_segment,proc1,NULL,0, USER_Q);
     addToProc(NR_TASKS+NR_SERVS+1,k_code_segment,proc2,NULL,0, USER_Q);
     addToProc(NR_TASKS+NR_SERVS+2,k_code_segment,proc3,NULL,0, USER_Q);
    #endif

    #if (TEST_PHASE==5 || TEST_PHASE==6 || TEST_PHASE==7)
     addToProc(NR_TASKS,k_code_segment,serv1,NULL,0, SERVER_Q);
    #endif

    #if defined(TEST_JOIN_FS_WITH_KERNEL)
     addToProc(FS,k_code_segment,fs_main,NULL,0, SERVER_Q);
    #else
      #if (TEST_PHASE>=PHASE_FS_FINAL)
     // dodelat: vyplnit slot FS (vcetne delky-asi VERY_BIG)
      #endif
    #endif

    #if defined(TEST_JOIN_MM_WITH_KERNEL)
     memoryStart=k_data_segment+0x1000+EXECINMEM_SIZE;
      /* prazdno zacina 64kB za DS kernelu */
     #if FSINMEM_SIZE != 0
     memoryStart+=FSINMEM_SIZE;
     /* ... a za image FS */
     #endif
      /* prazdno zacina 64kB za DS kernelu */
     addToProc(MM,k_code_segment,(void (*)(void)) mm_main,&memoryStart,1, SERVER_Q);
    #else
     #if (TEST_PHASE>MAX_PHASE_KERNEL_MM)
      // dodelat: vyplnit slot MM (vcetne delky-asi VERY_BIG)
      #error Samotatny MM - vyplnit slot proc[]
     #endif
    #endif

    #if (TEST_PHASE>MAX_PHASE_KERNEL_ONLY && TEST_PHASE<=MAX_PHASE_KERNEL_MM)
     addToProc(NR_TASKS+NR_SERVS,k_code_segment,proc1,NULL,0, USER_Q);
    #endif

    #if (EXECINMEM_SIZE!=0)
     if ((i=loadExecFile(EXECINMEM_FILE))!=0) {
      kprintf(20,0,7,"Load test exec file "EXECINMEM_FILE" failure (error code %i)",i);
      dosExit(-1);
     } else
	kprintf(20,0,7,"Load test exec file "EXECINMEM_FILE" successful");
    #endif

    #if (FSINMEM_SIZE!=0)
     if ((i=loadFS(FSINMEM_FILE,k_data_segment+0x1000+EXECINMEM_SIZE))!=0) {
      kprintf(21,0,7,"Load FS image failed (error code %i)",i);
      dosExit(-2);
     } kprintf(21,0,7,"Loading FS image successful");
    #endif

    bill_ptr = proc + IDLE;
    cur_proc=IDLE; // aby se rozumne nastavil prev_ptr
    lost_ticks=0;

    // schovat puvodni vektory do orig_vectors)


    /*
    /* The table of interrupt vectors will be set as follows:

       vector       points to       notes
       --------------------------------------------------------------------------
       08 (IRQ0)    clock_int       [kernel/mpx88.c]
       09 (IRQ1)    tty_int         [kernel/mpx88.c]
       0E (IRQ6)    disk_int        [kernel/mpx88.c]
       31           s_call          [kernel/mpx88.c]
       others       unexpected_int  [kernel/mpx88.c]
       --------------------------------------------------------------------------
    */


     schovejInterrupty();
     ohniInterrupty();

     pick_proc(); // tim se vybere nejprioritnejsi task schopny behu
     k_restart(); // ... a spusti se
}

void exit(int i) { }
/* exit() se stejne nikdy nevola (teoreticky po vyskoceni z main() (c0fs.asm),
   ale k tomu nikdy nedojde. */