/*********************************************************
 *               Hlavickove soubory pro TUOX             *
 *                                                       *
 * Soubor:    stdio.h                                    *
 * Popis:     Definice proudovych V/V funkci             *
 *                                                       *
 * Autori:    Rostislav Bosak                            *
 *            Jaroslav Hurdes                            *
 *            Martin Perina                              *
 *            Martin Vojvodik                            *
 *                                                       *
 * Preprac:   Daniel Tvrdy				 *
 *	      Ivo Orlik                         (c) 1997 *
 *********************************************************/

#ifndef _STDIO_H_

#define _STDIO_H_

#include "_null.h"

#define _NFILE_ 20

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned size_t;
#endif

/* Definition of the file position type */
typedef long    fpos_t;


/* Definition of the control structure for streams */
typedef struct  {
        int             level;          /* fill/empty level of buffer */
        unsigned        flags;          /* File status flags          */
        char            fd;             /* File descriptor            */
        unsigned char   hold;           /* Ungetc char if no buffer   */
        int             bsize;          /* Buffer size                */
        unsigned char    *buffer;       /* Data transfer buffer       */
        unsigned char    *curp;         /* Current active pointer     */
        unsigned        istemp;         /* Temporary file indicator   */
        short           token;          /* Used for validity checking */
}       FILE;                           /* This is the FILE object    */

/* Bufferisation type to be used as 3rd argument for "setvbuf" function */
#define _IOFBF  0
#define _IOLBF  1
#define _IONBF  2

/*  "flags" bits definitions */
#define _F_RDWR 0x0003                  /* Read/write flag       */
#define _F_READ 0x0001                  /* Read only file        */
#define _F_WRIT 0x0002                  /* Write only file       */
#define _F_BUF  0x0004                  /* Malloc'ed Buffer data */
#define _F_LBUF 0x0008                  /* line-buffered file    */
#define _F_ERR  0x0010                  /* Error indicator       */
#define _F_EOF  0x0020                  /* EOF indicator         */
#define _F_BIN  0x0040                  /* Binary file indicator */
#define _F_IN   0x0080                  /* Data is incoming      */
#define _F_OUT  0x0100                  /* Data is outgoing      */
#define _F_TERM 0x0200                  /* File is a terminal    */

/* End-of-file constant definition */
#define EOF (-1)            /* End of file indicator */

/* Number of files that can be open simultaneously */
#define FOPEN_MAX (_NFILE_)     /* Able to have 20 files */
#define SYS_OPEN  (_NFILE_)

#define FILENAME_MAX 80

/* Default buffer size use by "setbuf" function */
#define BUFSIZ  512         /* Buffer size for stdio */

/* Size of an arry large enough to hold a temporary file name string */
#define L_ctermid   5       /* CON: plus null byte */
#define P_tmpdir    ""      /* temporary directory */
#define L_tmpnam    13      /* tmpnam buffer size  */

/* Constants to be used as 3rd argument for "fseek" function */
#define SEEK_CUR    1
#define SEEK_END    2
#define SEEK_SET    0

/* Number of unique file names that shall be generated by "tmpnam" function */
#define TMP_MAX     0xFFFF

/* Standard I/O predefined streams */

extern  FILE    cdecl _streams[];
extern  unsigned    cdecl _nfile;

#define stdin   (&_streams[0])
#define stdout  (&_streams[1])
#define stderr  (&_streams[2])

/* nevim ???
#define stdaux  (&_streams[3])
#define stdprn  (&_streams[4]) */

FILE far * far __getStream(int);

/* duplikaty ?
#define stdin   __getStream(0)
#define stdout  __getStream(1)
#define stderr  __getStream(2)
*/
/* nevim ???
#define stdaux  __getStream(3)
#define stdprn  __getStream(4) */

void    cdecl  clearerr(FILE  *__stream);
int     cdecl  fclose(FILE  *__stream);
/* int     cdecl  fflush(FILE  *__stream); */
/* int     cdecl  fgetc(FILE  *__stream); */
int     cdecl  fgetpos(FILE  *__stream, fpos_t  *__pos);
/* char    *cdecl  fgets(char  *__s, int __n, FILE  *__stream); */
FILE    *cdecl  fopen(const char  *__path, const char  *__mode);
/* int     cdecl  fprintf(FILE  *__stream, const char  *__format, ...); */
/* int     cdecl  fputc(int __c, FILE  *__stream); */
/* int     cdecl  fputs(const char  *__s, FILE  *__stream); */
/* size_t  cdecl  fread(void  *__ptr, size_t __size, size_t __n, FILE  *__stream); */
FILE    *cdecl  freopen(const char  *__path, const char  *__mode, FILE  *__stream);
/* int     cdecl  fscanf(FILE  *__stream, const char  *__format, ...); */
int     cdecl  fseek(FILE  *__stream, long __offset, int __whence);
int     cdecl  fsetpos(FILE  *__stream, const fpos_t  *__pos);
long    cdecl  ftell(FILE  *__stream);
/* size_t  cdecl  fwrite(const void  *__ptr, size_t __size, size_t __n, FILE  *__stream); */
/* char    *cdecl  gets(char  *__s); */
/* void    cdecl  perror(const char  *__s); */
int     cdecl  printf(const char  *__format, ...);
/* int     cdecl  puts(const char  *__s); */
/* int     cdecl  remove(const char  *__path); */
/* int     cdecl  rename(const char  *__oldname,const char  *__newname); */
void    cdecl  rewind(FILE  *__stream);
/* int     cdecl  scanf(const char  *__format, ...); */
 int     cdecl  sprintf(char  *__buffer, const char  *__format, ...);
/* int     cdecl  sscanf(const char  *__buffer, const char  *__format, ...); */
/* char    *cdecl  strerror(int __errnum); */
/* FILE    *cdecl  tmpfile(void); */
/* char    *cdecl  tmpnam(char  *__s); */
/* int     cdecl  ungetc(int __c, FILE  *__stream); */
/* int     cdecl  vfprintf(FILE  *__stream, const char  *__format, void  *__arglist); */
/* int     cdecl  vfscanf(FILE  *__stream, const char  *__format, void  *__arglist); */
/* int     cdecl  vprintf(const char  *__format, void  *__arglist); */
/* int     cdecl  vscanf(const char  *__format, void  *__arglist); */
int     cdecl  vsprintf(char  *__buffer, const char  *__format, void  *__arglist);
/* int     cdecl  vsscanf(const char  *__buffer, const char  *__format, void  *__arglist); */
/* int     cdecl  getc(FILE  *__fp); */
/* int     cdecl  getchar(void); */
/* int     cdecl  putchar(const int __c); */
/* int     cdecl  putc(const int __c, FILE  *__fp); */
int     cdecl  feof(FILE  *__fp);
int     cdecl  ferror(FILE  *__fp);



/*  The following macros provide for common functions */

#define ferror(f)   ((f)->flags & _F_ERR)
#define feof(f)     ((f)->flags & _F_EOF)

#endif

