/*********************************************************
 *               Hlavickove soubory pro TUOX             *
 *                                                       *
 * Soubor:    stdlib.h                                   *
 * Popis:     Definice zakladnich typu, promenych a 	 *
 * 	      funkci              			 *
 *                                                       *
 * Autori:    Rostislav Bosak                            *
 *            Jaroslav Hurdes                            *
 *            Martin Perina                              *
 *            Martin Vojvodik                            *
 *							 *
 * Preprac:   Daniel Tvrdy				 *
 *	      Ivo Orlik				(c) 1997 *
 *********************************************************/


#ifndef __STDLIB_H
#define __STDLIB_H

#ifndef NULL
#define NULL 0
#endif /* NULL */

#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned size_t;
#endif /* _SIZE_T */

#ifndef _DIV_T
#define _DIV_T
typedef struct {
        int quot;
        int rem;
} div_t;
#endif /* _DIV_T */

#ifndef _LDIV_T
#define _LDIV_T
typedef struct {
	long quot;
        long rem;
} ldiv_t;
#endif /* _LDIV_T */

typedef void (*atexit_t)(void);

#ifndef _WCHAR_T
#define _WCHAR_T
typedef char wchar_t;
#endif /* _WCHAR_T */

/* Maximalni hodnota vracena funkci "rand" */
#define RAND_MAX 0x7FFFU

/* Defaultni navratove hodnoty pro "exit" */
#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1


/* "abort funkce vyvola nenormalni ukonceni programu (raise SIGABRT) */
void    abort(void);

/* vraci absolutni hodnotu argumentu */
int     abs(int __x);
/* registruje funkci, ktera ma byt vyvolana pri beznem ukonceni programu */
int     atexit(void (*__func)(void));
/* prevede retezec s na cislo typu int */
int     atoi(const char *__s);
/* prevede retezec s na cislo typu long */
long    atol(const char *__s);
/* binarni vyhledavani */
void  * bsearch(const void *__key, const void *__base, size_t __nelem,
		size_t __width, int (*fcmp)(const void *,const void *));
/* alokace pole s nulovanim prvku */
void *  calloc(size_t __ecnt, size_t __esize);
/* celociselne deleni se zbytkem */
div_t   div(int __numer, int __denom);
/* normalni ukonceni programu */
void    exit(int __status);
/* dealokace pameti */
void    free(void *block);
/* vraci absolutni hodnotu argumentu typu long*/
long    labs(long __x);
/* celociselne deleni se zbytkem typu long */
ldiv_t  ldiv(long __numer, long __denom);
/* alokace pameti velikosti __size v bytech */
void *  malloc(size_t __size);

/* quick sort */
/*void    qsort(void *__base, size_t __nelem, size_t __width,
	      int (*fcmp)(const void *,const void *))*/

/* vraci pseudonahodne cislo z rozsahu 0..MAX_RAND */
int     rand(void);
/* realokace pameti s kopirovanim obsahu */
void *  realloc(void *__ptr, size_t __size);
/*inicializace generatoru pseudonahodnych cisel */
void    srand(unsigned int __seed);
/* prevod retezce na long */
long    strtol(const char *__s, char **__endptr, int __radix);
/* prevod retezce na unsigned long */
unsigned long   strtoul(const char *__s, char **__endptr, int __radix);

/* spusteni systemoveho prikazu */
/*int     system(const char __command); /* anebo to nebudem podporovat ? */

#endif /* __STDLIB_H */
