/*************************************************************************
 *                          TUOX - knihovny                              *
 *************************************************************************
 * Soubor:    execv.c                                                    *
 * Popis:     Obsahuje tyto funkce:                                      *
 *************************************************************************
 *                                                                       * 
 *  EXECV     spusti proces predstavovany __path a __arg bez enviromentu *
 *  EXECVE    spusti proces predstavovany __path a __arg s enviromentem  *
 *                                                                       * 
 *************************************************************************
 * Autor:     Martin Vojvodik	                                         *
 *	      Bosak Rostislav                                            *
 *            Hurdes Jaroslav                                            *
 *            Perina Martin                                              *
 * 							                 *
 * Preprac:   Daniel Tvrdy 			                         *
 *            Ivo Orlik	                                        (c) 1997 *
 *************************************************************************/

#include "LIB\include\string.h"
#include "LIB\include\process.h"  // konstanata __EXEC_STACK_SIZE, ...
#include "COMMON\callnr.h"
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "MM\mm.h"              /* hlavickovy soubor MM */
#include "FS\common\fsconst.h"         /* hlavickovy soubor FS */
#include "kernel\makra.h"


char _prgStack[__EXEC_STACK_SIZE];
static message m;

/*-------------------------------------------------------------------------*

Name            execv - exec with args

Usage           int execv(const char *__path, char *__argv[]);

Prototype in    dir.h

Description     execv executes program specified by __path, passing it
                arguments specified by __argv (arguments don't include
                first - program name - parameter, last argument pointer
                must be NULL)

Return value    execv returns int - errno

/*------------------------------------------------------------------------*/

int execv (const char *__path, char *__argv[])
{
  int __argc;
  char prgName[NAME_SIZE+1];
  int i, j;
  int psPtrP, psStrP;
  int __retv;

  /* extract & store prgName */
  i = NAME_SIZE;
  prgName[i] = 0;                                 /* EndOfString */
  j = strlen(__path)-1;
  while ( i > 0 && __path[j] != '/' && j >= 0 ) /* backward copying */
  {
    -- i;
    prgName[i] = __path[j];
    -- j;
  }
  __argc = 2;     		   /* first argument is always prgname */
  j = 0;
  while (*(__argv+j) != NULL)      /* __argc je jako sizeof(void *) * argc */
  {
    j ++;
    __argc +=2;
  }
  psStrP = __argc + 4;         /* append one NULL for argv and one for env */
  psPtrP = 0;

  *(int *)(_prgStack+psPtrP) = psStrP;
  psPtrP += 2;
  for ( ; i <= NAME_SIZE ; i ++ )
  {
    if (psStrP == __EXEC_STACK_SIZE)   // __EXEC_STACK_SIZE = 1024
      return __ERR_STACK_SIZE;
    _prgStack[psStrP++] = prgName[i];
  }

  /* store arguments */
  while (*__argv != NULL)
  {
    *(int *)(_prgStack+psPtrP) = psStrP;
    psPtrP += 2;
    for ( i = 0 ; i <= strlen(*__argv) ; i ++ )
    {
      if (psStrP == __EXEC_STACK_SIZE)
	return __ERR_STACK_SIZE;
      _prgStack[psStrP++] = (*__argv)[i];
    }
    __argv++;
  }
  *((int *)(_prgStack+psPtrP)) = 0;
  psPtrP += 2;
  *((int *)(_prgStack+psPtrP)) = 0;
  psPtrP += 2;

  m.m_type = C_EXEC;
  m.MM_EXECSTACK = _prgStack;
  m.MM_EXECSIZE = psStrP;
  m.MM_EXECFNAME = (char*) __path;
  m.MM_PATHSIZE = strlen(__path) + 1;
  send_recv(MM,m);

  return m.MM_ERRORNO;
}

/*-------------------------------------------------------------------------*

Name            execve - exec with args and environment

Usage           int execve(const char *__path, char *__argv[], char **__env);

Prototype in    dir.h

Description     execve executes program specified by __path, passing it
		arguments specified by __argv (arguments don't include
		first - program name - parameter) and environment strings
		specified by __env. Last argument and environment pointers
		must be NULLs.

Return value    execve returns int - errno

/*------------------------------------------------------------------------*/

int execve (const char *__path, char *__argv[], char *__env[])
{
  int __argc;
  char prgName[NAME_SIZE+1];
  int i, j;
  int psPtrP, psStrP;


  /* extract & store prgName */
  i = NAME_SIZE;
  prgName[i] = 0;                                 /* EndOfString */
  j = strlen(__path)-1;
  while ( i > 0 && __path[j] != '/' && j >= 0 ) /* backward copying */
  {
    -- i;
    prgName[i] = __path[j];
    -- j;
  }
  __argc = 2;     		   /* first argument is always prgname */
  j = 0;
  while (*(__argv+j) != NULL)      /* __argc je jako sizeof(void *) * argc */
  {
    j ++;
    __argc +=2;
  }
  j = 0;
  while (*(__env+j) != NULL)
  {
    j ++;
    __argc += 2;
  }
  psStrP = __argc + 4;         /* append one NULL for argv and one for env */
  psPtrP = 0;

  *((int *)(_prgStack+psPtrP)) = psStrP;
  psPtrP += 2;
  for ( ; i <= NAME_SIZE ; i ++ )
  {
    if (psStrP == __EXEC_STACK_SIZE)
      return __ERR_STACK_SIZE;
    _prgStack[psStrP++] = prgName[i];
  }
  /* store arguments */
  while (*__argv != NULL)
  {
    *(int *)(_prgStack+psPtrP) = psStrP;
    psPtrP += 2;
    for ( i = 0 ; i <= strlen(*__argv) ; i ++ )
    {
      if (psStrP == __EXEC_STACK_SIZE)
	return __ERR_STACK_SIZE;
      _prgStack[psStrP++] = (*__argv)[i];
    }
    __argv++;
  }
  *((int *)(_prgStack+psPtrP)) = 0;
  psPtrP += 2;

  /* store environment strings */
  while (*__env != NULL)
  {
    *(int *)(_prgStack+psPtrP) = psStrP;
    psPtrP += 2;
    for ( i = 0 ; i <= strlen(*__env); i++ )
    {
      if (psStrP == __EXEC_STACK_SIZE)
	return __ERR_STACK_SIZE;
      _prgStack[psStrP++] = (*__env)[i];
    }
    __env++;
  }
  *((int *)(_prgStack+psPtrP)) = 0;
  psPtrP += 2;

  m.m_type = C_EXEC;
  m.MM_EXECSTACK = _prgStack;
  m.MM_EXECSIZE = psStrP;
  m.MM_EXECFNAME =(char*) __path;
  m.MM_PATHSIZE = strlen(__path) + 1;
  send_recv(MM,m);
  return m.MM_ERRORNO;
}

