/*
 ****************************************************************************
 * soubor: break.c                                                          *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

/* The TUOX model of memory allocation reserves a fixed amount of memory for
 * the combined text, data, and stack segments.  The amount used for a child
 * process created by FORK is the same as the parent had.  If the child does
 * an EXEC later, the new size is taken from the header of the file EXEC'ed.
 *
 * The layout in memory consists of the text segment, followed by the data
 * segment, followed by a gap (unused memory), followed by the stack segment.
 * The data segment grows upward and the stack grows downward, so each can
 * take memory from the gap.  If they meet, the process must be killed.  The
 * procedures in this file deal with the growth of the data and stack segments.
 *
 * The entry points into this file are:
 *   do_brk:      BRK/SBRK system calls to grow or shrink the data segment
 *   adjust:      see if a proposed segment adjustment is allowed
 *   size_ok:     see if the segment sizes are feasible
 * ? stack_fault: grow the stack segment when sp underflow occurs
 */

#include "COMMON\type.h"
#include "COMMON\com.h"
#include "COMMON\const.h"

#include "kernel\makra.h"
#include "systask\syst_pub.h"

#include "mm\mm.h"
#include "mm\mm_glob.h"
#include "mm\mmerror.h"
#include "mm\break.h"


/*===========================================================================*
 *                              do_brk                                       *
 *===========================================================================*/
PUBLIC int do_brk()
{
/* Perform the brk(addr) system call. Addr is a data size (in bytes).
  Z kernelu zjisti SP (SYS_GETSP), over, zda nekoliduje a nastav
  v mp_seg*/

  if (mm_in.MM_BRKINCR==0) {
   /* jen zjistit adresu */
   mm_out.MM_BRKADDR=(char *)(mproc[who].mp_seg[D].mem_len*CLICK_SIZE);
   return OK;
  }

/* Uprava na paragrafy (vstupuji bajty)*/
  if (mm_in.MM_BRKINCR<0) mm_in.MM_BRKINCR-=15; else mm_in.MM_BRKINCR+=15;
  mm_in.MM_BRKINCR/=16;

/* SYS_GETSP */
  mm_out.PROC=who;
  mm_out.m_type=SYS_GETSP;
  send_recv(SYSTASK,mm_out);
/* Uprava na paragrafy (vstupuji bajty)*/
  mm_out.RET_SP=((unsigned int)mm_out.RET_SP-15)/16;

/* Podminka zda se nam posun hranice nedostane za SP */
  if ((mm_out.RET_SP)<(mm_in.MM_BRKINCR+mproc[who].mp_seg[D].mem_len)) return EBREAK;

/* Aby nekdo nesnizil hranici prilis */
  if (mm_in.MM_BRKINCR<0 && mproc[who].mp_seg[D].mem_len<-mm_in.MM_BRKINCR)
   return EBREAK;

  mproc[who].mp_seg[D].mem_len+=mm_in.MM_BRKINCR;

/* SYS_NEWMAP */
  mm_out.PROC=who;
  mm_out.MEM_PTR=(char *)&(mproc[who].mp_seg);
  mm_out.m_type=SYS_NEWMAP;
  send_recv(SYSTASK,mm_out);

  mm_out.MM_BRKADDR=(char *)(mproc[who].mp_seg[D].mem_len*CLICK_SIZE);
  return OK;
};

/*===========================================================================*
 *                              stack fault                                  *
 *===========================================================================*/

/* Not implemented asi
PUBLIC stack_fault(int proc_nr)
{
 Obsluz stack fault zvetsovanim stack segmentu, az je SP uvnitr.
 Pokud se setka s daty, kill proces (SIGSEGV)
}
*/