/*
 ****************************************************************************
 * soubor: getset.c                                                         *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

/* This file handles the 4 system calls that get and set uids and gids.
 * It also handles getpid().
 */
#include "COMMON\callnr.h"
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "mm\mm.h"
#include "mm\mm_glob.h"
#include "mm\mmerror.h"
#include "mm\getset.h"
#include "mm\mm2fsmsg.h"
#include "kernel\makra.h"


PRIVATE tell_fs(unsigned int msg, int aWho, int realValue, int effValue);

PRIVATE int FindIndex(pid_t prcs)
{
  int pom=0;
/*  while(pom<NR_PROCS) if (mproc[pom].mp_flags&IN_USE) 
		      if (mproc[pom].mp_pid==prcs) return pom;*/
  while(pom<NR_PROCS+NR_TASKS+NR_SERVS) if (mproc[pom].mp_flags&IN_USE) 
		      if (mproc[pom].mp_pid==prcs) return pom;
  return 0;
};

/*===========================================================================*
 *                              do_getset                                    *
 *===========================================================================*/
PUBLIC int do_getset()
{
  /* Ve switchi pouze vratime prislusnou hodnotu z mproc[] */
  int prcs;
  switch(mm_call) {
	case C_GETUID:
	  prcs=(mm_in.MM_GETIDPID==0)?who:FindIndex(mm_in.MM_GETIDPID);
          if (prcs==0) return EInvalidPID;
	  mm_out.MM_GRUID=mproc[prcs].mp_realuid;
	  mm_out.MM_GEUID=mproc[prcs].mp_effuid;
          break;
	case C_GETGID:
	  prcs=(mm_in.MM_GETIDPID==0)?who:FindIndex(mm_in.MM_GETIDPID);
	  if (prcs==0) return EInvalidPID;
	  mm_out.MM_GRGID=mproc[prcs].mp_realgid;
	  mm_out.MM_GEGID=mproc[prcs].mp_effgid;
	  break;
	case C_GETPID:
	  mm_out.MM_GPID=mproc[who].mp_pid;
	  mm_out.MM_GPPID=mproc[mproc[who].mp_parent].mp_pid;
	  break;
	case C_SETUID:
	  if (mproc[who].mp_effuid==SUPER_USER)
	    {
	      mproc[who].mp_realuid=mm_in.MM_SUID;
	      tell_fs(C_SETUID, who, mproc[who].mp_realuid, mproc[who].mp_effuid);
	    } else return ENORIGHTS;
	  break;
	case C_SETGID:
	  if (mproc[who].mp_effuid==SUPER_USER)
	    {
	      mproc[who].mp_realgid=mm_in.MM_SGID;
	      tell_fs(C_SETGID, who, mproc[who].mp_realgid, mproc[who].mp_effgid);
	    } else return ENORIGHTS;
	  break;
	case C_GETPGRP:
          prcs=(mm_in.MM_GPGRPPID==0)?who:FindIndex(mm_in.MM_GPGRPPID); 
          if (prcs==0) return EInvalidPID;
          mm_out.MM_PGRP=mproc[prcs].mp_procgrp;
          break;
	case C_SETPGRP:
          prcs=(mm_in.MM_SPGRPPID==0)?who:FindIndex(mm_in.MM_SPGRPPID); 
	  if (prcs==0) return EInvalidPID;
          if ((mproc[who].mp_effuid==SUPER_USER)||
              (mproc[prcs].mp_effuid==mproc[who].mp_effuid)||
	      (who==mproc[prcs].mp_parent))
	       mproc[prcs].mp_procgrp=mm_in.MM_SPGRPVALUE;
          else return ENORIGHTS;
          break;
	default:
	  return  EINVAL;
    };
  return OK;
}

PRIVATE tell_fs(unsigned int msg, int aWho, int realValue, int effValue)
{
/****************************************************************
 Poslat zpravu FS
 ****************************************************************/

#if ((TEST_PHASE>MAX_PHASE_KERNEL_MM) || (TEST_PHASE==0) \
      || defined(LOAD_FS_MODULE))

  mm_out.m_source=MM;
  if (msg==C_SETUID) mm_out.m_type=C_SETUID;
  else if (msg==C_SETGID) mm_out.m_type=C_SETGID;
       else return EINVAL;

  mm_out.fs_value_real=realValue;
  mm_out.fs_value_eff=effValue;
  mm_out.fs_proc=aWho;
  send_recv(FS,mm_out);

#endif

  return 0;
}