/*
 ****************************************************************************
 * soubor: main.c                                                           *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

/* This file contains the main program of the memory manager and some related
 * procedures.  When TUOX starts up, the kernel runs for a little while,
 * initializing itself and its tasks, and then it runs MM and FS.

 MM at this point does not know where FS is in memory and how big it is.
 By convention, FS starts at the click following MM, so MM can deduce
 where it starts at least. Later, when FS runs for a first time, it makes 
 a pseudo-call BRK2, to tell how big it is. This allows MM to figure out
 where INIT is.

 *
 * The entry points into this file are:
 *   main:      starts MM running
 *   reply:     reply to a process making an MM system call
 */

/*===========================================================================*
 *                              main                                         *
 *===========================================================================*/

#include "COMMON\com.h"
#include "COMMON\callnr.h"
#include "kernel\makra.h"
#include "test.cnf"

#include "mm\mm.h"
#include "mm\alloc.h"
#include "mm\mm_glob.h"
#include "mm\table.h"
#include "mm\reply.h"

#include "lib\include\string.h"

//#include "..\..\bootstrap\sources\s.h"

PRIVATE void get_work(void);
PRIVATE void mm_init(phys_clicks h_base);


/* h_base je prvni volny segment v pameti */

#if (TEST_PHASE<=MAX_PHASE_KERNEL_MM && TEST_PHASE!=0)
 extern Word k_code_segment,k_data_segment;
#endif

#if (TEST_PHASE>MAX_PHASE_KERNEL_MM || TEST_PHASE==0)
PUBLIC void main(phys_clicks h_base)
#else
PUBLIC void mm_main(phys_clicks h_base)
#endif
/* h_base - zacatek volne pameti */
{
/* Main routine of the memory manager. */
  int error;
  mm_init(h_base);                    /* initialize memory manager tables */
  while (TRUE) {
	get_work();             /* wait for an MM system call */
	mp = &mproc[who];

	error = OK; dont_reply = FALSE;
	   /* globalni, behem zpracovani se muze zmenit */
 /*	Over platnost cisla volani, pokud platne: */
	    error = (*call_vector[mm_call])();

	/* Send the results back to the user to indicate completion. */
	if (dont_reply) continue;       /* no reply for EXIT and WAIT */
	if (mm_call == C_EXEC && error == OK) continue;
	reply(who, error);
  }
}


/*===========================================================================*
 *                              get_work                                     *
 *===========================================================================*/
PRIVATE void get_work()
{
/* Wait for the next message and extract useful information from it. */

//  if (receive(ANY, mm_in)!=OK) panic("MM receive error", NO_NUM);
  receive(ANY, mm_in)
  who = mm_in.m_source;         /* who sent the message */
  mm_call = mm_in.m_type;       /* system call number */
}




/*===========================================================================*
 *                              mm_init                                      *
 *===========================================================================*/
PRIVATE void mm_init(phys_clicks h_base)
{
  /* Initialize the memory manager. h_base je prvni volny segment (click) */
  int i;

  for (i=0; i<NR_TASKS+NR_SERVS; i++) mproc[i].mp_flags = IN_USE;
  for (i=INIT+1; i<NR_TASKS+NR_SERVS+NR_PROCS; i++) mproc[i].mp_flags = 0;

  /* slot procesu INIT musi byt zcela inicializovan, z nej se vsechny ostatni */
  /* kopiruji */
  mproc[INIT].mp_flags = IN_USE;

  #if (TEST_PHASE<=MAX_PHASE_KERNEL_MM && TEST_PHASE!=0)
   mproc[INIT].mp_seg[D].mem_phys=k_data_segment;
   mproc[INIT].mp_seg[D].mem_vir=0;
   mproc[INIT].mp_seg[D].mem_len=(0xfff0-JOINT_DEBUG_STACK_SIZE-JOINT_DEBUG_GAP_SIZE)/CLICK_SIZE;

   mproc[INIT].mp_seg[T].mem_phys=k_code_segment;
   mproc[INIT].mp_seg[T].mem_vir=0;
   mproc[INIT].mp_seg[T].mem_len=k_data_segment-k_code_segment;

   mproc[INIT].mp_seg[S].mem_phys=k_data_segment;
   mproc[INIT].mp_seg[S].mem_vir=(0xfff0-JOINT_DEBUG_STACK_SIZE);
   mproc[INIT].mp_seg[S].mem_len=JOINT_DEBUG_STACK_SIZE / CLICK_SIZE;

   mproc[MM].mp_seg[D].mem_phys=k_data_segment;
   mproc[MM].mp_seg[D].mem_vir=0;
   mproc[MM].mp_seg[D].mem_len=0xfff0/16;

   mproc[MM].mp_seg[T].mem_phys=k_code_segment;
   mproc[MM].mp_seg[T].mem_vir=0;
   mproc[MM].mp_seg[T].mem_len=k_data_segment-k_code_segment;

   mproc[MM].mp_seg[S].mem_phys=k_data_segment;
   mproc[MM].mp_seg[S].mem_vir=0;
   mproc[MM].mp_seg[S].mem_len=VERY_BIG;

  #else
   #error main.c: mm_init() - dopsat inicializaci mproc[INIT] podle zavadece
  #endif

  mproc[INIT].mp_exitstatus=mproc[INIT].mp_sigstatus=0;
  mproc[INIT].mp_pid=INIT_PID;
  mproc[INIT].mp_parent=INIT; /* zacykleni vazby parent v koreni sam na sebe */
  mproc[INIT].mp_procgrp=INIT_PROCGRP;
  mproc[INIT].mp_waitfor=0;
  mproc[INIT].mp_realuid=mproc[INIT].mp_effuid=0;
  mproc[INIT].mp_realgid=mproc[INIT].mp_effgid=0;

  mproc[INIT].mp_ignore=0;  /* 1 means ignore the signal, 0 means don't */
  mproc[INIT].mp_catch=0;   /* 1 means catch the signal, 0 means don't */
  mproc[INIT].mp_func=NULL; /* all signals vectored to a single user fcn */

  strcpy(mproc[INIT].mp_filename,"INIT");

  procs_in_use=1;

  /* Vybuduj mapu pameti */
  /* konec realne pameti: 640 kB (512+128kB=0xA0000 - segment A000 */

  mem_init(h_base,0xA000-h_base /* delka volne pameti v paragrafech */ );
 /* (set stack limit, ktery se testuje pri kazdem volani) not implemented  */
 /* ********************************************************************** */
}