/* Helper Functions to Call FS Services (used in MM) */

#include "mm\mmtofs.h"
#include "COMMON\callnr.h"
#include "COMMON\const.h"
#include "COMMON\stat.h"
#include "kernel\makra.h"
#include "fs\common\fsmes.h"
#include "mm\mproc.h"
#include "test.cnf"

// Vsechny funkce vraci chybu, ktera se predava knihovnam (nebo 0 pokud OK)
/*
 podpora pro debugovani: EXEC image se nebere ze souboru (dokud neni k MM
 zaveden i FS), ale z oblasti pameti za DS (delky EXECINMEM_SIZE), kam se
 (DOSovym EXE zavadecem) zavede soubor EXEC_INMEM_FILE.
*/


#if (EXECINMEM_SIZE!=0)
 #define EXECINMEM_HANDLE 32000
 static unsigned long memFilePos;
  /* pozice v EXE image v pameti */
 extern Word execinmem_size;
  /* skutecna velikost EXE image v pameti */
 extern Word k_data_segment;

 #include "kernel\klib88.h"
#endif

static message m;
static struct stat fstatus;

/* Over, jestli je zadany soubor spustitelny */

int test_exec(char* filename) {
#if (EXECINMEM_SIZE!=0)
 return 0;
#else
   m.m_type=C_ACCESS;
   m.fs_name=filename;
   m.fs_mode=S_IEXEC;
   send_recv(FS,m);
   return m.fs_errcode; /* 0=OK */
#endif
}


int open_file(char* filename, int* fd) {
#if (EXECINMEM_SIZE!=0)
   memFilePos=0L;
   *fd=EXECINMEM_HANDLE;
   return 0;
#else
   m.m_type = C_OPEN;
   m.fs_name = filename;
   m.fs_mode = S_IREAD;

   send_recv(FS,m);
   if (m.fs_fd>=0) { *fd=m.fs_fd; return 0; }
    else return m.fs_fd;
#endif
}

int close_file(int fd) {
#if (EXECINMEM_SIZE!=0)
   return 0;
#else
   m.m_type = C_CLOSE;
   m.fs_fd = fd;
   send_recv(FS,m);
   return m.fs_errcode;
#endif
}

int read_file_mm(int fd, char*  buf, int len, procnr_t toProc, int seg) {
#if (EXECINMEM_SIZE!=0)
   Word restingb;
   if (memFilePos+len>execinmem_size) len=execinmem_size-memFilePos;
   k_phys_copy(((DWord)(k_data_segment+0x1000))*16L+memFilePos,
	       ((DWord) mproc[toProc].mp_seg[seg].mem_phys)*16L+(unsigned int) buf,
	       len);
   memFilePos+=len;
   return len;
#else
   m.m_type = C_READ;
   m.fs_fd = fd;
   m.fs_buffer=buf;
   m.fs_nbytes=len;
   m.fs_seg=seg | (toProc<<SEG_MASK_SHIFT);
   send_recv(FS,m);
   return m.fs_errcode;
#endif
}


int get_setuid(char* filename, int* bit) {
#if (EXECINMEM_SIZE!=0)
   *bit=0; return 0;
#else
   m.m_type = C_STAT;
   m.fs_name = filename;
   m.fs_buff=(char*) &fstatus;
   send_recv(FS,m);
   if (m.fs_errcode) return m.fs_errcode;
   if (fstatus.st_mode & S_ISUID) *bit=1; else *bit=0;
   return 0;
#endif
}

int get_setgid(char* filename, int* bit) {
#if (EXECINMEM_SIZE!=0)
   *bit=0; return 0;
#else
   m.m_type = C_STAT;
   m.fs_name = filename;
   m.fs_buff=(char*) &fstatus;
   send_recv(FS,m);
   if (m.fs_errcode) return m.fs_errcode;
   if (fstatus.st_mode & S_ISGID) *bit=1; else *bit=0;
   return 0;
#endif
}

int get_file_owner(char* filename, uid_t* owner) {
#if (EXECINMEM_SIZE!=0)
   *owner=0; /* root */
   return 0;
#else
   m.m_type = C_STAT;
   m.fs_name = filename;
   m.fs_buff=(char*) &fstatus;
   send_recv(FS,m);
   if (m.fs_errcode) return m.fs_errcode;
   *owner=fstatus.st_uid;
   return 0;
#endif
}

int get_file_owner_g(char* filename, gid_t* owner_g) {
#if (EXECINMEM_SIZE!=0)
   *owner_g=0; /* root */
   return 0;
#else
   m.m_type = C_STAT;
   m.fs_name = filename;
   m.fs_buff=(char*) &fstatus;
   send_recv(FS,m);
   if (m.fs_errcode) return m.fs_errcode;
   *owner_g=fstatus.st_gid;
   return 0;
#endif
}

/*
 pozn: FS:
 FS/syscall/misc.c
   do_fork() C_FORK(fs_parent,fs_child)
   do_exit() C_EXIT(fs_proc)
    vraci: fs_errcode
 FS/syscall/stadir.c
  do_setuid() C_SETUID(fs_proc,fs_value_real,fs_value_eff)
  do_setgid() C_SETGID(fs_proc,fs_value_real,fs_value_eff)

 FS/device/pipe.c
  _C_UNPAUSE(fs_proc)
*/