/*
 ****************************************************************************
 * soubor: mproc.h                                                          *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

#ifndef MPROC_H
#define MPROC_H

/* This table has one slot per process.  It contains all the memory management
 * information for each process.  Among other things, it defines the text, data
 * and stack segments, uids and gids, and various flags.  The kernel and file
 * systems have tables that are also indexed by process, with the contents
 * of corresponding slots referring to the same process in all three.
 */
#include "mm\mm_const.h"
//#include "..\..\fs\common\fsmes.h"


extern struct mproc {
  struct mem_map mp_seg[NR_SEGS];/* points to text, data, stack */
  char mp_exitstatus;           /* storage for status when process exits */
  char mp_sigstatus;            /* storage for signal # for killed procs */
  pid_t mp_pid;                 /* process id */
  pid_t mp_procgrp;             /* pid of process group (used for signals) */
  int mp_parent;                /* index of parent process */

  pid_t mp_waitfor;             /* na koho presne ceka */

  /* Real and effective uids and gids. */
  uid_t mp_realuid;             /* process' real uid */
  uid_t mp_effuid;              /* process' effective uid */
  gid_t mp_realgid;             /* process' real gid */
  gid_t mp_effgid;              /* process' effective gid */


  /* Signal handling information. */
  sigset_t mp_ignore;           /* 1 means ignore the signal, 0 means don't */
  sigset_t mp_catch;            /* 1 means catch the signal, 0 means don't */
  sighandler_t mp_func;             /* all signals vectored to a single user fcn */

  unsigned mp_flags;            /* flag bits */

  char mp_filename[PATH_MAX+1]; /* nazev souboru */

} mproc[NR_PROCS+NR_TASKS+NR_SERVS];

/* Flag values */
#define IN_USE           001    /* set when 'mproc' slot in use */
#define HANGING          002    /* set by EXIT system call */
#define PAUSED           004    /* set by PAUSE system call */
#define ALARM_ON         010    /* set when SIGALRM timer started */
#define WAITING          020    /* set by WAIT system call */
/* Pridano pro cekani na konkretni process (id nebo group) */
#define WAITINGFOR      0120    /* Cekam na konkretni dite: mp_waitfor */
#define WAITINGGROUP    0220    /* Cekam na dite ze skupiny: mp_waitfor */
 /* Vsechny stavy skupiny WAITING maji jeden spolecny bit
    - pro snadne testovani */

#define NIL_MPROC ((struct mproc *) 0)
#endif

/*
Nasledne zde budou symbolicky pojmenovane cleny unionu message
pro parametry tech zprav, ktere MM  zpracovava */
