/*
 ****************************************************************************
 * soubor: signal.c                                                         *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

/* Version 1.1

/*            >>>>>>>>>>>>>>> SIGNAL.C  <<<<<<<<<<<<<<<<<<<<
 * This file handles signals, which are asynchronous events and are generally
 * a messy and unpleasant business.  Signals can be generated by the KILL
 * system call, or from the keyboard (SIGINT) or from the clock (SIGALRM).
 * In all cases control eventually passes to check_sig() to see which processes
 * can be signaled.  The actual signaling is done by sig_proc().
 *
 * The entry points into this file are:
 *   do_signal: perform the SIGNAL system call
 *   do_kill:   perform the KILL system call
 *   do_ksig:   accept a signal originating in the kernel (e.g., SIGINT)
 *   do_alarm:  perform the ALARM system call by calling set_alarm()
 *   set_alarm: tell the clock task to start or stop a timer
 *   do_pause:  perform the PAUSE system call
 *   unpause:   check to see if process is susended
 */

#include "COMMON\callnr.h"
#include "COMMON\signal.h"
#include "COMMON\com.h"
#include "kernel\makra.h"
#include "clock\h_clock.h"
#include "systask\syst_pub.h"
#include "mm\mm_signl.h"
#include "mm\mmerror.h"
#include "mm\mm_glob.h"
#include "mm\mproc.h"
#include "mm\mm.h"
#include "mm\reply.h"
#include "test\trap.h"
#include "mm\mm2fsmsg.h"
#include "test.cnf"



/*===========================================================================*
 *                               do_signal                                   *
 *===========================================================================*/
PUBLIC int do_signal() /*PUBLIC*/
{ sigset_t *PSignalNastav,*PSignalZrus;
  int Maska=1;

/*Obsluha vola signal(sig,func=SIG_IGN | SIG_DFL | ptr_na_funkci)*/
/*Over platnost cisla signalu*/
  if (mm_in.MM_SIGNAL > 16 || mm_in.MM_SIGNAL < 1) { return ErrSigOutRange;}

/*SIGKILL: nemuze byt chytan ani ignorovan
Nastaveni prislusneho bitu v mp_ignore, resp. mp_catch,
resp. nastaveni mp_func*/
  if (mm_in.MM_SIGNAL != SIGKILL)
  { Maska<<= mm_in.MM_SIGNAL-1; /*Priprava bitove masky (jednickovy je ten bit, ktery ma index MM_SIGNAL*/
    if (mm_in.MM_PTRFUNC != SIG_DFL)
    { if (mm_in.MM_PTRFUNC  == SIG_IGN)
      { PSignalNastav= &mp->mp_ignore; /*Ukazatel na promennou, kde se ma prislusny bit nastavit na 1*/
        PSignalZrus= &mp->mp_catch;   /*Ukazatel na promennou, kde se ma prislusny bit nastavit na 0*/
      }
      else      /*Jedna se o ptr_na_funkci a ma se nastavit mp_catch*/
      { PSignalNastav= &mp->mp_catch;
        PSignalZrus= &mp->mp_ignore;
        mp->mp_func= mm_in.MM_PTRFUNC;
      }
      /*nastavit prislusny bit */

      *PSignalNastav|= Maska;
      *PSignalZrus&= ~ Maska;
    }
    else {/*Jedna se o SID_DFL ... vynulovat mp_catch i mp_ignore*/
      mp->mp_catch&= ~ Maska;
      mp->mp_ignore&= ~ Maska;
      }

  }


  return OK;
}


/*===========================================================================*
 *                              do_kill                                      *
 *===========================================================================*/
PUBLIC int do_kill() /* PUBLIC */
{
/* Perform the kill(pid, signo) system call. */

  return check_sig(mm_in.MM_PID, mm_in.MM_SIGNAL);
}


/*===========================================================================*
 *                              do_ksig                                      *
 *===========================================================================*/
PUBLIC int do_ksig()    /* PUBLIC */
{ /*Signaly - bitove pole signalu, ktere se maji provest*/
  /*Proc_nr - cislo (index) procesu o ktery se jedna      */
  int Maska= 1, Err,i;
  pid_t Pid;

  sigset_t Signaly; int Proc_nr;

  Proc_nr= mm_in.MM_PID;
  Signaly= (sigset_t) mm_in.MM_SIGBMP;

/* Certain signals, such as segmentation violations and DEL, originate in the
 * kernel.  When the kernel detects such signals, it sets bits in a bit map.
 * As soon as MM is awaiting new work, the kernel sends MM a message containing
 * the process slot and bit map.  That message comes here.  The File System
 * also uses this mechanism to signal writing on broken pipes (SIGPIPE).
 */

 /*Test, zda zpravu odeslal kernel (HARDWARE) nebo FS (FS_PROC_NR) nebo SYSTASK
   ci CLOCK (ten totiz vola funkci ze systask) */
  if ((who == HARDWARE)||(who == FS)||(who == SYSTASK)||(who==CLOCK))    /* who je  proc_nr*/
  {
 /*Zjisti, neni-li slot uz procesu !IN_USE nebo HANGING*/
    if ((mproc[Proc_nr].mp_flags & IN_USE)&&
	(!(mproc[Proc_nr].mp_flags & HANGING)))
    {
  /* Projed bity bitoveho pole a vyrid jednotlive signaly (check_sig()).
      (Muze byt nastaveno i vice bitu).
      Pro SIGINT a SIGQUIT pouzij pri volani pid=0, muze byt signalizovano
      i vice procesum  */

      for(i= 1; i < 16; i++)
      { if (Signaly & Maska)
	{ if (i == SIGINT || i == SIGQUIT) Pid= 0;
	  else Pid= mproc[Proc_nr].mp_pid;
	  if ((Err=check_sig(Pid, i))!=0) return Err;
	}
	Maska<<= 1;
      }
    }
    else return ErrProcNotUse;
  }
  else return ErrProcNotFSorKERNEL;

  dont_reply=1; /* zpravu nam poslal CLOCK pomoci send(), neceka odpoved */
  return OK;
}

/*===========================================================================*
 *                              check_sig                                    *
 *===========================================================================*/
PUBLIC int check_sig(pid_t proc_id, int signo) /* PUBLIC */
{ /*pid_t proc_id;                  /* pid of proc to sig, or 0 (process group) */
                                /* or -1: all processes (superuser only) */
  /*int signo;                      /* signal to send to process (1 to 16) */
  int i, Nalezen= 0, Maska= 1;
/* Check to see if it is possible to send a signal.  The signal may have to be
 * sent to a group of processes.  This routine is invoked by the KILL system
 * call, and also when the kernel catches a DEL or other signal.
 */

 /*Najdi v tabulce procesu ty, kterym se ma signalizovat
 Musi platit:
  sender's uid=proc's uid nebo je sender superuser
  proces jeste neskoncil */
  Maska<<= signo - 1;
  for(i= INIT; i < NR_PROCS+NR_TASKS+NR_SERVS; i++)
  { if ((!(mproc[i].mp_flags & HANGING))&&(mproc[i].mp_flags & IN_USE))
    { if ((mp->mp_effuid == mproc[i].mp_effuid)||(mp->mp_effuid == SUPER_USER))
      { if (proc_id == 0)
	{ /*  je-li proc_id=0: signalizovat kazdemu z process group krome volajiciho*/
	  if ((mproc[i].mp_procgrp == mp->mp_procgrp) &&
              (mproc[i].mp_pid != proc_id))
          { /* Jde-li o SIGALRM, vypni ALARM_ON v mp_flags               */
            if (signo == SIGALRM) mproc[i].mp_flags= mproc[i].mp_flags & (~ ALARM_ON);
            /*Over, nema-li se signal pro proces ignorovat*/
            if ((mproc[i].mp_ignore & Maska) == 0)
            {  /*pokud je proces waiting, unpause()*/
	      if ((mproc[i].mp_flags & WAITING) || (mproc[i].mp_flags & PAUSED)) unpause(i);
              /* Doruc signal: send_proc()*/
              sig_proc(i, signo);
              Nalezen= 1;
            }
          }
	}
        else /* je_li proc_id>0: musi se pid shodovat*/
        { if (proc_id == mproc[i].mp_pid)
          { /* Jde-li o SIGALRM, vypni ALARM_ON v mp_flags               */
	    if (signo == SIGALRM) mproc[i].mp_flags= mproc[i].mp_flags & (~ ALARM_ON);
            /*Over, nema-li se signal pro proces ignorovat*/
            if ((mproc[i].mp_ignore & Maska) == 0)
            {  /*pokud je proces waiting, unpause()*/
	      if ((mproc[i].mp_flags & WAITING) || (mproc[i].mp_flags & PAUSED)) unpause(i);
              /* Doruc signal: sig_proc()*/
              sig_proc(i, signo);
              Nalezen= 1;
            }
            break;
	  }

        }
      }
    }
  }
  if (Nalezen == 0)  /*nebyl nalezen zadny odpovidajici proces*/
    return ErrProcNotFind;

  /*  Pokud proces signalem killnul sam sebe, neodpovidej*/
  if ((mp->mp_flags & IN_USE==0) || (mp->mp_flags & HANGING)) dont_reply=1;
  return OK;
}



/*===========================================================================*
 *                              sig_proc                                     *
 *===========================================================================*/
PUBLIC void sig_proc(int proc_nr, int signo) /* PUBLIC */
{ /*register struct mproc *rmp;     /* pointer to the process to be signaled */
  /*int signo;                      /* signal to send to process (1 to _NSIG) */
  int Maska= 1, i;
/* Send a signal to a process.  Check to see if the signal is to be caught.
 * If the signal is to be caught, coordinate with
 * KERNEL to push a pc,psw and signal # onto
 * the catcher's stack.
 * If there is insufficient stack space, kill the process.
 */
 /*Over zda proces stale existuje */
 if (mproc[proc_nr].mp_flags & IN_USE)
 { /*Zjisti, zda signal ma byt odchytavan.   */
   Maska<<= signo - 1;
   if (mproc[proc_nr].mp_catch & Maska)
   { /*Pokud ano {
     zrus v masce mp_catch;*/
     mproc[proc_nr].mp_catch= mproc[proc_nr].mp_catch & (~ Maska);
     /*  nachystej stack frame pro signal, posli SYS_SIG*/
     mm_out.m_source= MM;
     mm_out.m_type= SYS_SIG;
     mm_out.PROC= proc_nr;/*proc_nr */
     mm_out.SIG_NR= signo;
     mm_out.HANDLER=mproc[proc_nr].mp_func;
     send_recv(SYSTASK,mm_out);
     if (mm_out.RET_ERROR_CODE!=0) trap(TRAP_SIG_STACK);
   }
   else
   { /* else: default obsluha:
	prirazeni mp_sigstatus v mproc[],*/
     mproc[proc_nr].mp_sigstatus= signo; /**/
    /*    pripadny core dump
	exit*/
     dump_core(&mproc[proc_nr]);
   }
 }
}

/*===========================================================================*
 *                              do_alarm                                     *
 *===========================================================================*/
PUBLIC int do_alarm()   /* PUBLIC */
{
/* Perform the alarm(seconds) system call. */

  return(set_alarm(who, mm_in.MM_ALARMSECOND));
}



/*===========================================================================*
 *                              set_alarm                                    *
 *===========================================================================*/
PUBLIC int set_alarm(int proc_nr, int sec)   /* PUBLIC */
{ /*int proc_nr;                    /* process that wants the alarm */
  /*int sec;                        /* how many sec delay before the signal */

/* This routine is used by do_alarm() to set the alarm timer.  It is also used
 * to turn the timer off when a process exits with the timer still on.
 */

 /*Nastav mproc[proc_nr].mp_flags na ALARM_ON, je-li sec nenulovy;   */
  if (sec != 0)
  { mproc[proc_nr].mp_flags|= ALARM_ON;

/* Posli zpravu SET_ALARM Clock tasku (ten pozdeji iniciuje SIG_ALRM)*/

    mm_out.m_source= MM;
    mm_out.m_type= SET_ALARM;
    mm_out.CLOCK_PROC= proc_nr;
    mm_out.FUNC= mproc[proc_nr].mp_func;
    mm_out.NEW_TIME= sec;
    send_recv(CLOCK,mm_out);
    if (mm_out.RET_ERROR_CODE!=0) trap(TRAP_SET_ALARM);

  }
  return OK;
}


/*===========================================================================*
 *                              do_pause                                     *
 *===========================================================================*/
PUBLIC int do_pause()     /* PUBLIC */
{
/* Perform the pause() system call. */

  mp->mp_flags |= PAUSED;
 /* Neodpovidej procesu*/
  dont_reply=1;
  return OK;
}




/*===========================================================================*
 *                              unpause                                      *
 *===========================================================================*/
PRIVATE void unpause(int proc_nr)   /* PRIVATE */
{ /*int proc_nr;                        /* which process number */

/* A signal is to be sent to a process.  If that process is hanging on a
 * system call, the system call must be terminated with EINTR.  Possible
 * calls are PAUSE, WAIT, READ and WRITE, the latter two for pipes and ttys.
 * First check if the process is hanging on an MM call.  If not, tell FS,
 * so it can check for READs and WRITEs from pipes, ttys and the like.
 */

 /*Cela-li proces v pause() - flags=PAUSED: zrus flag, reply(EINTR)
 Cela-li proces ve wait() - flags=WAITING: zrus flag, reply(EINTR)
 posli UNPAUSE FS, ceka se v jeho volani*/
 if ((mproc[proc_nr].mp_flags & PAUSED)||(mproc[proc_nr].mp_flags & WAITING))
 { if (mproc[proc_nr].mp_flags & PAUSED)
   { mproc[proc_nr].mp_flags&= ~PAUSED;
   }
   else
   { mproc[proc_nr].mp_flags&= ~WAITING;
   }

   reply(proc_nr,EINTR);

#if ((TEST_PHASE>MAX_PHASE_KERNEL_MM) || (TEST_PHASE==0) \
      || defined(LOAD_FS_MODULE))
   mm_out.fs_proc=proc_nr;
   mm_out.m_type=_C_UNPAUSE;
   send_recv(FS,mm_out);
#endif
 }

}


/*===========================================================================*
 *                              dump_core                                    *
 *===========================================================================*/
PRIVATE void dump_core(register struct mproc *rmp)   /* PRIVATE */
{ /*register struct mproc *rmp;     /* whose core is to be dumped */

 // DOPSAT
}
