/*
 ****************************************************************************
 * soubor: table.c                                                          *
 * autor:  Marcel Kruzel, Zdenek Sawa, Milan Gvozdik                        *
 * ladici: Rojicek Radim, Sela Jaroslav                                     *
 ****************************************************************************
*/

#include "COMMON\type.h"
#include "COMMON\const.h"

#include "mm\mproc.h"
#include "mm\table.h"
#include "mm\getset.h"
#include "mm\forkexit.h"
#include "mm\break.h"
#include "mm\exec.h"
#include "mm\mm_signl.h"
#include "mm\getmproc.h"


/*
   This file contains the table used to map system call numbers onto the
   routines that perform them.
   Calls assigned to no_sys routine are serviced by MM
 */

int no_sys() { return 0; }

PUBLIC int (*call_vector[NCALLS])(void)  = {
	no_sys,         /*  0 = unused  */

	do_mm_exit,      /*  1 = exit    */
	do_fork,        /*  2 = fork    */
	do_exec,        /*  3 = exec    */
	do_brk,         /*  4 = break   */

	no_sys,		/*  5 = read    */
	no_sys,         /*  6 = write   */
	no_sys, 	/*  7 = open    */
	no_sys,       	/*  8 = close   */
	no_sys,		/*  9 = lseek   */
	no_sys, 	/* 10 = creat   */
	no_sys,         /* 11 = mknod   */
	no_sys,         /* 12 = link    */
	no_sys,		/* 13 = unlink  */
	no_sys,         /* 14 = stat    */
	no_sys,         /* 15 = fstat   */
	no_sys,         /* 16 = ioctl   */
 	no_sys,         /* 17 = utime   */
	no_sys,         /* 18 = mount   */
	no_sys,         /* 19 = umount  */
	no_sys,         /* 20 = access  */
	no_sys,         /* 21 = dup     */
	no_sys,         /* 22 = pipe - not implemented   */

	do_wait,        /* 23  = waitpid,wait*/
	no_sys,         /* 24 = UNUSED */

	no_sys,         /* 25 = chmod   */
	no_sys,         /* 26 = chown   */
	no_sys,         /* 27 = chdir   */
	no_sys,         /* 28 = chroot  */
	no_sys,         /* 29 = umask   */

	do_getset,      /* 30 = getpid  */
	do_getset,      /* 31 = getpgrp */
	do_getset,      /* 32 = setpgrp */
	do_getset,      /* 33 = getuid  */
	do_getset,      /* 34 = setuid  */
	do_getset,      /* 35 = getgid  */
	do_getset,      /* 36 = setgid  */


	no_sys,         /* 37 = time    */
	no_sys,         /* 38 = stime   */

	do_signal,      /* 39 = signal  */
	do_kill,        /* 40 = kill    */
	do_alarm,       /* 41 = alarm   */
	do_pause,       /* 42 = pause   */

	no_sys,         /* 43 = times   */
	no_sys,         /* 44 = sync    */

	do_ksig,        /* 45 = KSIG: signals originating in the kernel */
	no_sys,         /* 46 = UNPAUSE */
	no_sys,         /* 47 = REVIVE  */
	do_getmproc	/* 48 = getproctbl */
};


