/* Projekt TUOX: DOS Loader. Slouzi k zavedeni modulu serveru do pameti.
   Spousti se z jadra ve tvaru EXE urceneho pro debugovani systemu
   jako programu DOS.

   Protoze se ladici k zadne akci nevzchopili, napsal Petr Grygarek

   ! Modul predpoklada, ze je k dispozici DOS pres INT21h !

   POZOR: interpretace mem_len (mem_vir) v pametove mape ma byt v paragrafech,
	  zde nezkontrolovano !
*/
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "kernel\proc_pub.h"
#include "test\tuoxhead.h"

extern Word k_dgroup;
extern Word k_bdata;
extern Word k_edata;

static char buf[BLOCK_SIZE];

int openFile(char* fName) {
// vraci: handle (kladne cislo), jinak error kod
 int result;

 asm mov ah,0x3d /* open file */
 asm mov dx, fName /* ds:dx: ptr na filename */
 asm mov al,0 /* open for reading */
 asm int 0x21
 asm mov result,ax
 asm jc open_err

 asm jmp open_end
open_err:
 result*=-1;

open_end:
 return result;
}

int closeFile(int handle) {
 // vraci: 0=OK, jinak error code (zaporne)
 int result=0;

 asm mov ah,0x3e /* close file */
 asm mov bx, handle
 asm int 0x21
 asm jc close_err

 return 0;

close_err:
 asm mov result,ax
 result*=-1;
 return result;
}

#define ERR_COUNT_TOO_BIG	(-100)
#define ERR_BAD_TUOX_MAGIC	(-101)

int readFile(int handle, char* buf, Word count, int* errNo) {
 // vraci: pocet prectenych bytes nebo -1 pri chybe (pak nastavi errNo)
 // count je maximalne 65534 (aby se nepletlo s -1)

 int result=0;

 if (count>65534) { *errNo=ERR_COUNT_TOO_BIG; return -1; }

 asm mov ah,0x3f /* read file */
 asm mov bx,handle
 asm mov dx, buf /* DS:DX - buffer */
 asm mov cx,count
 asm int 0x21
 asm jc read_err
 asm mov result,ax
 *errNo=0;
 return result;

read_err:
 asm mov result,ax
 *errNo=result*(-1);
 return -1;
}

int seekFile(int handle, unsigned long pos) {
 // vraci: 0=OK, jinak error code (zaporne)
 int result=0;

 asm mov ah,0x42 /* lseek */
 asm mov bx, handle
 asm mov cx,word ptr pos /* cx:dx - pos */
 asm mov dx,word ptr (pos+2)
 asm mov al,0 /* from start */
 asm int 0x21
 asm jc seek_err

 return 0;

seek_err:
 asm mov result,ax
 result*=-1;
 return result;
}

int readTuoxHeader(int handle) {
 // vraci: 0=OK, jinak zaporny chybovy kod
 int res;
 long magic;

 if ((res=seekFileDOS(handle,0L))!=0) return res;

 if (readFile(handle,buf,TUOX_HEADER_LEN,&res)==-1) return res;
 magic= *((long*) buf);
 if (magic!=TUOX_MAGIC) return ERR_BAD_TUOX_MAGIC;

 return 0;
}

#define MK_FP( seg,ofs )( (void _seg * )( seg ) +( void near * )( ofs ))
int loadSegment(int handle, long fOffset, Word seg, Word len) {
// vraci: 0=OK, jinak zaporny error kod z DOSu
 int res,cnt,i;
 char far* p=MK_FP(seg,0);

 if ((res=seekFile(handle,fOffset))!=0) return res;

 while (len) {
	 cnt=len<BLOCK_SIZE ? len : BLOCK_SIZE;
	 if (readFile(handle,buf,cnt,&res)!=cnt) return res;
	 for (i=0; i<cnt; i++) *p++=buf[i];
	 len-=cnt;
 }
 return 0;
}

// predpoklada, ze pozice v proc[] se obsazuji po rade od pozice 1. serveru

int loadModule(char* fName, int procPos) {
// vraci: 0=OK, jinak zaporny error kod z DOSu

 Word d_seg,c_seg,c_len,d_len;
 int handle,r;

 if ((handle=openFile(fName))<0) return handle;
 if ((r=readTuoxHeader(handle))!=0) { closeFile(handle); return r; }

 if (procPos==NR_TASKS) {
  // Prvni modul. Zjisti, kde konci jadro
  // Volna pamet za kernelem (segment): k_dgroup+(k_edata / 16)+1
  c_seg=k_dgroup+(k_edata / 16)+1;
 }
  else c_seg=proc[procPos-1].p_map[D].mem_phys+(proc[procPos-1].p_map[D].mem_len/16) + 1;

 c_len=*((Word*) (buf+TH_TextLen));
 d_len=*((Word*) (buf+TH_Total))-*((Word*) (buf+TH_TextLen));
 d_seg=c_seg+(c_len/16)+1;

 proc[procPos].p_map[T].mem_phys=c_seg;
 proc[procPos].p_map[T].mem_len=c_len;
 proc[procPos].p_map[T].mem_vir=0;

 proc[procPos].p_map[D].mem_phys=d_seg;
 proc[procPos].p_map[D].mem_len=d_len;
 proc[procPos].p_map[D].mem_vir=0;

 if ((r=loadSegment(handle,TUOX_HEADER_LEN,c_seg,c_len))!=0) return r;
 if ((r=loadSegment(handle,TUOX_HEADER_LEN+c_len,d_seg,d_len))!=0) return r;

 return closeFile(handle);
}