/* This file contains a collection of miscellaneous procedures.  Some of them
 * perform simple system calls.  Some others do a little part of system calls
 * that are mostly performed by the Memory Manager.
 *
 * The entry points into this file are
 *   do_dup:      perform the DUP system call
 *   do_sync:     perform the SYNC system call
 *   do_fork:     adjust the tables after MM has performed a FORK system call
 *   do_exit:     a process has exited; note that in the tables
 *   do_set:      set uid or gid for some process
 *   do_revive:   revive a process that was waiting for something (TTY or pipe)
 */

/*===========================================================================*
 *                              do_dup                                       *
 *===========================================================================*/
PUBLIC int do_dup()
{
/* Perform the dup(fd) or dup(fd,fd2) system call. */
Over, je-li descriptor platny
pro dup(): alokuj novy descriptor (vyhledej prazdnou pozici)
pro dup2: over rozsah fd2, pokud je na te pozici otevreny fd, zavri (do_close)
zkopiruj do noveho fd odkaz na filp[], v pozici filp[].count++
}


/*===========================================================================*
 *                              do_sync                                      *
 *===========================================================================*/
PUBLIC int do_sync()
{
  /* Perform the sync() system call.  Flush all the tables. */

  struct inode* ip;
  struct super_block* sp;

  /* The order in which the various tables are flushed is critical.  The
   * blocks must be flushed last, since rw_inode() leaves its results in
   * the block cache.
   */

  /* Write all the dirty inodes to the disk. */
  for (ip=&inode[0]; ip<&inode[NR_INODES]; ip++)
   if (ip->i_count>0 && ip->i_dirt==DIRTY) rw_inode(ip,WRITING);

  /* Write all the dirty superblocks to the disk. */
  for (sp=&super_blocks[0]; sp<&super_blocks[NR_SUPERS]; sp++)
   if (sp->s_dev != NO_DEV && sp->s_dirty==DIRTY) rw_super(sp,WRITING);

  /* Write all the dirty blocks to the disk */
  sync();
}


/*===========================================================================*
 *                              do_fork                                      *
 *===========================================================================*/
PUBLIC int do_fork()
{
/* Perform those aspects of the fork() system call that relate to files.
 * In particular, let the child inherit its parent's file descriptors.
 * The parent and child parameters tell who forked off whom. The file
 * system uses the same slot numbers as the kernel.  Only MM makes this call.
 */
  Over, ze je opravdu volano od MM

  
  fproc[child] = fproc[parent];

  /* Increase the counters in the 'filp' table (because we inherited files). */
  
  for (i = 0; i < OPEN_MAX; i++)
	if (cp->fp_filp[i] != NIL_FILP) cp->fp_filp[i]->filp_count++;

  /* Fill in new process id. It was thought out by MM */
  
  /* Record the fact that both root and working dir have another user. */
  dup_inode(cp->fp_rootdir);
  dup_inode(cp->fp_workdir);
}



/*===========================================================================*
 *                              do_exit                                      *
 *===========================================================================*/
PUBLIC int do_exit()
{
/* Perform the file system portion of the exit(status) system call. */

  /* Only MM may do the EXIT call directly. */
  /* Nevertheless, pretend that the call came from the user. */

  Pozavirej vsechny descriptory
  uvolni rootdir a workdir (put_inode)
  Byl-li suspendovan na PIPE, susp_count--;
  odsuspenduj
  }
}


/*===========================================================================*
 *                              do_set                                       *
 *===========================================================================*/
PUBLIC int do_set()
{
/* Set uid or gid field */

  /* Only MM may make this call directly. */
  if (fs_call == SETUID) { nastav realuid, effuid }
  if (fs_call == SETGID) { nastav realgid, effgid }
}


/*===========================================================================*
 *                              do_revive                                    *
 *===========================================================================*/
PUBLIC int do_revive()
{
/* A task, typically TTY, has now gotten the characters that were needed for a
 * previous read.  The process did not get a reply when it made the call.
 * Instead it was suspended.  Now we can send the reply to wake it up.  This
 * business has to be done carefully, since the incoming message is from
 * a task (to which no reply can be sent), and the reply must go to a process
 * that blocked earlier.  The reply to the caller is inhibited by setting the
 * 'dont_reply' flag, and the reply to the blocked process is done explicitly
 * in revive().
 */
 revive(m.REP_PROC_NR)
 dont_reply;
}
