/*******************************************************************
				SYSTASK.C

			Projekt TUOX, System Task

			Autori: Jiri Horacek
				Alan Kornuta
				David Stupnanek

			Upravil: Petr Grygarek, k.456
********************************************************************/


#include "COMMON\type.h"
#include "COMMON\const.h"
#include "COMMON\com.h"
#include "COMMON\errno.h"
#include "COMMON\glo.h"
#include "COMMON\signal.h"
#include "COMMON\callnr.h"

#include "kernel\makra.h"
#include "kernel\proc_pub.h"
#include "test\trap.h"
#include "mm\mm.h"

#include "systask\systask.h"
#include "test.cnf"

static message m;
static procnr_t who;
/* zdroj zpravy (kdo pozaduje sluzbu) */

PRIVATE void do_copy(message * m_ptr);
PRIVATE void do_fork(message * m_ptr);
PRIVATE void do_getsp(message * m_ptr);
PRIVATE void do_times(message * m_ptr);
PRIVATE void do_sig(message * m_ptr);
PRIVATE void do_xit(message * m_ptr);
PRIVATE void do_exec(message * m_ptr);
PRIVATE void do_newmap(message * m_ptr);

void KSIG (procnr_t proc_nr, sigset_t sigs);

/*===========================================================================*
 *                              sys_task                                     *
 *		Vstupni bod. Inicializace, hlavni smycka		     *
 *===========================================================================*/

void sys_task() {

	while (TRUE) {
		receive (ANY, m);
		who=m.m_source;

		switch (m.m_type) {
			case SYS_FORK: 		do_fork (&m); 	break;
			case SYS_NEWMAP:	do_newmap (&m);	break;
			case SYS_EXEC:		do_exec (&m);	break;
			case SYS_XIT:		do_xit (&m);	break;
			case SYS_GETSP:		do_getsp (&m);	break;
			case SYS_TIMES:		do_times (&m);	break;
			case SYS_SIG:		do_sig (&m);	break;
			case SYS_COPY:		do_copy (&m);	break;
			default: 	m.m_type=TASK_REPLY;
					m.RET_ERROR_CODE = EBADCALL;
					break;
		}
	       send(who,m);
	}
}


/*===========================================================================*
 *                              do_fork                                      *
 *	Zkopirovani informaci se slotu rodice do slotu detskeho procesu	     *
 * args (m1):								     *
 *	m_ptr.PARENT - slot procesu, ktery se stepi                           *
 *	m_ptr.CHILD - slot nove vznikajiciho procesu			     *
 *	m_ptr.PID   - PID noveho procesu 				     *
 * ret:			         					     *
 *	OK			- vse v poradku				     *
 *	ESRCH			- nektery proces je neplatny		     *
 *===========================================================================*

*/

void do_fork(message * m_ptr) {

	int i = 0;
	char *src, *dst;

	/* Zjistime platnost obou procesu v tabulce 'proc' */
	if ( !isuserp(proc_addr(m_ptr->PARENT)) ||
	     !isuserp(proc_addr(m_ptr->CHILD)) ||
		(proc[m_ptr->PARENT].p_flags & P_SLOT_FREE) ||
		(!(proc[m_ptr->CHILD].p_flags & P_SLOT_FREE))) {

		m_ptr->RET_ERROR_CODE = ESRCH;
		m_ptr->m_type = TASK_REPLY;
		return;
	}

	/* kopirovani slotu */
	dst=(char*) &(proc[m_ptr->CHILD]);
	src=(char*) &(proc[m_ptr->PARENT]);

	for (i=0; i<sizeof(struct proc); i++) *dst++=*src++;

	proc[m_ptr->CHILD].p_flags |= NO_MAP;
	proc[m_ptr->CHILD].p_pid = m_ptr->PID;

	proc[m_ptr->CHILD].user_time = 0;
	proc[m_ptr->CHILD].sys_time = 0;
	proc[m_ptr->CHILD].child_utime = 0;
	proc[m_ptr->CHILD].child_stime = 0;

	proc[m_ptr->CHILD].p_alarm = 0; /* pokud se nerozhodneme dedit alarmy */

	m_ptr->m_type = TASK_REPLY;
	m_ptr->RET_ERROR_CODE = OK;
}

/*===========================================================================*
 *                              do_newmap                                    *
 *		Prirazeni mapy pameti procesu (do proc[])		     *
 * args (m1):								     *
 *	m_ptr.PROC  		- proces, kteremu se pametova mapa prideluje *
 *	m_ptr.MEM_PTR		- ptr na tabulku pameti                      *
 *				  (mem_map[NR_SEGS] v DS volajiciho)         *
 * ret:									     *
 *	OK			- vse v poradku				     *
 *	ERSCH			- proces je neplatny			     *
 *	EFAULT			- neplatna adresa bufferu s mapou pameti     *
 *      ENOMEM 			- buffer s mapou pametu presahuje segment    *
 *===========================================================================*/

void do_newmap(message * m_ptr) {

	int err, procSlot=m_ptr->PROC;
	struct proc* procPtr=&(proc[procSlot]);

	/* Overime cislo procesu ze zpravy */

	if (!isuserp(procPtr) ||
		(procPtr->p_flags & P_SLOT_FREE)) {
		m_ptr->RET_ERROR_CODE = ESRCH;
		m_ptr->m_type = TASK_REPLY;
		return;
	}

	/* Precti mapu pameti z adresoveho prostoru odesilatele zpravy (nejspise MM)
	   a uloz do proc[proces] */

	if ((err=sys_copy(m_ptr->m_source,D,(vir_bytes) m_ptr->MEM_PTR,
			  SYSTASK,D,(vir_bytes) &(proc[procSlot].p_map),
			  NR_SEGS*sizeof(struct mem_map))) != OK) {
		m_ptr->RET_ERROR_CODE = err;
		m_ptr->m_type = TASK_REPLY;
		return;
	}
	procPtr->p_reg[DS_REG]=proc[procSlot].p_map[D].mem_phys;
	procPtr->p_reg[ES_REG]=proc[procSlot].p_map[D].mem_phys;
	procPtr->p_reg[SS_REG]=proc[procSlot].p_map[D].mem_phys;
	procPtr->p_pcpsw.cs=proc[procSlot].p_map[T].mem_phys;
	procPtr->p_pcpsw.pc=0; /* pro jistotu ... */

	procPtr->p_flags &= ~NO_MAP;
	ready(procPtr);

	m_ptr->m_type = TASK_REPLY;
	m_ptr->RET_ERROR_CODE = OK;
}


/*===========================================================================*
 *                              do_exec                                      *
 *   	Nastaveni startovaciho bodu a SP po vymene image a naplanovani	     *
 * args (m1):								     *
 *	m_ptr.PROC		- cislo procesu                              *
 *	m_ptr.NEW_SP		- nova hodnota pro stack pointer	     *
 * ret:				- ulozen v m_ptr->m1i2			     *
 *	OK			- vse v poradku				     *
 *	ESRCH 			- proces neni uzivatelsky		     *
 *===========================================================================*/

void do_exec(message * m_ptr) {

	/* Over, ze jde o user proces */

	if (!isuserp(proc_addr(m_ptr->PROC)) ||
	    (proc[m_ptr->PROC].p_flags & P_SLOT_FREE))
	{
		m_ptr->RET_ERROR_CODE = ESRCH;
		m_ptr->m_type = TASK_REPLY;
		return;
	}

	proc[m_ptr->PROC].p_alarm = 0; /* zrusime pripadne alarmy */
	proc[m_ptr->PROC].p_sp = m_ptr->NEW_SP-6;
	proc[m_ptr->PROC].p_pcpsw.pc = 0; /* novy kod startuje od adr. 0 */
	proc[m_ptr->PROC].p_pcpsw.cs = proc[m_ptr->PROC].p_map[T].mem_phys;

	/* na zasobnik musime nachystat vstupni bod ("navratovou adresu")
	   pro IRET */

	sys_copy(SYSTASK,D,(vir_bytes) &(proc[m_ptr->PROC].p_pcpsw),m_ptr->PROC,D,(vir_bytes) (proc[m_ptr->PROC].p_sp),6);

	/* Jelikoz se uziv. procesu na volani exec() neodpovida (je vymenen kod),
	   zaridime opetovne naplanovani tohoto procesu rucne */

	proc[m_ptr->PROC].p_flags &= ~RECEIVING;
	if (proc[m_ptr->PROC].p_flags == 0) ready (&(proc[m_ptr->PROC]));

	m_ptr->RET_ERROR_CODE = OK;
	m_ptr->m_type = TASK_REPLY;
	return;
}

/*===========================================================================*
 *                              do_xit                                       *
 *	Uvolneni slotu konciciho procesu, sumace spotrebovaneho casu	     *
 *	a zapis sum k procesu rodice 					     *
 * args (m1):								     *
 *	m_ptr.PROC  		- cislo konciciho procesu         	     *
 *	m_ptr.PARENT		- cislo rodice konciciho procesu             *
 * ret:				                            		     *
 *	OK			- vse v poradku				     *
 *	ESRCH			- proces je neplatny			     *
 *===========================================================================*/

void do_xit(message * m_ptr) {

	struct proc* destroyPtr=&(proc[m_ptr->PROC]);
	int i;

	if (!isuserp(proc_addr(m_ptr->PARENT)) ||
	    !isuserp(proc_addr(m_ptr->PROC)) ||
	    (proc[m_ptr->PARENT].p_flags & P_SLOT_FREE) ||
	    (proc[m_ptr->PROC].p_flags & P_SLOT_FREE))
	{
		m_ptr->RET_ERROR_CODE = ESRCH;
		m_ptr->m_type = TASK_REPLY;
		return;
	}


	/* K child_utime a child_stime parentu pricteme casy ruseneho procesu a jeho potomku*/

	proc[m_ptr->PARENT].child_utime += proc[m_ptr->PROC].user_time + proc[m_ptr->PROC].child_utime;
	proc[m_ptr->PARENT].child_stime += proc[m_ptr->PROC].sys_time + proc[m_ptr->PROC].child_stime;

	/* Odplanujeme proces */
	unready (&(proc[m_ptr->PROC]));

	/* Odstranime alarm ruseneho procesu */
	proc[m_ptr->CHILD].p_alarm = 0;

	/* Je-li proc[].flags==SENDING (napr. proces byl nasilne zrusen
	   signalem behem vykonavani systemoveho volani) odstran
	   z jej front procesu, kterym se pokousel vyslat */

	if (proc[m_ptr->CHILD].p_flags & SENDING) {
		for (i=NR_TASKS+NR_SERVS; i<NR_TASKS+NR_SERVS+NR_PROCS; i++) {
		   if ((proc[i].p_flags & P_SLOT_FREE)==0) {
			if (proc[i].p_callerq==destroyPtr) {
			  proc[i].p_callerq=destroyPtr->p_sendlink;
			  break;
			 }
			  else
			   if (proc[i].p_sendlink==destroyPtr) {
				proc[i].p_sendlink = destroyPtr->p_sendlink;
				break;
			   }
		   }
		}
	}

	/* Uvolnime slot proc[] */
	proc[m_ptr->CHILD].p_flags = P_SLOT_FREE;

	m_ptr->m_type = TASK_REPLY;
	m_ptr->RET_ERROR_CODE = OK;
	return;
}

/*===========================================================================*
 *                              do_getsp                                     *
 *		Zjisteni hodnoty stack pointru zadaneho procesu		     *
 * args (m1):								     *
 *	m_ptr.PROC 		- cislo procesu                   	     *
 * ret:									     *
 *	OK			- vse v poradku				     *
 *	ESRCH			- proces je neplatny			     *
 *	hodnota SP registru	- ulozena v m.RET_SP			     *
 *===========================================================================*/

void do_getsp(message * m_ptr) {

	/* zjistim jestli je proces platny */

	if (!isuserp(proc_addr(m_ptr->PROC)) ||
	   (proc[m_ptr->PROC].p_flags & P_SLOT_FREE)) {

		m_ptr->m_type = TASK_REPLY;
		m_ptr->RET_ERROR_CODE = ESRCH;
		return;
	}

	/* Ulozime do zpravy SP registr - navratova hodnota */

	m_ptr->m_type = TASK_REPLY;
	m_ptr->RET_SP = proc[m_ptr->PROC].p_sp;
	m_ptr->RET_ERROR_CODE = OK;
	return;
}

/*===========================================================================*
 *                              do_times                                     *
 * args (m1):								     *
 *	m_ptr.PROC 	     - cislo procesu            		     *
 *	m_ptr.MEMPTR	     - adresa bufferu (v D segmentu odesilatele)     *
 *			       (bude vyplnen strukturou time_struc)	     *
 * 				  					     *
 * ret:				                       		    	     *
 *	OK		     - vse v poradku				     *
 *	ESRCH		     - neplatny (nebo ne uzivatelsky) proces	     *
 *      ENOMEM 		     - buffer presahuje segment		     	     *
 *===========================================================================*/

void do_times(message * m_ptr) {

       static time_struct ts;
       int result;

	/* Over, ze jde o user proces */

	if (!isuserp(proc_addr(m_ptr->PROC)) ||
	    (proc[m_ptr->PROC].p_flags & P_SLOT_FREE)) {
		m_ptr->m_type = TASK_REPLY;
		m_ptr->RET_ERROR_CODE = ESRCH;
		return;
	}

	/* Casy procesu naskladame do message */

	ts.user_time = proc[m_ptr->PROC].user_time;
	ts.sys_time = proc[m_ptr->PROC].sys_time;
	ts.child_user_time  = proc[m_ptr->PROC].child_utime;
	ts.child_sys_time = proc[m_ptr->PROC].child_stime;

	/* ... a zkopirujeme k uzivateli */
	result=sys_copy(SYSTASK,D,(vir_bytes) &ts,
			m_ptr->PROC,D,(vir_bytes) m_ptr->MEM_PTR,
			sizeof(time_struct));

	m_ptr->m_type = TASK_REPLY;
	m_ptr->RET_ERROR_CODE = result;
	return;
}


/*===========================================================================*
 *                               do_sig                                      *
 *     Signal procesu (nasimulovani preruseni - odskok na obsl. rutinu)      *
 * args (m6):								     *
 *	PROC          		- cislo procesu, kteremu signalizovat 	     *
 *	SIG_NR			- cislo signalu				     *
 *	HANDLER			- adresa obsluzne rutiny 		     *
 * ret:			                                		     *
 *	OK			- vse v poradku				     *
 *	ESRCH			- neplatny (nebo ne uzivatelsky) proces	     *
 *	EINVAL			- neplatne cislo signalu		     *
 *	ENOMEM			- na stack procesu nelze umistit flags,cs,   *
 * 				  pc,sig_nr	     			     *
 *===========================================================================*/

void do_sig(message * m_ptr) {

	int ret;
	static struct pc_psw stack;

	/* Over, ze jde o user proces */

	if (!isuserp(proc_addr(m_ptr->PROC)) ||
	    (proc[m_ptr->PROC].p_flags & P_SLOT_FREE)) {
		m_ptr->m_type = TASK_REPLY;
		m_ptr->RET_ERROR_CODE = ESRCH;
		return;
	}

	/* Navratova adresa uz na zasobniku je - od scheduleru. Vlozime
	   na zasobnik parametr-signal#, flags a adresu obsluzne rutiny,
	   kdyz scheduler provede IRET, zavola se tato rutina.
	   parametr DUMMY je prazdmy a jen kvuli tomu, aby se prekladac,
	   ktery pred parametry ocekava (near) navratovou adresu,
	   spravne pristupoval k parametrum - pres BP.
		    flags
		    cs
		    ip
		    ------
		    signal #
		    DUMMY
		    flags
		    cs - handler
		    ip - handler
	 */

	proc[m_ptr->PROC].p_pcpsw.pc=(reg_t) m_ptr->HANDLER;

	stack.cs=proc[m_ptr->PROC].p_pcpsw.cs;
	stack.pc=proc[m_ptr->PROC].p_pcpsw.pc;
	stack.psw=proc[m_ptr->PROC].p_pcpsw.psw;

	/* test podteceni zasobniku */
	if ((proc[m_ptr->PROC].p_sp - 10)<proc[m_ptr->PROC].p_map[S].mem_vir) {

	  trap(TRAP_OUT_STACK);

	  m_ptr->RET_ERROR_CODE = ENOMEM;
	  m_ptr->m_type = TASK_REPLY;
	  return;
	}

	proc[m_ptr->PROC].p_sp -= 10;
	/* posuneme vrchol stacku o 5 slov */
	sys_copy(SYSTASK,D,(vir_bytes) &(m_ptr->SIG_NR),m_ptr->PROC,D,proc[m_ptr->PROC].p_sp+8,2);
	sys_copy(SYSTASK,D,(vir_bytes) &(m_ptr->SIG_NR),m_ptr->PROC,D,proc[m_ptr->PROC].p_sp+6,2); 	 /* dummy*/
	sys_copy(SYSTASK,D,(vir_bytes) &stack,m_ptr->PROC,D,(vir_bytes) (proc[m_ptr->PROC].p_sp),6);

	m_ptr->RET_ERROR_CODE = OK;
	m_ptr->m_type = TASK_REPLY;
	return;
}

void do_copy(message * m_ptr) {
/*===========================================================================*
 *                              do_copy                                      *
 * 	Kopirovani bloku pameti mezi adresovymi prostory ruznych procesu     *
 * args (m5):								     *
 *	m_ptr.SRC_PROC, DST_PROC  	- zdrojovy/cilovy proces	     *
 *      m_ptr.SRC_SEG, DST_SEG		- zdrojovy/cilovy segment	     *
 *	m_ptr.SRC_VIR, DST_VIR		- zdrojova/cilova virt. adresa	     *
 *	m_ptr.BYTE_CNT - pocet prenasenych bajtu			     *
 * ret:				                          		     *
 *	OK			- vse v poradku				     *
 *	ESRCH			- src/dest proces neexistuje		     *
 *	EFAULT			- chyba adresace (mimo rozsah pam.prostoru)  *
 *      ENOMEM - blok presahuje (src/dest) segment			     *
 *===========================================================================*/

 m_ptr->RET_ERROR_CODE = sys_copy(m_ptr->SRC_PROC,m_ptr->SRC_SEG,m_ptr->SRC_VIR,
				  m_ptr->DST_PROC,m_ptr->DST_SEG,m_ptr->DST_VIR,
				  m_ptr->BYTE_CNT);
 m_ptr->m_type = TASK_REPLY;
}


/*===========================================================================
				cause_sig
		 Zaslani signalu procesu nekterym taskem
 ============================================================================*/

void cause_sig(int proc_nr, int sig_nr) {
	int i;

	if (!isuserp(proc_addr(proc_nr)) ||
	    (proc[proc_nr].p_flags & P_SLOT_FREE)) {
	  trap(TRAP_BAD_PROCESS);
	  return;
	}

	if (sig_nr<1 || sig_nr > NR_SIGS) {
	  trap(TRAP_ERRN_SIG);
	  return;
	}

	i = 1 << (sig_nr - 1);


	if (proc[proc_nr].p_pending==0) sig_procs++;
	proc[proc_nr].p_pending |= i;

	/* Pokus se o signalu informovat MM */
	inform ();
 }

/*===========================================================================
				inform
 Pokus o zaslani zpravy KSIG memory manageru pro kazdy proces,
 kteremu ma byt dorucen signal (proc[].p_pending!=0).
 Vola se z cause_sig() a pak z jadra, vzdy, kdyz  MM zavola receive(ANY)
 a zadna jina zprava pro nej neni (tj. MM nema nic na praci).
 ===========================================================================*/

void inform(void) {
	int i;

	if ((proc[MM].p_flags & RECEIVING) && (proc[MM].p_getfrom == ANY)) {
	      /* MM se nudi */

	      for (i=NR_TASKS+NR_SERVS; i<NR_TASKS+NR_SERVS+NR_PROCS; i++) {
			if ( ((proc[i].p_flags & P_SLOT_FREE)==0) && proc[i].p_pending != 0) {
				KSIG (i, proc[i].p_pending);
				/* KSIG vyridi vsechny signaly, ktere se mely procesu dorucit */
				sig_procs --;
				proc[i].p_pending = 0;
			}
		}
	}
	return;
}

/*===========================================================================
				umap
	mapovani adresy z adr. prostoru procesu na linearni
 ============================================================================*/

long umap(int proc_nr, int seg, vir_bytes vir_addr, vir_bytes count) {

	if (!isokprocn(proc_nr)) return ESRCH;
	if (proc[proc_nr].p_flags & P_SLOT_FREE) return ESRCH;

	if (proc_nr<NR_TASKS) count=0;  // jedna se o task, netestuj count
	 // Tasky slinkovany dohromady s jadrem, p_map je inicializovan, ale
	 // bez p_map[].mem_len (je tam VERY_BIG), takze count netestujeme
	#if (TEST_PHASE!=0)
	 // pro ucely debugovani nedavame mem_len ani serverum
	 if (proc[proc_nr].p_map[seg].mem_len==(Word) VERY_BIG) count=0;
	#endif

	  if (count > 0)
	   switch (seg) {
	    case T: if (((DWord) vir_addr) + count > (DWord) (proc[proc_nr].p_map[seg].mem_len*CLICK_SIZE))
		     return ENOMEM;
		    break;
	    case D: if (((DWord) vir_addr) + count > (DWord) (proc[proc_nr].p_map[S].mem_vir+proc[proc_nr].p_map[S].mem_len*CLICK_SIZE))
		     return ENOMEM;
		    break;
	    default: trap(TRAP_UMAP_PARAM); break;
	  }
	  return ( ((DWord) proc[proc_nr].p_map[seg].mem_phys) << CLICK_SHIFT) + vir_addr;
}


/*===========================================================================*
 *                            sys_copy                                       *
 *	 Kopirovani meze virtualnimi adresovymi prostory.		     *
 *===========================================================================*/


int sys_copy(procnr_t src_proc, Byte src_seg /* T/D */ , vir_bytes src_addr,
	      procnr_t dst_proc, Byte dst_seg /* T/D */, vir_bytes dst_addr,
	      Word count)
{
	long src,dst;

	src=umap(src_proc,src_seg,src_addr,count);
	if (src<0) return (int) src; /* chybovy kod z umap */
	dst=umap(dst_proc,dst_seg,dst_addr,count);
	if (dst<0) return (int) dst; /* chybovy kod z umap */

	k_phys_copy(src,dst,count);
	return OK;
}

PRIVATE void KSIG (procnr_t proc_nr, sigset_t sigs) {
 /* Posle MM zpravu, aby zaridil doruceni (vsech) signalu sigs procesu
    proc_nr */

  m.m_type=_C_KSIG;
  m.MM_PID=proc_nr;
  m.MM_SIGBMP=sigs;
  send(MM,m);
}
