// TEST_PHASE 128
/* Testovaci shell (na pozici INIT) */
/* Predpoklady behu:
	pritomnost FS, MM
	SPEC_FILE miri na ovladac TTY
*/
// Written by P.G.

/*
 resty: :-(
-  S_IREAD, S_IWRITE v open() - melo by byt O_RDONLY, O_WRONLY
   (spravit v knihovnach a FS)
- lseek() - offset (v knihovnach) nestaci int - jak je to ve FS ?
   (stejne tak vystup z lseek() )

*/

#define SPEC_FILE "/console"
#define MAXINPLINE 80
#define MAXOUTLINE 80

#include "kernel\makra.h"
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "COMMON\com.h"
#include "COMMON\callnr.h"
#include "COMMON\stat.h"
#include "COMMON\errno.h"

#include "test\kprintf.h"
#include "test\exitor.h"

#include "FS\COMMON\fsconst.h"
#include "FS\COMMON\fsmes.h"

#include "MM\mm.h"
#include "LIB\include\string.h"
#include "LIB\include\stdio.h"
#include "LIB\include\dir.h"

#define PROC1 (NR_TASKS+NR_SERVS)


static message m;
int ttyfd;
char inputLine[MAXINPLINE];
char outputLine[MAXOUTLINE];

#define VARNAME_LEN 16
#define MAXNUMVARS  32
char environment[MAXNUMVARS][VARNAME_LEN];


int writeLine(char* txt) {
 int len;

 len=strlen(txt);
 m.m_type = C_WRITE;
 m.fs_fd = ttyfd;
 m.fs_buffer=txt;
 m.fs_nbytes=len;
 m.fs_seg=D;
 send_recv(FS,m);
 return m.fs_errcode;
}


int readLine(char* buffer, int maxLen) {
  m.m_type = C_READ;
  m.fs_fd = ttyfd;
  m.fs_buffer=buffer;
  m.fs_nbytes=maxLen;
  m.fs_seg=D;
  send_recv(FS,m);
  return m.fs_errcode;
}

char* cd(char* dir) {
 strcpy(outputLine,"cd: ");
 strcat(outputLine,dir);
 strcat(outputLine,"\n");
 return outputLine;
}

char* su(char* userId) {
 strcpy(outputLine,"su: ");
 strcat(outputLine,userId);
// sprintf(outputLine,dir);
 strcat(outputLine,"\n");
 return outputLine;
}

char* setvar(char* var) {
 strcpy(outputLine,"setvar: ");
 strcat(outputLine,var);
 strcat(outputLine,"\n");
 return outputLine;
}


char* printvars(void) {
 strcpy(outputLine,"printvars :\n");
 return outputLine;
}

char* exec(char* pgm) {
 strcpy(outputLine,"exec: ");
 strcat(outputLine,pgm);
 strcat(outputLine,"\n");
 return outputLine;
}

char* whoami(void) {

 uid_t  uid, ruid;
 gid_t gid, rgid;
 pid_t pid,ppid;

 m.m_type = C_GETUID;
 m.MM_GETIDPID=0; /* uid tohoto procesu */
 send_recv(MM,m);
 if (m.MM_ERRORNO!=0) {
  printf("getuid() failed: %i",m.MM_ERRORNO);
  return outputLine;
 }

 uid=m.MM_GEUID; ruid=m.MM_GRUID;

 m.m_type = C_GETGID;
 m.MM_GETIDPID=0; /* uid tohoto procesu */
 send_recv(MM,m);
 if (m.MM_ERRORNO!=0) {
  printf("getgid() failed: %i",m.MM_ERRORNO);
  return outputLine;
 }

 gid=m.MM_GEGID; rgid=m.MM_GRGID;

 m.m_type = C_GETPID;
 send_recv(MM,m);
 if (m.MM_ERRORNO!=0) {
  printf(outputLine,"getpid() failed: %i",m.MM_ERRORNO);
  return outputLine;
 }

 pid=m.MM_GPID; ppid=m.MM_GPPID;

 printf(outputLine,"euid: %u, ruid: %u, egid: %u, rgid: %u, pid: %u, ppid: %u\n",
 uid,ruid,gid,rgid,pid,ppid);

 return outputLine;
}

char* interpretCommand(char* input) {

 if (strcmp(input,"quit")==0) return NULL;
 if (strstr(input,"cd ")==input) return cd(input+3);
 if (strstr(input,"su ")==input) return su(input+3);
 if (strstr(input,"set ")==input) return setvar(input+4);
 if (strcmp(input,"set")==0) return printvars();
 if (strcmp(input,"whoami")==0) return whoami();
 if (strstr(input,"exec ")==input) return exec(input+5);
 printf("Unknown command\n");
 return "Unknown command\n";
}

void proc1(void) {

 int len,err,_stdout,_stdin;
 char* response;

 /* vytvor specialni soubor ovladace konzoly */

 if ( (err=mknod (SPEC_FILE,I_CHAR_SPECIAL | 0777,256*TTY+0))!=0 && err!=EEXIST ) {
   kprintf(1,0,7,"mknod() error: code %i",err);
   returnToDOS();
 }

 /* otevrit stdin a stdout (normalne bychom podedili z INITu) */
 _stdout=open(SPEC_FILE,S_IREAD); /* fd 0 */
 if (_stdout<0) {
   kprintf(1,0,7,"Error preparing stdout: %i",_stdout);
   returnToDOS();
 }

 _stdin=open(SPEC_FILE,S_IWRITE);  /* fd 1 */
 if (_stdin<0) {
   kprintf(1,0,7,"Error preparing stdin: %i",_stdin);
   close(_stdout);
   returnToDOS();
 }

 do {
  printf("TUOX>\n");
  if ((len=read(_stdin,inputLine,MAXINPLINE))<=0)
   printf("TTY reading error\n");
  else {
   inputLine[len-1]=0;
   response=interpretCommand(inputLine);
   printf("Nic nebude\n");
//   if (response) writeLine(response);
  }
 } while(response);

 /* to by tu mozna nemuselo byt - v normalnich programech take neni
    (ale pred sync() a returnToDOS() je to treba) */
 close(_stdin);
 close(_stdout);

 m.m_type = C_SYNC;
 send_recv(FS,m);

 returnToDOS();
}

#include "lib/stdio/printf.c"
#include "lib/stdio/vsprintf.c"
#include "lib/dir/dir.c"