// TEST_PHASE 6
/* Uplny test blokovych zarizeni. Nejprve zjisteni informaci o podporovanych
   minor zarizenich a jejich velikostech, pak postupny zapis a verifikace
   unikatnich dat do jednotlivych bloku. */

// Written by P.G.

// MUSI BYT NASMEROVANO INT 08 a INT0E

#define BDEV_TO_TEST FLOPPY
 // which block device to test

#include "kernel\makra.h"
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "COMMON\com.h"
#include "fs\ioctl.h"
#include "test\kprintf.h"
#include "clock\h_clock.h"

#define SERV1 NR_TASKS

static char buf[BLOCK_SIZE];

static message m;

void fillBuffer(long seed) {
 int i;
 long* b=(long*) buf;
 for (i=0; i<sizeof(buf)/sizeof(seed); i++)
  *b++=seed++;
}

int verifyBuffer(long seed) {

 int i;
 long* b=(long*) buf;
 for (i=0; i<sizeof(buf)/sizeof(seed); i++)
  if (*b++!=seed++) return 0;
 return 1;
}

void destroyBuffer(void) {
 int i;
 for (i=0; i<BLOCK_SIZE; i++) buf[i]=0;
}

long getSeed(void) {
 static message mc;
 mc.m_type=GET_TIME;
 send_recv(CLOCK,mc);
 return mc.RET_REAL_TIME_M;
}


// na pozici 1.serveru
void serv1(void) {
 static int devs;
 static long blocks[2];
 int i;
 long seed,b,wr_errs,rd_errs,ver_errs;

 // Determine the # of minor devs

 m.m_type=DEV_IOCTL;
 m.m_u.m_dio.device=0; /* aby to driver neodmitnul */
 m.m_u.m_dio.count=GETNUMBERMINORDEV;
 m.m_u.m_dio.proc_nr=SERV1;
 m.m_u.m_dio.address=(vir_bytes) (&devs);

 send_recv(BDEV_TO_TEST,m);
 if (m.REP_STATUS!=OK)
  kprintf(1,0,7,"Error reply to GETNUMBERMINORDEV (%i), exiting",m.REP_STATUS);
 else {
  kprintf(0,0,7,"Number of minor devices: %i",devs);

  // Determine # of blocks for each minor device

  for (i=0; i< ((devs<2) ? devs : 2); i++) {

   m.m_type=DEV_IOCTL;
   m.m_u.m_dio.device=i;
   m.m_u.m_dio.count=GETDEVSIZE;
   m.m_u.m_dio.proc_nr=SERV1;
   m.m_u.m_dio.address=(vir_bytes) (blocks+i);

   send_recv(BDEV_TO_TEST,m);
   if (m.REP_STATUS==OK)
    kprintf(5+i,0,7,"Minor device %u has %l blocks",i,blocks[i]);
   else {
    kprintf(5+i,0,7,"Minor device %u: Error reply to GETDEVSIZE (%i)",i,m.REP_STATUS);
    blocks[i]=0;
   }
  }

   // write and verify data pattern to all blocks of all devs

  for (i=0; i< ((devs<2) ? devs : 2); i++) {

   wr_errs=rd_errs=ver_errs=0;

   for (b=0; b<blocks[i]; b++) {

    seed=getSeed();
    fillBuffer(seed);

    m.m_type=DEV_WRITE;
    m.m_u.m_dio.device=i;
    m.m_u.m_dio.count=1;
    m.m_u.m_dio.position=b;
    m.m_u.m_dio.proc_nr=SERV1;
    m.m_u.m_dio.address=(vir_bytes) buf;
    send_recv(BDEV_TO_TEST,m);
    if (m.REP_STATUS!=1) wr_errs++;

    destroyBuffer();

    m.m_type=DEV_READ;
    m.m_u.m_dio.device=i;
    m.m_u.m_dio.count=1;
    m.m_u.m_dio.position=b;
    m.m_u.m_dio.proc_nr=SERV1;
    m.m_u.m_dio.address=(vir_bytes) buf;
    send_recv(BDEV_TO_TEST,m);
    if (m.REP_STATUS!=1) rd_errs++;

    if (!verifyBuffer(seed)) ver_errs++;

    kprintf(8,(i*40),7,"Minor device %i, block %l ",i,b);

    kprintf(10,(i*40),7,"Write errors  : %l    ",wr_errs);
    kprintf(11,(i*40),7,"Read errors   : %l    ",rd_errs);
    kprintf(12,(i*40),7,"Verify errors : %l    ",ver_errs);
   }
  }
 }
 while(1);
}


