// TEST_PHASE 7
/* Test TTY */
// Written by P.G.

// MUSI BYT NASMEROVANO INT 09

#include "kernel\makra.h"
#include "COMMON\type.h"
#include "COMMON\const.h"
#include "COMMON\com.h"
#include "COMMON\errno.h"
#include "COMMON\callnr.h"
#include "COMMON\sgtty.h"
#include "fs\ioctl.h"
#include "test\kprintf.h"
#include "tty\tty_com.h"
#include "test\exitor.h"

#include "lib\include\string.h"
#include "lib\include\stdlib.h"

#define SERV1 NR_TASKS
#define T_DEVICE CONSOLE
#define BUF_SIZE 128


static char buf[BUF_SIZE];

static message m;

int t_read(char* buf, int cnt) {

   m.m_type=DEV_READ;
   m.m_u.m_dio.device=T_DEVICE;
   m.m_u.m_dio.count=cnt;
   m.m_u.m_dio.address=(vir_bytes) buf;
   m.m_u.m_dio.proc_nr=SERV1;
   send_recv(TTY,m);

   if (m.REP_STATUS>0) return m.REP_STATUS;

   if (m.REP_STATUS==EAGAIN)
    { kprintf(2,0,7,"Device is busy"); return EAGAIN; }

   // suspended, wait for REVIVE

   kprintf(2,0,7,"R: SUSPENDED, waiting for revive msg from TTY");
   receive(TTY,m);

   if (m.m_type!=_C_REVIVE)
    { kprintf(2,0,7,"R: Unexpected message from TTY"); return -1; }

   kprintf(2,0,7,"                                             ");
   return m.m_u.m_dio.count;
}

int t_write(char* buf, int cnt) {

   m.m_type=DEV_WRITE;
   m.m_u.m_dio.device=T_DEVICE;
   m.m_u.m_dio.count=cnt;
   m.m_u.m_dio.address=(vir_bytes) buf;
   m.m_u.m_dio.proc_nr=SERV1;
   send_recv(TTY,m);
   if (m.REP_STATUS>0) return m.REP_STATUS;

   if (m.REP_STATUS==EAGAIN)
    { kprintf(2,0,7,"Device is busy"); return EAGAIN; }

   // suspended, wait for REVIVE

   kprintf(3,0,7,"W: SUSPENDED, waiting for revive msg from TTY\n");
   receive(TTY,m);

   if (m.m_type!=_C_REVIVE)
    { kprintf(3,0,7,"W: Unexpected message from TTY"); return -1; }

   kprintf(3,0,7,"                                                ");
   return m.REP_STATUS;
}

int t_ioctl(char* buf, int command) {
/*
type of buf:
 cmd=TIOCSETP 1 // nastaveni paranetru tty
 cmd=TIOCSETC 2 // nastaveni ridicich znaku tty
    struct tty_params:


 cmd=TIOCGETP 3 // cteni paranetru tty
 cmd=TIOCGETC 4 // cteni ridicich znaku tty
    struct tty_ctrlchars;

 cmd=SETRS232 5 //nastaveni parametru prenosu
    struct rs232_params;

Return value:  0 = OK
	      -1 = ioctl() error
	      -2 = setting verify error when getting value
	      -3 = verify error (value does not match the set value)
	      -4 = bad ioctl command
*/

  int invcmd,paramsize;
  static char replybuf[100]; // 100 musi stacit :-)

  if (command!=TIOCSETP && command!=TIOCGETP &&
      command!=TIOCSETC && command!=TIOCGETC && command!=SETRS232)
    return -4;

   m.m_type=DEV_IOCTL;
   m.m_u.m_dio.device=T_DEVICE;
   m.m_u.m_dio.count=command;
   m.m_u.m_dio.address=(vir_bytes) buf;
   m.m_u.m_dio.proc_nr=SERV1;
   send_recv(TTY,m);
   if (m.REP_STATUS>0) return -1;

   // pokud slo o set, zkontroluj, jestli get vraci nastavenou hodnotu
   switch (command) {
    case TIOCSETP: invcmd=TIOCGETP; paramsize=sizeof(struct tty_params); break;
    case TIOCSETC: invcmd=TIOCGETC; paramsize=sizeof(struct tty_ctrlchars); break;
    default: return 0;
   }

   m.m_type=DEV_IOCTL;
   m.m_u.m_dio.device=T_DEVICE;
   m.m_u.m_dio.count=invcmd;
   m.m_u.m_dio.address=(vir_bytes) replybuf;
   m.m_u.m_dio.proc_nr=SERV1;
   send_recv(TTY,m);
   if (m.REP_STATUS>0) return -2;

   if (memcmp(buf,replybuf,paramsize)!=0) return -3;

   return 0;
}


int strLen(char* c) {
 int l=0;
 while (*c) { c++; l++; }
 return l;
}

int moveTo(unsigned char col, unsigned char row) {
 static char esc[3];
 esc[0]=27;
 esc[1]=col+32;
 esc[2]=row+32;
 if (t_write(esc,3)==0) return 0;
  else return -1;
}

int setAttr(unsigned char attr) {
 static char esc[3];
 esc[0]=27;
 esc[1]='z';
 esc[2]=attr;
 if (t_write(esc,3)==0) return 0;
  else return -1;
}

int setDiscipline(int discpl) {
/*
#define CBREAK       0000002
#define COOKED       0000000
#define RAW          0000040
*/
 static struct tty_params params;
 int err;

 if (discpl!=CBREAK && discpl!=COOKED && discpl!=RAW) return -100;

 if ((err=t_ioctl((char*) &params,TIOCGETP))!=0) return err;
 params.tty_discipline=discpl;
 if ((err=t_ioctl((char*) &params,TIOCSETP))!=0) return err;
 return 0;
}

int setMode(int mode) {
/*
#define XTABS        0006000
#define CRMOD        0000020
#define ECHO         0000010
*/

 static struct tty_params params;
 int err;

 if ((err=t_ioctl((char*) &params,TIOCGETP))!=0) return err;
 params.tty_mode=mode;
 if ((err=t_ioctl((char*) &params,TIOCSETP))!=0) return err;
 return 0;
}


void printHelp(void) {
 char* helpMsg="Supported Commands:\n\nquit - exit tester\naN - set attribute to N\n"
 "Edit commands: BACKSPACE - erase one char, @ - erase line, ^Q/^S - XON/XOFF, ^D - EOF\n";
  t_write(helpMsg,strLen(helpMsg));
}

int interpretCmd(char *cmd) {
// vraci 0 kdyz nezinterpretoval


 int attr;

 if (!strcmp(cmd,"help\n")) { printHelp(); return 1; }

 if (*cmd=='a') { // set attr
  attr=atoi(cmd+1);
  if (attr==0) return 0;
  setAttr((unsigned char) attr);
  return 1;
 }
 return 0;
}

// na pozici 1.serveru
/*
 ESC=27
 ESC 32+x,32+y  - move cursor to (col,row)
 ESC '~' '0' - cls from cursor		???
 ESC '~' '1'  - reverse scroll 1 line	???
 ESC 'z' x - set attribute to x
 */
void serv1(void) {

 char* welcomeMsg1="TUOX TTY Testing Server Ready.\n";
 char* welcomeMsg2="Type 'quit' to finish, 'help' to get help.\n\n";

 char* exitMsg="Exiting TTY tester\n";
 char* promptMsg="TUOX> ";
 char* replyMsg="Unrecoginized command ";
 char* nothingMsg="len<0";

 int len,err;

 if ((err=t_write(welcomeMsg1,strLen(welcomeMsg1)))<0)
  kprintf(15,0,7,"TTY WRITE ERR/1: %i",err);
 if ((err=t_write(welcomeMsg2,strLen(welcomeMsg2)))<0)
  kprintf(15,0,7,"TTY WRITE ERR/2: %i",err);

// moveTo(0,10);

 do {
  t_write(promptMsg,strLen(promptMsg));
  len=t_read(buf,BUF_SIZE-1);

  if (len>0) {
    *(buf+len)=0;
    if (!interpretCmd(buf)) {
     t_write(replyMsg,strLen(replyMsg));
     t_write(buf,len);
    }
  }
   else { t_write(nothingMsg,strLen(nothingMsg)); *buf=0; }
 } while (strcmp(buf,"quit\n"));

 t_write(exitMsg,strLen(exitMsg));
 returnToDOS();
}


