/*===========================================================================*
 *                               Terminal Task                               *
 *===========================================================================*
 * Written by : Martin Bajgar                                                *
 * Last update: 04-06-1997                                                   *
 *===========================================================================*/

/*===========================================================================*
 *                              do_read                                      *
 *===========================================================================*/
// incrementace ukazatel na patu vstupni fronty

 PRIVATE void inc_intail(struct tty_struct *tp)
{
 if ((tp->tty_intail - tp->tty_inqueue) == TTY_IN_BYTES)
  tp->tty_intail = tp->tty_inqueue;
 else
  tp->tty_intail++;
}
/*
Zkus precist pozadovany pocet znaku. Proces bud dostane znak/radek
(v COOKED mode vzdy nejvyse 1 radek) nebo nic:
 Jestlize je tty_incount=0 nebo je COOKED mode a neprislo zadne LF:
  volajicimu procesu (FS) se posle odpoved SUSPEND
   (FS pak suspenduje task, ktery chtel vstup)
   send(FS, message)

Znaky se nakopiruji primo do adresoveho prostoru uziv. procesu
(misto bylo pri volani poznamenano do tty_struct[])
Prenese se bud tolik znaku, kolik chtel, nebo (je-li mensi) jen
tolik, kolik je k dispozici (v cooked mode nejvyse jeden radek
nebo po MARKER (vklada jej do bufferu CTRL-D, uziv. procesu se
nepredava))
Znaky se berou z ocasu tty_inqueue[], narazi-li se na LF,
dekrementuje se tty_lfct;
Odpovez volajicimu (FS): TASK_REPLY(pocet_prenesenych_byte)
*/

// provadi prenos ze vstupni fronty na adresu ze zpravy
PRIVATE int transfer(struct tty_struct *tp)
{
 int nCount, i;
 char ch;
 int endLoop=0;

 if (tp->tty_inleft > tp->tty_incount)
  nCount = tp->tty_incount;
 else
  nCount = tp->tty_inleft;

 for (i = 0; i < nCount && !endLoop; i++)
 {
  ch = *tp->tty_intail;

  if ((ch == tp->tty_eof) && (tp->tty_discipline == COOKED))
   {
    tp->tty_lfcount--;
	break;
   }

  if (ch == '\n')
   {
    if (tp->tty_discipline == COOKED)
     {
      nCount = i + 1;
      tp->tty_lfcount--;
      endLoop=1;
//P.G.: tady nemuze byt break, protoze se jeste musi provest kopirovani
// znaku a inc/dec citacu
// Misto toho ten flag endLoop
//      break;
     }
   }

  sys_copy(TTY, D, (vir_bytes) &ch, tp->tty_inproc, D, tp->tty_in_vir, 1);
      // ulozeni znaku na vystupni adresu

  inc_intail(tp);
  tp->tty_in_vir++;
  tp->tty_inleft--;
  tp->tty_incount--;

 }
 return nCount;
}

// zpracovani zpravy read
PRIVATE void do_read(struct tty_struct *tp, message *m_ptr)
{
  /* A process wants to read from a terminal. */

//Jestlize na vstup z terminalu uz nekdo ceka (tty_inleft>0),
//odpovez vrat E_TRY_AGAON  OK
 if ((tp->tty_inleft > 0))
  {
   task_reply(EAGAIN,EAGAIN, tp->tty_incaller, tp->tty_inproc);
   return;
  }

//Informace o volajicim a parametry nakopiruj do tty_struct[]
// (incaller,inproc,in_vir,inleft)
 tp->tty_incaller = m_ptr->m_source;
 tp->tty_inproc = m_ptr->m_proc_nr;
 tp->tty_in_vir = m_ptr->m_address;
 tp->tty_inleft = m_ptr->m_count;

 if ((tp->tty_incount == 0) || ((tp->tty_discipline == COOKED) && (tp->tty_lfcount == 0)))
  {
     task_reply(SUSPEND,SUSPEND, tp->tty_incaller, tp->tty_inproc);
     return;
  }

 task_reply(OK,transfer(tp), tp->tty_incaller, tp->tty_inproc);
}


