#include "test.cnf"
#ifdef TEST_EXIT_ON_KEYEND
 #include "test\exitor.h"
#endif
#include "kernel\proc_pub.h"

/*===========================================================================*
 *                               Terminal Task                               *
 *===========================================================================*
 * Written by : Rongi                                                        *
 * Last update: 27-05-1997                                                   *
 *===========================================================================*/

/*===========================================================================*
 *                              keyboard                                     *
 *===========================================================================*/
/* zpracovani preruseni od klavesnice */
/*
 Prevezni scan code z portu klavesnice
 Pokud jde o pusteni klavesy a klavesa neni zadny shift:
  povol preruseni na 8259, return
 jinak:
  zjisti, jde-li o CTRL-S
  pokud ano: Neni-li terminal v RAW mode: tty_struct[CONSOLE].tty_inhibited=STOPPED;
	     povol preruseni na 8259, return
     (jeste bychom meli (nejak) osetrit, kdyz proces nastavi STOP OUTPUT
      na jiny znak nez CTRL-S)

  pokud ne:
   Zjisti, je-li stisknuto CTRL-ALT-DEL: pokud ano, reboot()
   uloz znak v pomocne fronte tty_driver_buf[] (je-li v ni misto)
    uklada se zde vzdy (scan) kod a cislo terminalu, ze ktereho prisel
    Vytvor TTY_CHAR_INT message pro TTY task,
    zkus poslat (interrupt()) - kdyz se nepovede, nastavi se v busy_map
    a bude se to zkouset pri dalsich prerusenich
*/

/*
stav: funkcni
dodelat: prepsat do assembleru
         mozna se pred volanim k_interruptu() povoli preruseni na ICU (8259)
	 odstranit zakomentovane debug casti

rozsireno: v zavislosti na Scroll Lock skrolujeme
	   rozsvecujeme LEDky klavesnice
*/

/* keyboard_flag (Ctrl-Alt-Del testing) */
#define __CTRL            0x01
#define __ALT             0x02
#define __DEL             0x04

/* locks_flag (for LED setting) */
#define __SCROLL_LOCK     0x01
#define __NUM_LOCK        0x02
#define __CAPS_LOCK       0x04

PRIVATE unsigned char keyboard_flag=0;
PRIVATE unsigned char locks_flag=0;

PRIVATE int prefixed_0xe0 = 0; /* predchozi scancode prefix 0xe0 */
PRIVATE int prefixed_0xe1 = 0; /* predchozi scancode prefix 0xe1 */
PRIVATE int prefixed_0xe0_shift = 0; /* neosetrena posloupnost 0xe0 0x2a */

#ifdef TEST_USE_TTY
PUBLIC void keyboard(void)
{
 unsigned char c;
 int i;
 message msg;
 /* precteni scancodu z portu do c */
 asm cli
 asm in   al, 0x60
 asm mov  [c], al
 asm mov  dx, 0x61
 asm in   al, dx
 asm mov  ah, al
 asm or   al, 0x80
 asm out  dx, al
 asm xchg al, ah
 asm out  dx, al
 // DEBUG!!!
 // cprintf("0x%02x -->\r\n", (unsigned int)c);

#ifdef TEST_EXIT_ON_KEYEND
 if (c==sc_End) {
  asm mov dx, 0x20
  asm mov al, 0x20
  asm out dx, al
  returnToDOS();
 }
#endif

 /* osetreni klavesy Pause - jedine klavesy s prefixem 0xe1*/
 /* jednoduse ignorujeme */
 if (prefixed_0xe1)
  {
   prefixed_0xe1--;
   goto not_store;
  }
 if (c & BREAK_CODE)    /* pusteni klavesy */
  {
   switch(c & ~BREAK_CODE)
    {
     /* pusteni Shiftu, Ctrlu, Altu posilame dal */
     /* ostatni se ignoruje                      */
     case sc_LShift: if (prefixed_0xe0_shift)
		       {
			prefixed_0xe0_shift = FALSE;
			goto not_store;
		       }
						break;
     case sc_RShift:                            break;
     case sc_Ctrl:   keyboard_flag &= ~__CTRL;  break;
     case sc_Alt:    keyboard_flag &= ~__ALT;   break;
     case sc_Del:    keyboard_flag &= ~__DEL;   /* not breaked! */
     default:        goto not_store;
    }
  }
 else /* MAKE_CODE */
  {
   switch(c)
    {
    case sc_LShift:     if (prefixed_0xe0)
			  {
			   prefixed_0xe0_shift = TRUE;
			   goto not_store;
			  }
								     break;
     case sc_Ctrl:       keyboard_flag |= __CTRL;                    break;
     case sc_Alt:        keyboard_flag |= __ALT;                     break;
     case sc_Del:        keyboard_flag |= __DEL;                     break;
     case sc_NumLock:    locks_flag    ^= __NUM_LOCK;    set_leds(); break;
     case sc_CapsLock:   locks_flag    ^= __CAPS_LOCK;   set_leds(); break;
//     case sc_ScrollLock: locks_flag    ^= __SCROLL_LOCK; set_leds();
//			 if (locks_flag & __SCROLL_LOCK)
//			   scroll_on();
//			 else
//			   scroll_off();
//								     break;
    }
   if (keyboard_flag == (__CTRL | __ALT | __DEL) ||
       // DEBUG for win
       (keyboard_flag & (__CTRL | __ALT)) == (__CTRL | __ALT) && c == sc_Ins
   )
    {
     asm mov al, 0x20
     asm out 0x20, al
     #ifdef DEBUG
     // Az reboot() bude tak dejte pryc #ifdef DEBUG
     reboot();
     #endif
     return;
    }

   if (keyboard_flag & __CTRL && c == sc_S &&
       tty_table[CONSOLE].tty_discipline != RAW)
    {
     tty_table[CONSOLE].tty_inhibited = TRUE;
     goto not_store;
    }
  }

store:
 if (locks_flag & __SCROLL_LOCK)
  {
   switch (c)
    {
//     case sc_UpArrow:   scroll_up();     break;  // toto jsem vyradil
//     case sc_DownArrow: scroll_down();   break;  // toto jsem vyradil
    }
  }
 else
  {
   i = tty_driver_buf.count;
   if (i < TTY_DRIVER_BUF_SIZE)
    {
     tty_driver_buf.terminal_nr[i] = CONSOLE;
     tty_driver_buf.code[i] = c;
     tty_driver_buf.count++;

     msg.m_source = HARDWARE;
     msg.m_type = TTY_CHAR_INT;
     k_interrupt(TTY, &msg);
    }
  }
not_store:
 prefixed_0xe0 = FALSE;
 if (c == 0xe0)
   prefixed_0xe0 = TRUE;
 else if (c == 0xe1)
   prefixed_0xe1 = 2;
 asm mov al, 0x20
 asm out 0x20, al

/* DEBUG!!! */
/*
 if (keyboard_flag & __CTRL)
   put_char(0, 24, 'C'+0x0100);
 else
   put_char(0, 24, 'c'+0x0100);
 if (keyboard_flag & __ALT)
   put_char(1, 24, 'A'+0x0100);
 else
   put_char(1, 24, 'a'+0x0100);
 if (keyboard_flag & __DEL)
   put_char(2, 24, 'D'+0x0100);
 else
   put_char(2, 24, 'd'+0x0100);
*/
}
#endif


/* cekani na klavesnici */
void wait_8042(void)
{
 asm xor  cx, cx
loop:
 asm in   al, 0x64
 asm and  al, 2
 asm loopnz loop
}


/* skvela makra na prikazy klavesnice */

#define KEYBOARD_COMMAND(command)               \
 wait_8042();                                   \
 asm mov  al, command                ;          \
 asm out  0x60, al                   ;

#define KEYBOARD_COMMAND1(command, value)       \
 wait_8042();                                   \
 asm mov  al, command                ;          \
 asm out  0x60, al                   ;          \
 wait_8042();                                   \
 asm mov  al, value                  ;          \
 asm out  0x60, al                   ;



/* podle locks_flag se rosvecuji LEDky */
PRIVATE void set_leds(void)
{
 KEYBOARD_COMMAND1(0xed, [locks_flag]);
}


/* nastaveni rychlosti klavesnice */
PRIVATE void set_speed(int delay, int rate)
{
 unsigned char value;
 delay &= 0x03; /* 2bits  0 for max */
 rate  &= 0x1f; /* 5 bits 0 for max*/
 value = (unsigned char) rate | ((unsigned char) delay << 5);
 KEYBOARD_COMMAND1(0xf3, value);
}


/* inicializace klavesnice */
PRIVATE void keyboard_init(void)
{
 asm cli
 KEYBOARD_COMMAND(0xf6);        /* reset keyboard */
 set_speed(0, 0);
 key_flag = 0;
 keyboard_flag = 0;
 locks_flag = 0;
 prefixed_0xe0 = FALSE;
 prefixed_0xe1 = FALSE;
 prefixed_0xe0_shift = 0;
 set_leds();
 tty_driver_buf.count = 0;
 asm sti
}
