/*===========================================================================*
 *                               Terminal Task                               *
 *===========================================================================*
 * Written by : EVERYBODY (but mainly Rongi)                                 *
 * Last update: 27-05-1997                                                   *
 *===========================================================================*/

/*
stav: funkcni
dodelat: urcit a zkontrolovat XXX_CHAR
*/

#ifndef TTY_H
#define TTY_H

#include "common\callnr.h"
#include "common\com.h"
#include "common\const.h"
#include "common\signal.h"
#include "common\type.h"
#include "common\errno.h"
#include "common\sgtty.h"

#ifndef DEBUG
#include "kernel\makra.h"
#endif

#include "tty\tty_mess.h"
#include "tty\scancode.h"
#include "tty\tty_com.h"
#include "tty\rs232\rs232.h"

#define tty_addr(line)  (&tty_table[line]) // Pekne blbe prejmenovani

#define NR_TTYS            3    /* how many terminals can system handle */
#define TTY_IN_BYTES     256    /* tty input queue size */
#define TTY_OUT_BYTES    256    /* tty output queue size */
#define TAB_SIZE           8    /* distance between tab stops */

#define ERASE_CHAR       '\b'
#define KILL_CHAR        '@'

#define QUIT_CHAR         0x08  /* urcit !!! del */
#define INTR_CHAR         0x03  /* (CTRL-C) */
#define XON_CHAR          0x11  /* (CTRL-Q) */
#define XOFF_CHAR         0x13  /* (CTRL-S) */
#define EOT_CHAR          0x04  /* (CTRL-D) */


struct tty_struct {
  /* Input queue.  Typed characters are stored here until read by a program. */
  char tty_inqueue[TTY_IN_BYTES]; /* array used to store input chars */
  char *tty_inhead,*tty_intail;  /* pointers to place next char goes and to next char to be given to prog */
  int tty_incount;              /* # chars in the input queue */
  int tty_lfcount;              /* number of lines in input queue */

  /* Output section. */
  int tty_outqueue[TTY_OUT_BYTES]; /* buffer for output */
  int *tty_outhead, *tty_outtail;
  int tty_outcount;

  int (*tty_devstart)(struct tty_struct *tp);  // routine to start actual device output
   /* V pripade obrazovky zajisti tato rutina nakopirovani
   vystupni fronty do videoRAM najednou. Pokud by se jednalo o RS-232
   terminal, nastavilo by se preruseni, ktere by se generovalo vzdy
   pri schopnosti terminalu prevzit znak a v jeho obsluze by se znak
   vytisknul. Preruseni by se zpracovavala dokud by fronta nebyla prazdna */

  /* Terminal parameters and status. */
  int tty_discipline; // set by IOCTL
  int tty_mode;       /* set by IOCTL */
  int tty_column;     /* cursor position 0-based, row 0 at the bottom */
  int tty_row;
  int tty_attribute;
  char tty_out_escaped; /* ESC for output section*/
  char tty_esc_char;  /* 1st char that goes out after ESC */
  char tty_busy;      /* 1 when output in progress */
  char tty_escaped;   /* ESC detected: 0/1 */
  char tty_inhibited; /* 1 when output stopped by CTRL-S */
  char tty_makebreak; /* 1 for terminals which interrupts twice per key */
  char tty_waiting;   /* 1 when output process waiting for reply */

  struct rs232_params rs232; /* RS232 settings */

  /* user settable characters */
  char tty_erase;
  char tty_kill;
  char tty_intr;
  char tty_quit;
  char tty_xon;
  char tty_xoff;
  char tty_eof;

  /* Information about incomplete I/O requests is stored here. */
  char tty_incaller;            /* process that made the call (usually FS) */
  char tty_inproc;              /* process that wants to read from tty */
  vir_clicks tty_in_vir;        /* virtual address where data is to go */
  int tty_inleft;               /* how many chars are still needed */

  char tty_outcaller;           /* process that made the call (usually FS) */
  char tty_outproc;             /* process that wants to write to tty */
  vir_clicks tty_out_vir;       /* physical address where data out comes from */
  int tty_outleft;              /* # chars yet to be output */
  int tty_outcum;               /* # chars output so far (copied to tty_outqueue) */
};


#define TTY_DRIVER_BUF_SIZE     256

struct tty_driver_buf_struct {
  unsigned char count;
  unsigned char terminal_nr[TTY_DRIVER_BUF_SIZE];
  unsigned char code[TTY_DRIVER_BUF_SIZE];
};


PRIVATE void console(struct tty_struct *tp);
PRIVATE void do_cancel(struct tty_struct *tp, message *m_ptr);
PRIVATE void do_charint(void);
PRIVATE void do_ioctl(struct tty_struct *tp, message *m_ptr);
PRIVATE void do_read(struct tty_struct *tp, message *m_ptr);
PRIVATE void do_write(struct tty_struct *tp, message *m_ptr);
PRIVATE void echo(struct tty_struct *tp, char c);
PRIVATE in_char(int line, unsigned char ch /* scan code */);
PRIVATE char make_break(unsigned char ch);
PRIVATE void out_char(struct tty_struct* tp, char c);
PRIVATE void tty_init(void);

PRIVATE int  transfer(struct tty_struct* tp);

PRIVATE void set_leds(void);
PRIVATE void set_speed(int delay, int rate);
PRIVATE void keyboard_init(void);

PRIVATE void screen_init(void);
PRIVATE void scroll_up(void);
//PRIVATE void scroll_down(void); // Zatim - mozna bude casem potreba
//PRIVATE void scroll_on(void);   // zruseno
//PRIVATE void scroll_off(void);  // zruseno

PRIVATE void flush(struct tty_struct*);

#endif // ifndef TTY_H
