/*=========================================================================

   Program: EXE2TUOX.C - konverzni utilita pro prevedeni EXE souboru
			 z DOSu do TUOXu
   Autor: Zdenek SAWA, l93008, upravil P.G.
   E-mail: Zdenek.Sawa.fei.@vsb.cz
   Do predmetu: Operacni systemy                   Semestr: letni 1997

==========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DEFA_EXE_EXT  "EXE"   /* default pripona pro program v DOSu */
#define DEFA_TUOX_EXT ""      /* default pripona pro program v TUOXu */

#define TUOX_MAGIC      0x12345678L  /* magic number v hlavicce */
				     /* programu v TUOXu        */
#define STACK_DEFAULT   0x1000    /* default velikost stack + gap */

#define PATH_LEN        0x0100    /* max. delka nazvu souboru */

#define DOS_HEADER_LEN    0x1C    /* delka hlavicky - DOS  */
#define TUOX_HEADER_LEN   0x18    /* delka hlavicky - TUOX (!!!puvodne 32) */
#define DOS_INFO          0x0E    /* delka info bloku - DOS */

#define BUFFER_LEN     0x4000     /* velikost bufferu */

#define MAX_SEGLEN    0xFFFFL     /* max. velikost segmentu */

#define DH_MZ      0x00      /* polozky hlavicky EXE souboru */
#define DH_PartPag 0x02
#define DH_PageCnt 0x04
#define DH_ReloCnt 0x06
#define DH_HdrSize 0x08
#define DH_ExeCSIP 0x14

#define DI_Tuox    0x04      /* polozky info bloku */
#define DI_DGroup  0x08
#define DI_Bss	   0x0A
#define DI_BssEnd  0x0C

#define TH_Magic   0x00     /* polozky hlavicky v TUOXu */
#define TH_Res1    0x04
#define TH_TextLen 0x08	    /* kod */
#define TH_DataLen 0x0C     /* inicializovana data */
#define TH_BssLen  0x10 /* neinicializovana data (v DGROUP za inicializovanymi) */
#define TH_Total   0x14


typedef unsigned char Byte;     /* definice ruznych datovych typu */
typedef unsigned int  Word;
typedef unsigned long DWord;

typedef Byte *PByte;
typedef Word *PWord;
typedef DWord *PDWord;

typedef enum { False, True }  Bool;

typedef enum {           /* typy chyb */
	ErrOK,
	ErrOpenFIn,
	ErrOpenFOut,
	ErrSeekF,
	ErrReadF,
	ErrWriteF,
	ErrNotDosExe,
	ErrNotTuox,
	ErrFarPointers,
	ErrBadInfo,
	ErrCode64kB,
	ErrData64kB
  } TError;


Byte Buffer[BUFFER_LEN];        /* buffer pro cteni ze souboru */

FILE *FInput, *FOutput;        /* vstupni a vystupni soubor */
char InFName[PATH_LEN], OutFName[PATH_LEN];
  /* nazvy vstupniho a vystupniho souboru */

DWord CodeStart;     /* zacatek code segmentu */
DWord DataStart;     /* zacatek data segmentu */
DWord CodeLen;       /* delka code segmentu */
DWord DataLen;       /* delka data segmentu */
DWord BssLen;        /* delka bss segmentu  */
DWord StackLen = STACK_DEFAULT;  /* delka stack + gap (nasobek paragrafu) */

DWord HdrSize;  /* velikost headeru v DOS EXE */


/* zaokrouhleni delky na paragrafy nahoru  */
DWord roundUp (DWord size) {
 if (size%16 == 0) return size;
 return ((size+16)/16) * 16;
}

void CloseFiles(void);  /* zavreni otevrenych souboru */


/* nacteni Word z i-te pozice v buferu */
Word GetWord(Word i)
{
  return (Word)(Buffer[i]) | ((Word)(Buffer[i+1]) << 8);
}


/* nacteni DWordu z i-te pozice v buferu */
DWord GetDWord(Word i)
{
  return ((DWord)(Buffer[i]) | ((DWord)(Buffer[i+1]) << 8) |
    ((DWord)(Buffer[i+2]) << 16) | ((DWord)(Buffer[i+3]) << 24));
}


/* ulozeni DWordu do bufferu na i-tou pozici */
void SetDWord(Word i, DWord d)
{
  Buffer[i++] = (Byte)(d & 0xff);
  Buffer[i++] = (Byte)((d >> 8 ) & 0xff);
  Buffer[i++] = (Byte)((d >> 16) & 0xff);
  Buffer[i]   = (Byte)((d >> 24) & 0xff);
}


/* vypis chyboveho hlaseni a ukonceni programu */
void Error(TError err)
{
  char *mess;
  mess = "";
  switch (err) {
    case ErrOpenFIn:
      mess = "Can\'t open input file.";
      break;
    case ErrOpenFOut:
      mess = "Can\'t open output file.";
      break;
    case ErrSeekF:
      mess = "Can\'t seek in file.";
      break;
    case ErrReadF:
      mess = "Can't read input file.";
      break;
    case ErrWriteF:
      mess = "Can't write to output file.";
      break;
    case ErrNotDosExe:
      mess = "Input file is not a DOS EXE-file.";
      break;
    case ErrNotTuox:
      mess = "Input file is not exportable to TUOX.";
      break;
    case ErrFarPointers:
      mess = "Program uses far or huge pointers or functions.";
      break;
    case ErrBadInfo:
      mess = "Bad TUOX information block.";
      break;
    case ErrCode64kB:
      mess = "Text segment exceeds 64kB.";
      break;
    case ErrData64kB:
      mess = "Data + stack segment exceed 64kB.";
      break;
 }
  printf("Error: %s\n\n", mess);
  CloseFiles();
  exit(1);
}


/* zobrazeni zjistenych hodnot */
void ShowValues(void)
{
  printf("                    Segment   Start(image offset)  Lenght\n");
  printf("                   --------------------------------------\n");
  printf("                    CODE       %8ld            %5ld\n",
     CodeStart-HdrSize+TUOX_HEADER_LEN, roundUp(CodeLen));
  printf("                    DATA       %8ld            %5ld\n",
     DataStart-HdrSize+TUOX_HEADER_LEN, roundUp(DataLen));
  printf("                    BSS        %8ld            %5ld\n",
     DataStart-HdrSize+TUOX_HEADER_LEN + roundUp(DataLen), roundUp(BssLen));
  printf("                    STACK + GAP    ----            %5ld\n",
     StackLen);
}


/* otevreni souboru */
void OpenFiles(char *in, char *out)
{
  FInput = fopen(in, "rb");
  if (!FInput) Error(ErrOpenFIn);
  FOutput = fopen(out, "wb");
  if (!FOutput) Error(ErrOpenFOut);
}


/* zavreni souboru */
void CloseFiles(void)
{
  if (FInput)   fclose(FInput);
  if (FOutput)  fclose(FOutput);
}


/* zjisteni delky souboru */
long GetFileLen(void)
{
  long FileLen;
  if (fseek(FInput, 0L, SEEK_END))  Error(ErrSeekF);
  if ((FileLen = ftell(FInput)) < 0L)  Error(ErrSeekF);
  return FileLen;
}


/* nacteni bloku z pozice pos v souboru, delky len do buferu */
void ReadBlock(DWord pos, Word len)
{
  if (fseek(FInput, pos, SEEK_SET))  Error(ErrSeekF);
  if (fread(Buffer, 1, len, FInput) != len)  Error(ErrReadF);
}


/* porovnani bytu v buferu od pozice pos s textem text */
Bool TestText(Word pos, char *text)
{
  char *buf;
  buf = (char *)(Buffer + pos);
  while (*text)
    {
      if (*text++ != *buf++)  return False;
    }
  return True;
}


/* nacteni hlavicky a info bloku z EXE souboru a zjisteni dulezitych
   hodnot */
void ReadDosHeader(void)
{
  DWord ImageLen;
/*  long FileLen; */
  ReadBlock(0L, DOS_HEADER_LEN);
  if (!TestText(DH_MZ, "MZ"))  Error(ErrNotDosExe);
  HdrSize = GetWord(DH_HdrSize) * 0x10;
  ImageLen = (GetWord(DH_PageCnt)-1) * 0x200 + GetWord(DH_PartPag) - HdrSize;
/* Upravil P.G. - ale ono by to stejne mozna melo byt alignovane na paragrafy */

  if (GetWord(DH_ReloCnt) != 1)  Error(ErrFarPointers);
  if (GetDWord(DH_ExeCSIP) != 0L)  Error(ErrNotTuox);

/* P.G.: Mohou byt pripojeny debug info, po TDSTRIP by zabralo */
/*  FileLen = GetFileLen(); */
/*  if (FileLen < HdrSize + ImageLen)  Error(ErrNotDosExe); */
  ReadBlock(HdrSize, DOS_INFO);
  if (!TestText(DI_Tuox, "TUOX"))  Error(ErrNotTuox);
  CodeStart = HdrSize;
  CodeLen = GetWord(DI_DGroup) * 0x10;
  if (CodeLen > MAX_SEGLEN)  Error(ErrCode64kB);
  DataStart = CodeStart + CodeLen;
  DataLen = GetWord(DI_Bss); /* -1 ? */
  BssLen  = GetWord(DI_BssEnd) - DataLen; /* mozna -1 nebo -2 ? */
  if (DataStart + DataLen - HdrSize - 1 > ImageLen)
   // P.G. : upraveno: ImageLen je uz bez headeru, BssLen se nepocita
     Error(ErrBadInfo);
  if (DataLen + BssLen + StackLen > MAX_SEGLEN)  Error(ErrData64kB);
  ReadBlock(HdrSize, 0);
}


/* prekopirovani segmentu zacinajiciho na adrese start v jednom souboru a
   a delky len do druheho souboru */
void CopySegment(DWord start, DWord len)
{
  unsigned char pad, i;
  DWord length=len;

  if (fseek(FInput, start, SEEK_SET))
    Error(ErrSeekF);
  while (len > 0L)
    {
      Word rlen,l;
      rlen = (len > BUFFER_LEN) ? BUFFER_LEN : (Word)len;
      if ((l=fread(Buffer, 1, rlen, FInput)) != rlen)
	Error(ErrReadF);
      if (fwrite(Buffer, 1, rlen, FOutput) != rlen)
	Error(ErrWriteF);
      len -= rlen;
    }

 /* dopln na paragrafy */
 if (length % 16)
  if (fwrite(Buffer, 1, 16-(length % 16), FOutput) != (16-(length % 16)))
   Error(ErrWriteF);
}


/* zapis hlavicky programu pro TUOX */
void WriteTuoxHeader(void)
{
  SetDWord(TH_Magic, TUOX_MAGIC);
  SetDWord(TH_Res1, 0L);
  SetDWord(TH_TextLen, roundUp(CodeLen));
  SetDWord(TH_DataLen, roundUp(DataLen));
  SetDWord(TH_BssLen, roundUp(BssLen));
  SetDWord(TH_Total, roundUp(CodeLen) + roundUp(DataLen) + roundUp(BssLen) + roundUp(StackLen));
  if (fwrite(Buffer, 1, TUOX_HEADER_LEN, FOutput) != TUOX_HEADER_LEN)
    Error(ErrWriteF);
}


/* prekopirovani segmentu code a data */
void WriteTuoxSegs(void)
{
  CopySegment(CodeStart, CodeLen);
  CopySegment(DataStart, DataLen);
}


/* nalezeni tecky, za niz nasleduje pripona souboru */
char* FindExt(char *p)
{
  char *i;
  i = p;
  while (*i++);
  i--;
  if (p == i--)  return 0;
  do {
    if (*i == '.')  return p;
    if (*i == '\\' || *i == ':') return 0;
  } while (p != i--);
  return 0;
}


/* odstraneni pripony z nazvu souboru */
void RemoveExt(char *p)
{
  char *i;
  i = FindExt(p);
  if (i)  *i = 0;
}


/* nastaveni pripony nazvu souboru */
void SetExtension(char *fname, char *ext)
{
  if (FindExt(fname))  return;
  strcat(fname, ".");
  strcat(fname, ext);
}


/* precteni cisla v ceckove konvenci, vysledek je ulozen na adresu num */
Bool ReadNumber(char *p, DWord *num)
{
  DWord n;
  int radix;
  char x;
  n = 0;
  x = *p++ - '0';
  if (x == 0)
    {
      radix = 8;
      if (*p == 'x' || *p == 'X')  radix = 16;
    }
  else if (x >= 1 && x <= 9)  radix = 10;
  else return False;
  if (radix != 16)
    {
      n = x;
      while ((x = *p++) != 0)
	{
	  x -= '0';
	  if (x >= 0 && x < radix)  n = n * radix + x;
	  else return False;
	}
    }
  else
    {
      p++;
      do {
	x = *p;
	if (x >= '0' && x <= '9')  x -= '0';
	else if (x >= 'A' && x <= 'F')  x -= 'A' - 10;
	else if (x >= 'a' && x <= 'f')  x -= 'a' - 10;
	else return False;
	n = n * radix + x;
      } while (*++p);
    }
  *num = n;
  return True;
}


/* analyza options v prikazove radce */
Bool Option(char *p)
{
  if (*p == 'S' || *p == 's')  return ReadNumber(++p, &StackLen);
  StackLen=roundUp(StackLen);
  return False;
}


/* analyza prikazove radky */
Bool CommandLine(int argc, char**argv)
{
  Bool InFile, OutFile;
  int i;
  i = 0;
  InFile = OutFile = False;
  while (++i < argc)
    {
      char *p;
      p = argv[i];
      if (*p == '-' || *p == '/') {
	  if (!Option(++p)) return False;
      } else {
	  if (!InFile) {
	      strcpy(InFName, p);
	      InFile = True;
	  } else {
	      if (!OutFile) {
		  strcpy(OutFName, p);
		  OutFile = True;
	      } else
		  return False;
	  }
      }
    }
  if (!InFile) return False;
  if (!OutFile)
    {
      strcpy(OutFName, InFName);
      RemoveExt(OutFName);
    }
  SetExtension(InFName, DEFA_EXE_EXT);
  SetExtension(OutFName, DEFA_TUOX_EXT);
  return True;
}


/* napoveda k pouziti programu */
void Help(void)
{
  puts("Usage:\n\n"
       "   EXE2TUOX [-s#] input [ output ]\n\n"
       "      where:\n\n"
       "        input   -  filename of DOS EXE program linked with special"
	  " TUOX\n"
       "                   startup code - C0S.OBJ"
	  " (default extension is .EXE)\n\n"
       "        output  -  filename of TUOX executable file,"
	  " it is the same as input\n"
       "        	   filename without extension by default\n\n"
       "          -s#   -  set stack+gap size in output file ( # stands for"
	  " a number\n"
       "         	   in C-format - decimal, hexadecimal or octal)\n"
  );

}


/* vypis copyrightu */
void Copyright(void)
{
  puts("\nEXE2TUOX Version 0.6 - conversion utility, (c) 1997 Zdenek SAWA,"
   " VSB-TU Ostrava\n");
}

void pokus(void) {
 char buffer[100];
 int i;
 FILE* f=fopen("tx.exe","rb");
 if (f) {
  fseek(f,2544L,SEEK_SET);
  i=fread(buffer,1,64L,f);
  fclose(f);
 }

}

/* hlavni program */
int main(int argc, char **argv)
{
/*
pokus();
exit(-1);
*/

  Copyright();
  if (CommandLine(argc, argv))
    {
      OpenFiles(InFName, OutFName);
      ReadDosHeader();
      ShowValues();
      WriteTuoxHeader();
      WriteTuoxSegs();
      CloseFiles();
    }
  else
    Help();
  return 0;
}

